--[[----------------------------------------------------------------------------

LmCatalogModel.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.utils
local WFCloseSupport = require 'WFCloseSupport'

	-- WF.lightroom.models
local LmOzCatalog = require 'LmOzCatalog'

	-- WF.oz_client.sync_engine
local WFOzCatalogSpec = require 'WFOzCatalogSpec'
local WFOzUser = require 'WFOzUser'


--============================================================================--

-- @mark LmCatalogModel

local LmCatalogModel = {}

--------------------------------------------------------------------------------

-- @active catalog model

--------------------------------------------------------------------------------

function LmCatalogModel.modelForCatalog( serverBase, userId, catalogId )

	local wfOzUser = WFOzUser( serverBase, userId )

	local wfOzCatalogSpec = WFOzCatalogSpec( wfOzUser, catalogId )
	local catalog = LmOzCatalog( wfOzCatalogSpec )

	return {

		value = {

			catalogId = catalogId
			
		},

		models = {
			
			album = function( model, albumId )
				return require 'LmAlbumModel'.modelForOzAlbumId( model._catalog, albumId )
			end,
			
			albumAsset = function( model, albumId, assetId )
				-- TODO [matthewj]: Probably should be going through the album model here
				-- rather than a spaceasset model since the WF apis currently use the album
				-- id and asset id and not a space id
				return require 'LmAlbumAssetModel'.modelForOzAlbumAndAssetId(
															model._catalog, albumId, assetId )
			end,

		},
		
		_catalog = catalog,
		
		
		close = function( self )
		
			if not WFCloseSupport.hasBeenOrWillBeClosed( self._catalog ) then
				self._catalog:close()
			end

			collectgarbage( 'collect' )
			collectgarbage( 'collect' )
			collectgarbage( 'collect' )

		end,
	}

end

--------------------------------------------------------------------------------

return LmCatalogModel
