--[[----------------------------------------------------------------------------

LmOzCatalog.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

Manages the lifetime of the set of objects needed to keep a single Lightroom
user catalog running.

Can be used in either WF model threads or Lightroom's main thread.

Intended to replace LrWFSyncHelper.lua over time.

------------------------------------------------------------------------------]]

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local _ = require 'WFArguments_uuid'
local WFCloseSupport = require 'WFCloseSupport'
local WFObjectProxy = require 'WFObjectProxy'

	-- WF.lightroom.models
local LmCatalogPaths = require 'LmCatalogPaths'

	-- WF.oz_client.catalog
local WFOzCatalog = require 'WFOzCatalog'


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--

-- @mark LmOzCatalog class

local catalogMethods = {

	_wfOzCatalog = false,
		-- (WFOzCatalog) handles the tuple of doc store, index store and oz user
	
	WFCloseSupport.addCloseSupportMethods,

}

--------------------------------------------------------------------------------
--[[!
	\brief Create an LmOzCatalog.
	
	\param ozCatalog (WFOzCatalogSpec) specification for the catalog user and catalog ID
]]--

function catalogMethods:__init( ozCatalogSpec )

	check( 'LmOzCatalog', 'WFOzCatalogSpec' )
	
	local ozUserId = ozCatalogSpec:ozUserId()
	
	local userDocStorePath = LmCatalogPaths.getDocStorePathForUser( ozUserId )

	local wfOzCatalog = WFOzCatalog( ozCatalogSpec, { docStorePath = userDocStorePath } )

	self:setOwned {
		_wfOzCatalog = wfOzCatalog,
	}

	WFCloseSupport.mustClose( self )

end

--------------------------------------------------------------------------------

function catalogMethods:wfOzCatalog()

	return self._wfOzCatalog

end

--------------------------------------------------------------------------------

function catalogMethods:ozCatalogSpec()

	return self._wfOzCatalog:ozCatalogSpec()
	
end

--------------------------------------------------------------------------------

function catalogMethods:ozCatalogId()

	return self._wfOzCatalog:ozCatalogId()
	
end

--------------------------------------------------------------------------------

function catalogMethods:ozUser()

	return self._wfOzCatalog:ozUser()
	
end

--------------------------------------------------------------------------------

function catalogMethods:docStore()

	return self._wfOzCatalog:docStore()

end

--------------------------------------------------------------------------------

function catalogMethods:indexStore()

	return self._wfOzCatalog:indexStore()

end

--------------------------------------------------------------------------------

local LmOzCatalog_class = Rx.Class 'LmOzCatalog'( catalogMethods )

return WFObjectProxy( 'LmOzCatalog', LmOzCatalog_class, nil, true )
