--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local KSReactive = require "KSReactive"

	-- WF.core.utils
local WFDigest = require 'WFDigest'

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'
local WFUnknown = require 'WFUnknown'


	-- Lrmobile_models
local LrmAllPhotosQueryBuilder = require 'LrmAllPhotosQueryBuilder'
local LrmAlbumQueryBuilder = require 'LrmAlbumQueryBuilder'

local WFPlatformInfo = require 'WFPlatformInfo'
--------------------------------------------------------------------------------
local LrmAlbumUtils = require 'LrmAlbumUtils'
local LrmAllPhotosAlbumModel = {}
local Model = {}

--------------------------------------------------------------------------------

function Model:cacheSizeModel()
	
	local theQuery = {	
		sql 		= [[SELECT SUM(size) FROM cacheReferences]],
		calm 		= 3,
		nArgs 		= 0,	
	}

	local cacheSizeB =  self.catalog:indexStore()

		:queryB( theQuery )

		:closingMapB( function( rows )
		
			local cacheSize = WFUnknown.value()
			if rows and rows[ 1 ] and rows[ 1 ][ 1 ] then
				cacheSize = rows[1][1]
			end
			
			return cacheSize
		end )
		
	return {
		valueB = cacheSizeB,
	}
end

--------------------------------------------------------------------------------

function Model:isSharedModel()

	return {
		valueB = LrmAlbumUtils.isAlbumSharedB( self.catalog.indexStore(), self.albumId )
	}
	
end

--------------------------------------------------------------------------------

function Model:unsynchronizedAssets()

	local trickerSql = [[SELECT count( distinct docId ) FROM renditionRevisions]]

	local trickerB = self.catalog.indexStore()
	
		:queryB( {
			sql = trickerSql
		 } )
	
		:closingMapB( function( rows )
			if rows and #rows > 0 then
				return rows[1][1]
			end
			return nil
		end )

	return {
		valueB = trickerB
	}
	
end

--------------------------------------------------------------------------------

local function makeCollectionInfo( name, originalPhotoRows, displayDate, eventName, startDate, endDate, eventNameFirst )

	debug.print(name .. " " .. #originalPhotoRows)
	if not originalPhotoRows or #originalPhotoRows == 0 then
		local photosUnknown = originalPhotoRows == nil or WFUnknown.isUnknown( originalPhotoRows ) or nil
		return { name = name, displayDate = displayDate, photos = { }, eventName = eventName, startDate = startDate, endDate = endDate, assetCount = 0,
				 photosUnknown = photosUnknown }
	end
				
	local nPhotos = #originalPhotoRows
	
	local collectionInfo = {
		name = name,
		photos = {},
		displayDate = displayDate,
		eventName = eventName,
		startDate = startDate,
		endDate = endDate,
		eventNameFirst = eventNameFirst,
		assetCount = nPhotos,
	}
	
	local photos = collectionInfo.photos
	
	local countCollectionInfoFields = LrmAlbumQueryBuilder.kCountCollectionInfoFields
	
	for i = 1, nPhotos do
		
		local originalPhotoRow = originalPhotoRows[ i ]
		
		photos[ i ] = {
			id = originalPhotoRow[ 1 ], -- coreInfo.docId
			croppedWidth = originalPhotoRow[ 2 ],
			croppedHeight = originalPhotoRow[ 3 ],
			captureDate = originalPhotoRow[ 4 ], --captureDate
			fileName = originalPhotoRow[ 5 ],
			modifiedDate = originalPhotoRow[ 6 ], -- modifiedDate
			thumbnailMD5 = originalPhotoRow[ 7 ], -- thumbnailMD5
			importDate = originalPhotoRow[ 8 ], -- importDate
			isPopulated = true,
		}

	end

	return collectionInfo

end
local function globalGridCollectionInfoB( catalog, activeAlbum, ascending, assetTypeFilter, assetFilter, sortCriteria, ratingFilter, ratingComparisonOperator, isSearch, personId )

	local sql, queryParams = LrmAllPhotosQueryBuilder.buildCoreInfoQuery( activeAlbum, ascending,
												assetTypeFilter, assetFilter, sortCriteria,
												ratingFilter, ratingComparisonOperator, isSearch, personId
											)

    local resultB = catalog:indexStore():queryB {
		sql = sql, 
		nArgs = #queryParams, 
		unpack( queryParams )
	}

	return resultB:closingMapB( function( originalPhotoRows )
        
		return makeCollectionInfo(
				'grid', -- collection id for grid mode, not used by UI
				originalPhotoRows,
				true )

	end )
							
end

-- Check to see whether the lists disagree on a field we actually care about

local function photoListsCloseEnough( aList, bList )

	if aList == bList then
		return true
	end

	if aList == nil and bList ~= nil then
		return false
	end
	
	if aList ~= nil and bList == nil then
		return false
	end

	local a_n = #aList
	local b_n = #bList
	
	if a_n ~= b_n then
		return false
	end
	
	for i = 1, a_n do
		local a_item = aList[ i ]
		local b_item = bList[ i ]
		if a_item.id ~= b_item.id then
			return false
		end
		if a_item.flag ~= b_item.flag then
			return false
		end
		
		if a_item.rating ~= b_item.rating then
			return false
		end
		
		if a_item.commentCount ~= b_item.commentCount then
			return false
		end
		
		if a_item.favoriteCount ~= b_item.favoriteCount then
			return false
		end
		
		if a_item.wasDeveloped ~= b_item.wasDeveloped then
			return false
		end
		
		if a_item.croppedWidth ~= b_item.croppedWidth then
			return false
		end
		
		if a_item.croppedHeight ~= b_item.croppedHeight then
			return false
		end
	end
	
	return true

end

local function calculateAssetListHash( photoList )
	
	if not photoList then
		return nil
	end
	
	local n = #photoList

	if n == 0 then
		return nil
	end
	
	local digest = WFDigest.MD5.init()
	
	for i = 1, n do
		digest:update( photoList[ i ].id )
	end
	
	local result = digest:digest()
	return result
end
local function oldAssetListModelB( collectionInfoB )
	
	local prevCollectionInfo = {}
	prevCollectionInfo.photos = {}
	return collectionInfoB:closingMapB( function( collectionInfo )
	
		if WFUnknown.isUnknown( collectionInfo ) then
			return collectionInfo
		end
		
		if type( collectionInfo ) == 'userdata' then
			return collectionInfo, false
		end
		
		local newPhotos = collectionInfo and collectionInfo.photos

		if type( newPhotos ) == 'userdata' then
			return newPhotos, false
		end
		
		if collectionInfo ~= nil then
			local newPhotos = collectionInfo and collectionInfo.photos
		
			if collectionInfo and collectionInfo.photosUnknown then
				collectionInfo = nil
				newPhotos = nil
			end
			
			local prevPhotos = nil
			if prevCollectionInfo and prevCollectionInfo.photos then
				prevPhotos = prevCollectionInfo.photos
			end
			
			local closeEnough = photoListsCloseEnough( newPhotos, prevPhotos )
			
			if collectionInfo then
				collectionInfo.closeEnough = closeEnough
			end
			
			if closeEnough then
				return prevCollectionInfo
			else
				prevCollectionInfo = collectionInfo
				return prevCollectionInfo
			end
		end
	end )
	:closingMapB( function( collectionInfo )
	
		if WFUnknown.isUnknown( collectionInfo ) then
			return collectionInfo
		end
		
		local albumInfo = nil
		local photos = nil
		if collectionInfo ~= nil then
			albumInfo = {}
			albumInfo.photos = collectionInfo.photos
			albumInfo.assetCount = collectionInfo.assetCount
			albumInfo.closeEnough = collectionInfo.closeEnough
			photos = collectionInfo.photos
		end
		return {
			albumInfo = albumInfo,
			assetListHash = calculateAssetListHash( photos )
		}
	end )
end

function Model:assetListModel( sortCriteria, ascending, assetFilter, assetTypeFilter, ratingFilter, ratingComparisonOperator, isSearch, personId )

	local activeFilter = {}
	
	for i = 1, #assetFilter do
		local filter = assetFilter[ i ]
		activeFilter[ filter ] = true
	end
	
	if assetTypeFilter then
		for i = 1, #assetTypeFilter do
			local assetType = assetTypeFilter[ i ]
			activeFilter[ assetType ] = true
		end
	end
	
	self.catalog:docStore():applyOverlayToDocumentP( {
		docType	= 'gen-album',
		docId 	= self.albumId,
		overlay	= 
		{
			_localOnly = 
			{
				filter = 
				{
					_replace 					= true,
					image						= activeFilter.image,
					video						= activeFilter.video,
					pick 						= activeFilter.pick,
					reject 						= activeFilter.reject,
					unflagged					= activeFilter.unflagged,
					favorite 					= activeFilter.favorite,
					comment 					= activeFilter.comment,
					rating 						= ratingFilter,
					ratingComparisonOperator	= ratingComparisonOperator,
					sortCriteria 				= sortCriteria,
					ascending 					= ascending
				},
			},
		}
	} ):runUntilDone()
	
    local resultB
    
    if WFPlatformInfo.isAndroid then

        resultB = oldAssetListModelB( globalGridCollectionInfoB( self.catalog,
					self.albumId, ascending, assetFilter, assetTypeFilter, sortCriteria,
					ratingFilter, ratingComparisonOperator, isSearch, personId )
				)
    else
	local sql, queryParams	= LrmAllPhotosQueryBuilder.buildCoreInfoQuery (self.albumId, ascending, assetFilter, assetTypeFilter, sortCriteria, ratingFilter, ratingComparisonOperator, isSearch)
		
	local encoder 			= 'LrmOzAssetCoreInfo.encodeSql'
											
        resultB 			= self.catalog:indexStore():queryB
	{ 
		sql 	= sql, 
		nArgs 	= #queryParams,
		encoder	= encoder,
		calm 	= .125,
		unpack( queryParams )
	}
    end

	return 
	{
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

local function makeFlagStatus( self, rows )

	local flags = {
		unflagged = 0,
		pick = 0,
		reject = 0,
	}
	
	for i=1, #rows do
	
		local row = rows[i]
	
		if row[1] == 'unflagged' then	
		
			flags.unflagged = row[2]
		
	  	elseif row[1] == 'pick' then
		
			flags.pick = row[2]
			
	  	elseif row[1] == 'reject' then	
			
			flags.reject = row[2]
			
	  	end
		
	end
	
	return flags
	
end

--------------------------------------------------------------------------------

local function flagsB( self, catalog, albumId )
	local sql

	if WFPlatformInfo.isAndroid then
		sql = [[SELECT flag, COUNT(docId) FROM flags GROUP BY flag]]
	else
		sql = [[SELECT flag, COUNT(docId) FROM flags WHERE (userId == ? OR userId == "") GROUP BY flag]]
	end

	return catalog:ozUserIdP()
		:untilResolvedB( 'data', WFUnknown.value() )
		:closingMapSwitchB( function( userId )
		
			return catalog:indexStore()
	
				:queryB (sql, userId)
			
				:closingMapB ( 
					function( rows )
						
						if rows == nil or WFUnknown.isUnknown( rows ) then
							return WFUnknown.value()
						end
			
						return makeFlagStatus( self, rows )
		
					end )
			
		end )
		
	
end

--------------------------------------------------------------------------------

function Model:assetListWithSearch()
		
	local sql, queryParams	= LrmAllPhotosQueryBuilder.buildCoreInfoQueryWithSearch ()
		
	local encoder 			= 'LrmOzAssetCoreInfo.encodeSql'
											
    local resultB 			= self.catalog:indexStore():queryB 
	{ 
		sql 	= sql, 
		nArgs 	= #queryParams,
		encoder	= encoder,
		unpack( queryParams )
	}

	return 
	{
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function Model:flagStatusCountModel()

	return {
		valueB = flagsB( self, self.catalog, self.albumId )
	}
end

--------------------------------------------------------------------------------
function Model:coverImageModel()

	local sql = [[
		SELECT coreInfo.docId 
		FROM coreInfo 
		ORDER BY captureDate DESC 
		LIMIT 1]]

		
	local resultB = self.catalog:indexStore()

	:queryB( { 
		sql = sql,
		calm = 30
	} )
		
	:closingMapB( function ( rows )

		if WFUnknown.isUnknown( rows ) then
			return rows
		end

		local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]
		
		return coverAssetId
		
	end )

	return {
		valueB = resultB
	}
end

--------------------------------------------------------------------------------
function Model:alternativeCoverAssetId()

	local sql = [[SELECT coreInfo.docId FROM coreInfo WHERE coreInfo.docId NOT IN (SELECT assetId FROM ozUploadStatus WHERE docType == 'asset') ORDER BY captureDate DESC LIMIT 1]]
					
	local coverImageAssetIdB = self.catalog:indexStore()
	
		:queryB ( sql )

		:closingMapB( 
			function ( rows )
		
				local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]

				return coverAssetId
			
			end )
			
		:closingFilterB( 
			function( val ) 
				return val ~= nil 
			end, 
			WFUnknown.value() )

	return {
		valueB = coverImageAssetIdB,
	}
	
end

--------------------------------------------------------------------------------

function LrmAllPhotosAlbumModel.makeModel( catalog, albumId )
	
	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	catalog:docStore():docValueOnceP( 'gen-album', albumId )

	:whenResolvedDo( function( status, docValue )
	
		if status == 'error' then
			status = 'hallo'
		end
		
		if status == 'data' or #albumId ~=32 then
		
			local filter = KSTableUtils.derefOpt( docValue, "_localOnly", "filter" )
			
			if filter then
				
				local deref = KSTableUtils.derefOpt
				
				local activeFilter = {
					image = deref( filter, "image" ),
					video = deref( filter, "video" ),
					pick = deref( filter, "pick" ),
					reject = deref( filter, "reject" ),
					unflagged = deref( filter, "unflagged" ),
					favorite = deref( filter, "favorite" ),
					comment = deref( filter, "comment" ),
					rating = deref( filter, "rating" ),
					ratingComparisonOperator = deref( filter, "ratingComparisonOperator" ),
					sortCriteria = deref( filter, "sortCriteria" ),
					ascending = deref( filter, "ascending" ),
				}
				
				resultB:set( activeFilter )
			else
				resultB:set( nil )
			end
		end
		
	end ):runUntilDone()
	
	return {
		catalog = catalog,
		albumId = albumId,
		valueB = resultB,
		models = Model,
		
		events = {
		}
	}
end

--------------------------------------------------------------------------------

return LrmAllPhotosAlbumModel
