--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.rxlua
local _ = require 'WFRxPromiseBridge'

	-- WF.core.reactive
local KSReactive = require 'KSReactive'
local KSPromises = require 'KSPromises'

	-- WF.lrmobile_models
local LrmAssetUtils = require 'LrmAssetUtils'

-- wichitafoundation.core.logging
local WFLogger = require 'WFLogger'

--------------------------------------------------------------------------------

local exportModelLog = WFLogger 'LrmExportModel'
:setLevel( 'debug' )

--==============================================================================

local LrmExportModel = {}

--------------------------------------------------------------------------------

local Model = {}

--------------------------------------------------------------------------------

function LrmExportModel.makeModel( catalog, assetIds )

	assert( catalog )
	assert( assetIds )

	local result = {}
	local resultB = KSReactive.variableB( nil )

	local function getDocValueP(assetId)
		catalog:photoInfoB( assetId )
	end

	local function getExportInfoForAssetP( assetId)
		return catalog:docStore():docValueOnceP( 'asset', assetId )
			:mapP ( function (_, docValue)

				result [assetId] = {
					localUrl = docValue and docValue._localOnly and docValue._localOnly.localUrl,
					masterPath = LrmAssetUtils.getMasterPath( docValue ),
					hasMaster = LrmAssetUtils.hasMaster( docValue ),
					hasMasterLocally = LrmAssetUtils.hasMasterLocally( docValue ),
					hasPreviewLocally = LrmAssetUtils.hasPreviewLocally( docValue ),
					importSHA = docValue and docValue.importSource and docValue.importSource.sha256,
					hasProxyLocally = LrmAssetUtils.hasProxyLocally( docValue ),
					proxyPath = LrmAssetUtils.getProxyPath( docValue ),
					fileSize = docValue and docValue.importSource and docValue.importSource.fileSize
				}
		end)
	end

	KSPromises.allP( getExportInfoForAssetP, assetIds )
		:whenResolvedDo( function ()
			resultB:set(result)
		end

	)

	:attachTo(resultB)

	return {
		assetIds = assetIds,
		catalog = catalog,
		valueB = resultB,
		models = Model,
	}

end

return LrmExportModel
