--[[----------------------------------------------------------------------------

LrmKeywordsIndexDefinitions.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local KSTableUtils = require 'KSTableUtils'
local WFPlatformInfo = require 'WFPlatformInfo'
--------------------------------------------------------------------------------

local check = assert( WFArguments.check )
local derefOpt = assert( KSTableUtils.derefOpt )

--============================================================================--

-- @mark LrmKeywordsIndexDefinitions namespace

local LrmKeywordsIndexDefinitions = {}

--------------------------------------------------------------------------------

function LrmKeywordsIndexDefinitions.generateIndexSchema( indexStoreSchema )

	check( 'WFRxIndexStoreSchema' )

	-- table keywords

	indexStoreSchema:addTable  {

		sql = [[CREATE TABLE keywords( keyword TEXT )]],

		map = function( docValue, emit )

			if docValue and docValue.type == 'asset'
					and ( docValue.subtype == 'image' or docValue.subtype == 'video' )
					and not docValue._deleted then

				local id = docValue._id or docValue._docId

				if id and docValue.xmp and docValue.xmp.dc and docValue.xmp.dc.subject then

					for k, v in pairs(docValue.xmp.dc.subject) do
						emit( k )
					end

				end

			end

		end,

	}

	indexStoreSchema:addIndex [[CREATE INDEX keywords_keyword ON keywords( keyword )]]

	--------------------------------------------------------------------------------

	if not WFPlatformInfo.isAndroid then

		indexStoreSchema:addTable {

			sql = [[CREATE TABLE persons( name, hidden, primary_person, thumbnail_url )]],

			map = function( docValue, emit )

				if docValue._deleted then
					return
				end

				if docValue.type == 'asset' and docValue.subtype == 'person' then

					local thumbnail_href = derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/rendition_type/thumbnail2x', 'href' )
					local name = derefOpt( docValue, 'name' ) or ''
					local hidden = derefOpt( docValue, 'hidden' ) or false
					local primary = derefOpt( docValue, 'primary' ) or ''

					emit( name, hidden, primary, thumbnail_href )

				end

			end,
		}

	end

end

--------------------------------------------------------------------------------

return LrmKeywordsIndexDefinitions
