--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local KSReactive = require 'KSReactive'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'
local WFPlatformInfo = require 'WFPlatformInfo'

	-- WF.lrmobile
local LrmAlbumAssetUtils = require 'LrmAlbumAssetUtils'
local LrmCatAsset = require 'LrmCatAsset'
local LrmAlbumUtils = require 'LrmAlbumUtils'
local LrmCatAlbumAsset = require 'LrmCatAlbumAsset'
local LrmCatAlbum = require 'LrmCatAlbum'

--==============================================================================

local LrmLibraryAlbumAssetModelImp = {}

--==============================================================================

function LrmLibraryAlbumAssetModelImp:removeAssetsFromAlbum( albumId, assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatAlbumAsset.removeAssetsFromAlbumP( self.catalog, albumId, assetIds )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = data } )
			end
			
		end )
		:runUntilDone()

	return {
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:deleteAssets( assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )

	LrmCatAsset.deleteAssetsP( self.catalog, assetIds )
		:whenResolvedDo( function( status, data )

			if status == 'data' then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = data } )
			end
		
		end )
		:runUntilDone()

	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:addAssetsToAlbum( albumId, assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmAlbumAssetUtils.addAssetsToAlbumWithOrderP( self.catalog, albumId, assetIds )
	:whenResolvedDo( function( status, data )
		if status == 'data' then
			if type( data ) == 'boolean' then
				if data then
					resultB:set( { state = true } )
				else
					resultB:set( { state = false, error = data } )
				end
			end
		else
			resultB:set( { state = false, error = data } )
		end
	end )
	:runUntilDone()

	return {
		valueB = resultB,
	}
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:moveAssetsToAlbum( fromAlbumId, toAlbumId, assetIds )
	
	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	local catalog = self.catalog
	
	LrmCatAlbumAsset.removeAssetsFromAlbumP( self.catalog, fromAlbumId, assetIds )
	:mapDataP( function()
		return LrmAlbumAssetUtils.addAssetsToAlbumWithOrderP( catalog, toAlbumId, assetIds )
	end )
	:whenResolvedDo( function( status, data )
		if status == 'data' then
			if data == true then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = "Assets don't exist in album" } )
			end
		else
			resultB:set( { state = false, error = data } )
		end
	end ):runUntilDone()

	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:makeAlbumFilterCountModel( albumId, userId, isSearching )

	local docType = LrmAlbumUtils.getDocTypeForAlbumId(albumId)
	local resultB

	if isSearching then
		
		local sqlQuery = [[SELECT filterType, filterCount FROM TEMP_searchFilterCount]]
		resultB = self.catalog:indexStore():queryB( sqlQuery )
		
	else
		
		local sqlQuery = [[SELECT flag, COUNT(flags.docId) FROM flags ]]
		local params = {}
		
		if docType == 'gen-album' then
			if WFPlatformInfo.isAndroid then
				sqlQuery = sqlQuery .. [[GROUP BY flag]]
			else
				sqlQuery = sqlQuery .. [[WHERE (userId == ? OR userId == "") GROUP BY flag]]
				table.insert( params, userId )
			end
		else
			if WFPlatformInfo.isAndroid then
				sqlQuery = sqlQuery .. [[INNER JOIN album_asset ON album_asset.assetId == flags.docId
									WHERE album_asset.albumId == ? GROUP BY flags.flag]]
			else
				sqlQuery = sqlQuery .. [[INNER JOIN album_asset ON album_asset.assetId == flags.docId
									WHERE album_asset.albumId == ? AND ( flags.userId == ? OR flags.userId == "" ) GROUP BY flags.flag]]
				table.insert( params, userId )
			end

			table.insert( params, albumId )
		end
		
		resultB = self.catalog:indexStore():queryB
		{ 
			sql 	= sqlQuery, 
			calm 	= .125,
			nArgs 	= #params,
			unpack( params )
		}
		
	end
	
	
	local mappedResultB = resultB

		:closingMapB( function( rows )
		
			if rows == nil or WFUnknown.isUnknown( rows ) then
				return WFUnknown.value()
			end
			
			local unflaggedCount = 0
			local pickCount = 0
			local rejectCount = 0
			
			for i=1, #rows do
			
				local row = rows[i]
			
				if row[1] == 'unflagged' then	
					unflaggedCount = row[2]
			  	elseif row[1] == 'pick' then
					pickCount = row[2]
			  	elseif row[1] == 'reject' then	
					rejectCount = row[2]
			  	end
				
			end
			
			return {
				unflaggedCount = unflaggedCount,
				pickCount = pickCount,
				rejectCount = rejectCount
			}
			
		end )
		
	return {
		value = mappedResultB
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:makeAlbumRatingCountModel( albumId, isSearching, rating, compop )

	local docType = LrmAlbumUtils.getDocTypeForAlbumId(albumId)
	local resultB

	if isSearching then
	
		local sqlQuery = [[SELECT filterCount FROM TEMP_searchFilterCount WHERE filterType == ?]]
		resultB = self.catalog:indexStore():queryB( sqlQuery, 'rating' )

	else
	
		local sqlQuery = [[SELECT count(coreInfo.rating) FROM coreInfo ]]
		local params = {}
		
		if docType == 'gen-album' then
		    sqlQuery = sqlQuery .. [[WHERE rating ]] .. compop .. [[ ?]]
		else
		    sqlQuery = sqlQuery .. [[INNER JOIN album_asset ON album_asset.assetId == coreInfo.docId
									WHERE album_asset.albumId == ? AND rating ]] .. compop .. [[ ?]]
			table.insert( params, albumId )
		end
		
		table.insert( params, rating )
		
		resultB = self.catalog:indexStore():queryB
		{ 
			sql 	= sqlQuery, 
			calm 	= .125,
			nArgs 	= #params,
			unpack( params )
		}
		
	end

	local mappedResultB = resultB

		:closingMapB( function( data )
			
			if data and data[ 1 ] and data[ 1 ][ 1 ] then
				return data[ 1 ][ 1 ]
			end
			
			return WFUnknown.value()
			
		end )
		
	return {
		value = mappedResultB
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:makeAlbumAssetTypeCountModel( albumId, isSearching )

	local docType = LrmAlbumUtils.getDocTypeForAlbumId(albumId)
	local resultB
	
	if isSearching then
	
		local sqlQuery = [[SELECT filterType, filterCount FROM TEMP_searchFilterCount]]
		resultB = self.catalog:indexStore():queryB( sqlQuery )
	
	else
	
		local sqlQuery = [[SELECT 
							(case 
								when coreInfo.isVideo = 0 then 'image'
	             				when coreInfo.isVideo = 1 then 'video'
	        				end), count(coreInfo.docId) FROM coreInfo ]]
		local params = {}
		
		if docType == 'gen-album' then
		    sqlQuery = sqlQuery .. [[GROUP BY coreInfo.isVideo]]
		else
		    sqlQuery = sqlQuery .. [[INNER JOIN album_asset ON album_asset.assetId == coreInfo.docId
									WHERE album_asset.albumId == ?
									GROUP BY coreInfo.isVideo]]
			table.insert( params, albumId )
		end
		
		resultB = self.catalog:indexStore():queryB
		{ 
			sql 	= sqlQuery, 
			calm 	= .125,
			nArgs 	= #params,
			unpack( params )
		}
		
	end
	
	local mappedResultB = resultB

		:closingMapB( function( rows )
		
			if rows == nil or WFUnknown.isUnknown( rows ) then
				return WFUnknown.value()
			end
			
			local imageCount = 0
			local videoCount = 0
	
			for i=1, #rows do
				local row = rows[i]
	
				if row[1] == 'image' then	
					imageCount = row[2]
	  			elseif row[1] == 'video' then
					videoCount = row[2]
	  			end
			end
	
			return {
				imageCount = imageCount,
				videoCount = videoCount
			}
			
		end )
		
	return {
		value = mappedResultB
	}
	
end

--------------------------------------------------------------------------------

--[[!
	\brief Changes the sort order of existing assets in an album
	
	\param albumId (string) The unique ID of the album
	\param assetIds (array of strings) An array of unique IDs for the assets
	\param targetAssetId (string) The unique ID of the target asset where the assets should be inserted
	\param location (string) defining if the assets should be inserted before or after the target asset
	
	\return not yet defined
		
		
]]--

function LrmLibraryAlbumAssetModelImp:changeOrderInsertAssets( albumId, assetIds, targetAssetId, location )

	if assetIds and assetIds[ 1 ] then
	
	
		local sqlQuery = [[SELECT assetId FROM album_asset WHERE albumId == ? AND assetId NOT IN ('%s') ORDER BY sortOrder]]
		sqlQuery = string.format( sqlQuery, table.concat( assetIds, "', '" ) )
		
		local resultP = self.catalog:indexStore():queryP( sqlQuery, albumId)
		:mapDataP( function( status, data )
		
			if data and data [1] then
	
				local elements = {}
	
				for i = 1, #data do
				
					local assetId = data[ i ][ 1 ]
				
					if assetId == targetAssetId then
					
						if location == "after" then
							table.insert(elements, targetAssetId)
						end
						
						for j = 1, #assetIds do
							table.insert(elements, assetIds[j])
						end
						
						if location == "before" then
							table.insert(elements, targetAssetId)
						end
						 
					else
						table.insert(elements, assetId)
					end
				end
				
			
				if #elements > 0 then
					
					local touchedIds = assetIds
					table.insert( touchedIds, targetAssetId )
					
					self.catalog:updateAssetOrderingP( albumId, elements, touchedIds ):whenResolvedDo( function()
						
						LrmCatAlbum.touchAlbumP( self.catalog, albumId ):runUntilDone()
						return 'status', true
						
					end ):runUntilDone()
					
				end
				
			end
			
			return 'status', false
			
		end )
		
		return resultP
	
	end
	
	

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:updateAssetOrdering( albumId, assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatAlbumAsset.updateAssetOrderingP( self.catalog, albumId, assetIds, assetIds ):whenResolvedDo( function()
						
						LrmCatAlbum.touchAlbumP( self.catalog, albumId ):runUntilDone()
						return resultB
						
					end ):runUntilDone()
	
	return resultB

end

--------------------------------------------------------------------------------
local function createAlbumClusters(self, albumId)

    local KSStringUtils = require 'KSStringUtils'

    return self.catalog._wfCatalogStream

    :flatMap( function( wfCatalog )

        local bulkPostUrl = wfCatalog:ozCatalogSpec():catalogHref() ..
                '/albums/' .. albumId .. '/clusters'
                .. '?similarity_distance=-0.70'

        bulkPostUrl = bulkPostUrl:gsub( '/v1%.0/', '/v2/' )
        local WFURI = require 'WFURI'
        bulkPostUrl = WFURI.stringByAppendingPathComponents( wfCatalog:ozCatalogSpec():serverBaseUrl(), bulkPostUrl )

        local options = {
            requestHeaders = {
                { header = 'Content-Type', value = 'application/json' },
                { header = 'Accept', value = '*/*' },
            },
            requestMethod = 'POST',
            bypassRateLimiter = true,
        }

        return wfCatalog:ozCatalogSpec():put( bulkPostUrl, '{}', options )

        :map( function( _, headers )

            local location = headers.Location

            assert( type( location ) == 'string' )

            local tokens = KSStringUtils.split(location, "/")

            return {
                clusterId = tokens[#tokens]
            }

        end )
    end)

end

function LrmLibraryAlbumAssetModelImp:createAlbumClusters( albumId)

	local result = createAlbumClusters (self, albumId)
    :tap(function()

        -- restart the sync to initiate a new cycle
        self.catalog:getSyncClient():stopSync()
        self.catalog:getSyncClient():startSync()

    end)
	return {
		value = result
	}
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumAssetModelImp:deleteCluster(clusterId)
	local resultB = self.catalog:docStore():deleteDoc(clusterId)
	return {
		value = resultB
	}
end

--------------------------------------------------------------------------------

return LrmLibraryAlbumAssetModelImp
