--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.lrmobile
local LrmConfig = require 'LrmConfig'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFUnknown = require 'WFUnknown'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--==============================================================================

-- @mark LrmLibraryAlbumStatusModelImp namespace

--------------------------------------------------------------------------------

local LrmLibraryAlbumStatusModelImp = {}

--------------------------------------------------------------------------------

LrmConfig.registerBinaryStrategyCheck()

--------------------------------------------------------------------------------

function LrmLibraryAlbumStatusModelImp:unsynchronizedAssetsForAlbum( albumId, binaryDownloadStrategy )

	check( 'table', 'string', 'binaryStrategy' )
	
	local sql = [[
		SELECT 
			COUNT( distinct r.docId ) 
		FROM 
			renditionRevisions as r 
		INNER JOIN 
			album_asset 
		ON 
			album_asset.assetId = r.docId 
		WHERE 
			album_asset.albumId == ? 
			
			AND
			
			renditionName IN %s
	]]
	
	if binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy', 'master', 'xmp_develop' )" )
	else
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )" )
	end
	
	local queryB = self.catalog:indexStore():queryB( {
		sql = sql,
		albumId
	 } )
	:closingKnownMapB( function( rows )
		
		if rows and #rows > 0 then
			return rows[1][1]
		end
		
		return nil
		
	end )	
	
	return {
		value = queryB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumStatusModelImp:missingCacheSizeForAlbum( albumId, binaryDownloadStrategy )

	check( 'table', 'string', 'binaryStrategy' )
	
	local sql = [[
		SELECT 
			SUM(rr.size) 
		FROM
			renditionRevisions as rr 
		INNER JOIN
			album_asset 
		ON
			album_asset.assetId = rr.docId 
		WHERE
			renditionName IN %s 
			
			AND
			
			album_asset.albumId == ?
	]]
	
	if binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy', 'master', 'xmp_develop' )" )
	else
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )" )
	end
	
	local queryB = self.catalog:indexStore():queryP( { sql = sql, albumId } )
		:mapDataP_new( function( row )
		
			return row and row[ 1 ] and row[ 1 ][ 1 ] or 0
			
		end )
		:untilResolvedB( 'data', WFUnknown.value() )
	
	return {
		value = queryB
	}
	
end

--------------------------------------------------------------------------------

return LrmLibraryAlbumStatusModelImp
