--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local KSReactive = require 'KSReactive'
local KSPromises = require 'KSPromises'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'
local WFCatUtils = require 'WFCatUtils'
local WFTableMerge = require 'WFTableMerge'

	-- WF.core.os
local KSDate = require 'KSDate'

--==============================================================================

local LrmLibraryReviewModelImp = {}

--------------------------------------------------------------------------------

function LrmLibraryReviewModelImp:setAssetFlagStatus( flagStatus, assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	local catalog = self.catalog
	
	local function setFlagStatusP( assetId, userId )
	
		local baseDocValue = WFCatUtils.createBaseOverlayWithUserUpdatedField( assetId )
	
		local flagOverlay = {
			type = 'asset',
			reviews = {
				[ userId ] = {
					flag = flagStatus,
					date = KSDate.timeInIso8601FormatNearestSecond(),
				},
			},
		}
		
		WFTableMerge.applyOverlay( flagOverlay, baseDocValue )
		
		return catalog:docStore():applyOverlayP( flagOverlay )
			
	end
	
	catalog:ozUserIdP()
		:mapDataP_new( function( userId )
		
			return KSPromises.allP( function( assetId )
			
				return setFlagStatusP( assetId, userId )
				
			end, assetIds )
			
		end )
	
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
			
				resultB:set( { state = true } )
				
			else
			
				resultB:set( { state = false, error = data } )
				
			end
			
		end )
		:runUntilDone()
		
	return {
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryReviewModelImp:setAssetRating( rating, assetIds )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	local catalog = self.catalog
	
	local function setRatingP( assetId, userId )
	
		local baseDocValue = WFCatUtils.createBaseOverlayWithUserUpdatedField( assetId )
		
		local ratingOverlay = {
			type = 'asset',
			ratings = {
				[ userId ] = {
					rating = rating,
					date = KSDate.timeInIso8601FormatNearestSecond(),
				},
			},
		}
		
		WFTableMerge.applyOverlay( ratingOverlay, baseDocValue )
		
		return catalog:docStore():applyOverlayP( ratingOverlay )
		
	end
	
	catalog:ozUserIdP()
		:mapDataP_new( function( userId )
	
			return KSPromises.allP( function( assetId )
			
				return setRatingP( assetId, userId )
				
			end, assetIds )
			
		end )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
			
				resultB:set( { state = true } )
				
			else
			
				resultB:set( { state = false, error = data } )
				
			end
			
		end )
		:runUntilDone()
	
	return {
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

return LrmLibraryReviewModelImp
