--
-- Created by IntelliJ IDEA.
-- User: rkaushi
-- Date: 09/09/18
-- Time: 4:20 PM
-- To change this template use File | Settings | File Templates.
--
-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'
local KSStringUtils = require "KSStringUtils"
local WFArguments = require 'WFArguments'

local derefOpt = assert( KSTableUtils.derefOpt )


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

local LrmPeopleIndexDefinitions = {}

function LrmPeopleIndexDefinitions.generateIndexSchema( indexStoreSchema )

    check( 'WFRxIndexStoreSchema' )

    --persons table

    indexStoreSchema:addTable  {

        sql = [[CREATE TABLE persons( name, thumnailLink, hidden, primaryId )]],

        map = function(docValue, emit)

            if docValue.subtype ~= 'person' then
                return
            end

            local hasThumbnail =  derefOpt( docValue, '_serverProvidedMetadata', 'links',
                '/rels/rendition_type/thumbnail2x', 'href' )

            if docValue and ( docValue._deleted or not hasThumbnail ) then
                return
            end

            local name = docValue and docValue.name
            local hidden = docValue and docValue.hidden
            local primary = docValue and docValue.primary

            emit ( name, hasThumbnail, hidden, primary )

        end
    }

    indexStoreSchema:addIndex [[CREATE INDEX person_primaryId ON persons( primaryId )]]

    --person_assets table

    indexStoreSchema:addTable  {

        sql = [[CREATE TABLE faces( assetId, personId, regionId, person_confirmed , x_axis , y_axis , width , height )]],

        useLocalId = true,

        hasOneRowPerDocument = false,

        docValueFilter = function( docValue )
            return ( docValue.subtype == 'image' or docValue.subtype == 'video' ) and docValue.type == 'asset'
        end,

        map = function(docValue, emit)

            if docValue._deleted then
                return
            end

            local regionList = derefOpt( docValue, 'xmp','mwg-rs', 'Regions', 'RegionList' )

            local ppl = {}
            for regionId, region in pairs( regionList or {} ) do

                if region.Type == "Face" then

                    local area = derefOpt( region, 'Area' )

                    local people = derefOpt( region, 'Extensions', 'adobe-fi', 'People')


                    for personId, person in pairs( people or {} ) do

                        if (not ppl[personId] and not person.Rejected) then
                            ppl[personId] = true
                            emit( docValue._id, personId, regionId, person.Confirmed and true or false ,area.x, area.y, area.w, area.h )
                        end

                    end
                end
            end

        end
    }

    indexStoreSchema:addIndex [[CREATE INDEX faces_personId ON faces( personId )]]
    indexStoreSchema:addIndex [[CREATE INDEX faces_assetId ON faces( assetId )]]

    --similar_persons table


    indexStoreSchema:addTable  {

        sql = [[CREATE TABLE similar_person( personId, similarPersonId, confidence )]],
        docValueFilter = function( docValue )
            return docValue.subtype == 'person' and docValue.type == 'asset'
        end,
        map = function( docValue, emit )

        if docValue.type == 'asset' and docValue.subtype == 'person' and not docValue._deleted then

            local personId = derefOpt( docValue, '_id' )

            local rejectedMap = derefOpt( docValue, 'rejected' ) or  {}

            local links = derefOpt( docValue, '_serverProvidedMetadata', 'links' )

            local start = "/rels/people/similar"

            local len = string.len( start )

            for k,v in pairs( links or {} ) do

                if string.sub( k, 1, len ) == start  then

                    local similarPersonId = v.id

                    if not rejectedMap[similarPersonId] then

                        local confidence = v.confidence

                        if personId and similarPersonId and confidence and personId ~= similarPersonId then
                            emit( personId, similarPersonId, confidence )
                        end
                    end
                end
            end
        end

     end,
    }




end

return LrmPeopleIndexDefinitions
