--[[----------------------------------------------------------------------------

LrmSpaceIndexDefinitions.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'
local WFArguments = require 'WFArguments'

--------------------------------------------------------------------------------

local derefOpt = assert( KSTableUtils.derefOpt )

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--============================================================================--

-- @mark LrmSpaceIndexDefinitions namespace

local LrmSpaceIndexDefinitions = {}

--------------------------------------------------------------------------------

function LrmSpaceIndexDefinitions.generateIndexSchema( indexStoreSchema )

	check( 'WFRxIndexStoreSchema' )

	-- table space

	indexStoreSchema:addTable {

		sql = [[CREATE TABLE space( created, updateTimeStamp, private, download )]],

		hasOneRowPerDocument = true,

		map = function( docValue, emit )

			if docValue and not docValue._deleted
			  and docValue.type == 'space' and docValue.subtype == 'shared'
			then

				local localOnly = docValue._localOnly or {}
				
				local isPrivate = docValue.private
				local canDownload = docValue.download
				local updateTimeStamp = docValue.updated
				local created = docValue.created or localOnly.created or
								derefOpt( docValue, '_serverProvidedMetadata', 'created' )
								
				emit( created, updateTimeStamp, isPrivate and 1 or 0, canDownload and 1 or 0 )
					
			end
								
		end,

	}

	-- table space_album

	indexStoreSchema:addTable {

		sql = [[CREATE TABLE space_album( spaceId, albumId, updateTimeStamp )]],

		map = function( docValue, emit )

			if docValue and not docValue._deleted
			  and docValue.type == 'space_album'
			then

				local albumId = docValue.album and docValue.album.id
				local spaceId = docValue.space and docValue.space.id
				local updateTimeStamp = docValue.updated
				
				if spaceId and albumId then
					
					emit( spaceId, albumId, updateTimeStamp )
					
				end
					
			end
						
		end,

	}

	indexStoreSchema:addIndex [[CREATE INDEX space_album_index ON space_album( spaceId, albumId )]]
	indexStoreSchema:addIndex [[CREATE INDEX space_album_album ON space_album( albumId )]]
	
	-- table spacefavourite
	
	indexStoreSchema:addTable {

		sql = [[CREATE TABLE spacefavorite( assetId, spaceId, created, author_first_name, author_last_name, author_full_name, author_id )]],

		map = function( docValue, emit )

			local docType = docValue.type
			if docValue and not docValue._deleted and docType == 'favorite' then
				
				local localOnly = docValue._localOnly or {}
				local spaceId = ( docValue.space and docValue.space.id ) or ( localOnly.spaceId )
				local assetId = ( docValue.asset and docValue.asset.id ) or ( localOnly.assetId )
				local created = docValue.created or localOnly.created or derefOpt( docValue, '_serverProvidedMetadata', 'created' )
				
					-- check _localOnly.created, to allow indexing of comments that we've created locally
					-- but haven't yet gotten synced back from Oz

				local authorInfo = docValue.author or 
					( docValue._serverProvidedMetadata and docValue._serverProvidedMetadata.author )
					or localOnly.author
					
				local author_first_name, author_last_name, author_full_name, author_id
				
				if authorInfo then
				
					author_first_name, author_last_name, author_full_name, author_id =
						authorInfo.first_name,
						authorInfo.last_name,
						authorInfo.full_name,
						authorInfo.id
				
				end
				
				if assetId and created then
					
					emit( assetId, spaceId, created, author_first_name, author_last_name, author_full_name, author_id )
					
				end
					
			end
						
		end,

	}

	-- indexStoreSchema:addIndex [[CREATE INDEX spacefavorite_asset_space ON spacefavorite( assetId, spaceId )]]

	indexStoreSchema:addIndex [[CREATE INDEX spacefavorite_space ON spacefavorite( spaceId )]]
	
	
end

--------------------------------------------------------------------------------

return LrmSpaceIndexDefinitions
