--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFCloseSupport = require 'WFCloseSupport'

	-- WF.core.lua
local WFWorkerThread = require 'WFWorkerThread'
local WFPlatformInfo = require 'WFPlatformInfo'

	-- WF.core.table
local WFTableArchiving = require 'WFTableArchiving'

	-- WF.core.rxlua
local WFRxRunUntilDone = require 'WFRxRunUntilDone'

local LrmConfig = require "LrmConfig"

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

local runUntilDone = WFRxRunUntilDone

LrmConfig.registerBinaryStrategyCheck()

--==============================================================================

-- @mark LrmSyncClient namespace

-- Manages the reference to Sync MessageProcessor where our Sync infrastructure
-- is running on. See LrmSyncClientImp for the actual implementation

--------------------------------------------------------------------------------

local syncClientMethods = {

	_syncClientImp = false,
		-- (WFWorkerThread) Reference to the SyncClient WorkerThread instance
	
	WFCloseSupport.addCloseSupportMethods
	
}

--------------------------------------------------------------------------------

function syncClientMethods:__init( wfCatalog, optIndexSpecScript, optSyncClientImpName )
	
	check( 'LrmSyncClient', 'WFCatalog', '+string|nil', '+string|nil' )
	
	optIndexSpecScript = optIndexSpecScript or 'WFCatIndexDefinitions.lua'
	optSyncClientImpName = optSyncClientImpName or 'LrmSyncClientImp'
	
	local options = {
		syncClientImpName = optSyncClientImpName,
		serverBase = wfCatalog:ozUser():serverBaseUrl(),
		userId = wfCatalog:ozUserId(),
		userInfoCacheFolder = wfCatalog:ozUser():userInfoCacheFolder(),
		catalogId = wfCatalog:ozCatalogId(),
		indexSpecScript = optIndexSpecScript,
		rootCatalogPath = wfCatalog:fileSpec():rootPath(),
		docStorePath = wfCatalog:fileSpec():docStorePath(),
	}
	
	local optionsString = WFTableArchiving.tableToString( options )
	
	self:setOwned {
		_syncClientImp = WFWorkerThread( 'LrmSyncClientMessageProcessor.lua', optionsString )
	}
	
	WFCloseSupport.mustClose( self )
	
end

--------------------------------------------------------------------------------

function syncClientMethods:getDeleteFeedCallback( )

	check( 'LrmSyncClient' )

	return self._syncClientImp:request( 'getDeleteFeedCallback' )

end

function syncClientMethods:startSync( binaryStrategy )

    if WFPlatformInfo.isAndroid then
        check( 'LrmSyncClient' )
    else
        check( 'LrmSyncClient' , 'binaryStrategy' )
    end
	
	return self._syncClientImp:request( 'startSync', binaryStrategy )
		:call( runUntilDone )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:startAccountsSync( binaryStrategy )

	check( 'LrmSyncClient' , 'binaryStrategy' )

	return self._syncClientImp:request( 'startAccountsSync', binaryStrategy )
	:call( runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:stopSync()

	check( 'LrmSyncClient' )
	
	return self._syncClientImp:request( 'stopSync' )
		:call( runUntilDone )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:stopAccountsSync()

	check( 'LrmSyncClient' )

	return self._syncClientImp:request( 'stopAccountsSync' )
		:call( runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:startContinuousPurge( binaryStrategy )

	check( 'LrmSyncClient' , 'binaryStrategy' )
	
	return self._syncClientImp:request( 'startContinuousPurge', binaryStrategy )
		:call( runUntilDone )
		
end


--------------------------------------------------------------------------------

function syncClientMethods:purgeImmediately( binaryStrategy, noPurgeLimit )

	check( 'LrmSyncClient' , '?binaryStrategy', '?boolean' )
	
	return self._syncClientImp:request( 'purgeImmediately', binaryStrategy, noPurgeLimit )
		:call( runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:purgeForAlbum( albumId, binaryStrategy, noPurgeLimit )

	check( 'LrmSyncClient' , 'uuid', '?binaryStrategy', '?boolean' )
	
	return self._syncClientImp:request( 'purgeForAlbum', albumId, binaryStrategy, noPurgeLimit )
		:call( runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:isPushingMetadata()

	check( 'LrmSyncClient' )
	
	return self._syncClientImp:request( 'isPushingMetadata' )
	
end

--------------------------------------------------------------------------------

function syncClientMethods:isPullingMetadata()
	
	check( 'LrmSyncClient' )
	
	return self._syncClientImp:request( 'isPullingMetadata' )
	
end

--------------------------------------------------------------------------------

function syncClientMethods:getRequiredUpdateStream()

	check( 'LrmSyncClient' )
	
	return self._syncClientImp:request( 'getRequiredUpdateStream' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:isOzCatalogDeleted()

	check( 'LrmSyncClient' )
	
	return self._syncClientImp:request( 'isOzCatalogDeleted' )
		
end

--------------------------------------------------------------------------------

return Rx.Class( 'LrmSyncClient', syncClientMethods )
