--[[----------------------------------------------------------------------------

OzUserInfoCacheIndexDefinitions.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

DEPRECATION NOTICE: This class is used by Lightroom Mobile, but no new clients
should use it.

------------------------------------------------------------------------------]]

	-- WF.core.logging
local WFLogger = require 'WFLogger'

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'

	-- WF.oz_client.index_store
local WFRxIndexStoreSchema = require 'WFRxIndexStoreSchema'
local WFConflictResolver = require 'WFConflictResolver'


--============================================================================--

local indexStoreSchema = WFRxIndexStoreSchema()

--------------------------------------------------------------------------------

WFConflictResolver.generateIndexSchema( indexStoreSchema )

--------------------------------------------------------------------------------

local logger = WFLogger 'OzUserInfoCacheIndexDefinitions::uploadStatus'
--	:setLevel 'debug'

--------------------------------------------------------------------------------

indexStoreSchema:addTable {

	-- Warning [ arahn 11/19/2012 ]: this table definition must match the one defined by OzCatalogIndexDefinitions
	-- because some queries operate on both indexes, for example processUploadQueueP()'s query
	-- in WFOzSyncCycle.
	sql = [[CREATE TABLE ozUploadStatus( ozUploadStatus, docType, ozUser, captureDate, albumId, assetId, uploadOrder NOT NULL, changeCounter, freemiumUploadEmbargoUntil, uploadWave )]],

	map = function( docValue, emit )

		if docValue._docType == 'receipt' then
			
			local uploadStatus = docValue._ozMetadataUploadStatus or 'notUploaded'
	
			if uploadStatus == 'uploaded' then return end
		
			logger:debugf( "RECEIPT %s >> uploadStatus = %s", docValue, uploadStatus )
			
			emit( uploadStatus, docValue._docType, docValue.userId, nil, nil, nil, 1, nil, nil )
		
		end

	end,

}

indexStoreSchema:addIndex
	[[CREATE INDEX ozUploadStatus_status ON ozUploadStatus(
					ozUploadStatus, ozUser, captureDate, validRev ASC )]]

--------------------------------------------------------------------------------

indexStoreSchema:addTable {

	sql = [[CREATE TABLE catalogs(
				catalogName, ownerId, ownerFullName, url, userCount, sharingState, subtype )]],
	
	map = function( docValue, emit )
	
		if docValue.type == 'catalog' and not docValue._deleted then
			local catalogName = docValue.name
			local relativeUrl = KSTableUtils.derefOpt( docValue, 'links', 'self', 'href' )
			local url = relativeUrl and ( docValue.base .. relativeUrl )
			local sharedWith = docValue.user_count
				
			local ownerId = docValue.owner and docValue.owner._id
			local ownerFullName = docValue.owner and docValue.owner.full_name
				
			local subtype = docValue.subtype
			
			local sharingState = docValue and docValue.sharing and docValue.sharing.state
			
			if catalogName and url then
				emit( docValue.name,
					ownerId,
					ownerFullName,
					url,
					sharedWith,
					sharingState,
					subtype )
			end
		
		end
	
	end,
	
}

--------------------------------------------------------------------------------

indexStoreSchema:addTable {

	sql = [[CREATE TABLE users (
				firstName, lastName, fullName, email )]],
	
	map = function( docValue, emit )
	
		if docValue.type == 'user' and not docValue._deleted then
			local firstName, lastName = docValue.first_name, docValue.last_name
			local fullName = docValue.full_name
			local email = docValue.email
			
			emit( firstName, lastName, fullName, email )
		
		end
	
	end,
	
}

--==============================================================================

return {
	indexStoreSchema = indexStoreSchema,
}
