--[[----------------------------------------------------------------------------

PixelPathProviderRequestTypes.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law. Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

-- wf core
local WFFunctionUtils = require "WFFunctionUtils"
local KSTableUtils = require 'KSTableUtils'

--------------------------------------------------------------------------------

-- these are in priority order

local PixelPathProviderRequestTypesArray = {
	"highest", -- used by active loupe download
	"cache_only",
	"grid_visible",
	"download_soon", 
	"grid_prefetch",
	"loupe_next", -- for prefetch loupe
	"ensure_local_original_exists", -- for downloading originals for export, offline use
	"idle", -- for offscreen grid and idle downloads of proxies/originals
	"lowest", -- lowest priority.
}

--------------------------------------------------------------------------------

local nameToPriorityMap

--------------------------------------------------------------------------------

local function getPriority( name )

	nameToPriorityMap = nameToPriorityMap or 
			KSTableUtils.invert( PixelPathProviderRequestTypesArray )

	return nameToPriorityMap[ name ]

end

--------------------------------------------------------------------------------

local function getName( priority )

	return PixelPathProviderRequestTypesArray[ priority ]

end

--------------------------------------------------------------------------------

local function addToIdentityMap( map, name )

	map[ name ] = name

	return map

end

--------------------------------------------------------------------------------

local _names = WFFunctionUtils.reduce( {}, addToIdentityMap, 
									PixelPathProviderRequestTypesArray )

--------------------------------------------------------------------------------

local function comparePriorities( p1, p2 )

	if type( p1 ) ~= "number" or type( p2 ) ~= "number" then

		return 0

	end

	if p1 > p2 then

		return -1

	elseif p1 < p2 then

		return 1

	end

	return 0

end

--==============================================================================

return {

	getPriority = getPriority,

	getName = getName,

	names = _names,

	priorities = PixelPathProviderRequestTypesArray,

	comparePriorities = comparePriorities,

}
