--[[----------------------------------------------------------------------------

PixelProviderDownloadNegativeModel.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require "Rx"

--------------------------------------------------------------------------------

local function matches( pixStreamIdToMatch )

	return function( actualPixStreamId )

		return pixStreamIdToMatch == actualPixStreamId

	end

end

--------------------------------------------------------------------------------

local function pack( ... )

	return { ... }

end

--------------------------------------------------------------------------------

local function makeGetNegativeStream( pixelProviderProcessorStream, pixStreamId, 
										pixelProviderAPIName, activeAssetIdStream )

	local prevAssetId

	return Rx.Observable.combineLatest( 
				pixelProviderProcessorStream,
				activeAssetIdStream )
			:flatMapLatest( 
				function( pixelProviderProcessor, activeAssetId )

					if not pixelProviderProcessor then

						return Rx.Observable.just( {} )

					end

					if not activeAssetId or 
							( prevAssetId and 
								prevAssetId ~= activeAssetId  ) then

						pixelProviderProcessor
							:runUntilDone( "abandonAllPixStreamRequests",
												pixStreamId )

					end

					prevAssetId = activeAssetId

					if activeAssetId then

						pixelProviderProcessor
							:runUntilDone( pixelProviderAPIName, 
											'asap', pixStreamId,
											activeAssetId )

					end

					return pixelProviderProcessor
								:relayStream( "pixelPathStream" )
								:where( matches( pixStreamId ) )
								:map( pack )

				end )

end

--------------------------------------------------------------------------------

local function makeModel( pixelProviderAPIName )

	return function ( pixelProviderProcessorStream, pixStreamId )

		pixStreamId = string.format( 
						"%s_%s", pixStreamId, pixelProviderAPIName )

		local activeAssetIdStream = Rx.Subject()

		return {

			value = makeGetNegativeStream( 
						pixelProviderProcessorStream, pixStreamId,
						pixelProviderAPIName, activeAssetIdStream ),

			events = {

				updateAssetId = function( _, maybeAssetId )

					activeAssetIdStream:onNext( maybeAssetId )

				end,

				
			},

		}

	end

end

--------------------------------------------------------------------------------

local function makePrefetchNegativePathsStream( pixelProviderProcessorStream, 
												pixStreamId, 
												assetIdsStream )

	local prevAssetIds

	return Rx.Observable.combineLatest( 
				pixelProviderProcessorStream, 
				assetIdsStream )
			:flatMapLatest( 
				function( pixelProviderProcessor, assetIds )

					if not pixelProviderProcessor then

						return Rx.Observable.just( {} )

					end

					pixelProviderProcessor
						:runUntilDone( "prefetchNegativePaths", 
										'soon', pixStreamId,
										prevAssetIds, assetIds )

					prevAssetIds = assetIds 

					return pixelProviderProcessor
								:relayStream( "pixelPathStream" )
								:where( matches( pixStreamId ) )
								:map( pack )

				end )

end

--------------------------------------------------------------------------------

local function ensureLocalNegativePathForPrefetchImageIds( 
				pixelProviderProcessorStream, pixStreamId )

	pixStreamId = string.format( 
					"%s_ensureLocalNegativePathForPrefetchImageIds", 
					pixStreamId )

	local prefetchAssetIdsStream = Rx.Subject()

	return {

		value = makePrefetchNegativePathsStream( 
					pixelProviderProcessorStream, pixStreamId,
					prefetchAssetIdsStream ),

		events = {

			updateAssetIds = function( _, assetIds )

				assetIds = type( assetIds ) == "table" and assetIds or {}

				prefetchAssetIdsStream:onNext( assetIds )

			end,

			
		},

	}

end

--==============================================================================

return {

	getAnyNegative = makeModel( 'getLocalNegativePath' ),
		
	getBestNegative = makeModel( 'getBestPossibleLocalNegativePath' ),

	ensureLocalNegativePathForPrefetchImageIds =
		ensureLocalNegativePathForPrefetchImageIds,


}
