--[[----------------------------------------------------------------------------

WFAsyncDigest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.rxlua
local WFRxNativeObservable = require 'WFRxNativeObservable'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFAsyncDigestImp = require 'WFAsyncDigestImp'

local _ = require 'WFDigest'
	-- define WFDigestAlgorithm for check( "WFDigestAlgorithm" ) below

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--
--[[!
	\class WFAsyncDigest

	\brief This namespace provides asynchronous digest operations.
]]--

-- @mark WFAsyncDigest namespace

local WFAsyncDigest = {}

--------------------------------------------------------------------------------

local function checkDigestAlgorithms( ... )

	local function CheckIsWFDigestAlgorithm( arg )
		check( "WFDigestAlgorithm" )
	end

	local digestCount = select( '#', ... )
	for i = 1, digestCount do
		CheckIsWFDigestAlgorithm( select( i, ... ) )
	end

end

--------------------------------------------------------------------------------

--[[!
	\brief Calculates digests for the specified file on another thread.

	This observable will schedule the digests when the subscription happens.

	\param filePath (string) The absolute path to the file to digest.
	\param ... < strings > The names of the digests to calculate.

	\return Observable that sends a single value that contains the digest values
		as a tuple of strings.

]]--

function WFAsyncDigest.digestFile( filePath, ... )

	check( "+string" )

	checkDigestAlgorithms( ... )

	return WFRxNativeObservable( WFAsyncDigestImp.digestFile, filePath, ... )
		:unpack()

end

--------------------------------------------------------------------------------

function WFAsyncDigest.digestData( data, ... )
	
	checkDigestAlgorithms( ... )
		
	return WFRxNativeObservable( WFAsyncDigestImp.digestData, data, ... )
		:unpack()
		
end


--------------------------------------------------------------------------------

return WFAsyncDigest
