--[[----------------------------------------------------------------------------

WFCatPerson.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2017 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.utils
local KSIdGenerator = require 'KSIdGenerator'
local WFArguments = require 'WFArguments'
local KSTableUtils = require 'KSTableUtils'

--------------------------------------------------------------------------------

local check = WFArguments.check
local derefOpt = KSTableUtils.derefOpt

--==============================================================================

local WFCatPerson = {}

--------------------------------------------------------------------------------

local function createPersonDocument( person )
	
	check( 'table' )
	
	local docId = person.uuid or KSIdGenerator.uuidInLowercaseHex( )
	return {

		_id = docId,
		type = 'asset',
		subtype = 'person',
		name = person.name,
	}
end

--------------------------------------------------------------------------------

--[[!
	\brief 

	\param 
	\param 

	\return 
--]]

function WFCatPerson.addPersons( catalog, persons )
	
	check( 'WFCatalog', 'table' )
	
	local docs = {}
	for i, person in ipairs( persons ) do
		
		docs[ #docs + 1 ] = createPersonDocument( person )
		
	end
	
	return catalog:docStore():bulkSetDocValue( docs )

end

--------------------------------------------------------------------------------

function WFCatPerson.generateIndexSchema( indexStoreSchema )

	check( 'WFRxIndexStoreSchema' )
	
	indexStoreSchema:addTable {

		sql = [[CREATE TABLE person( personId, name )]],

		useLocalId = true,
		hasOneRowPerDocument = true,
		docValueFilter = function( docValue )
			return docValue.subtype == 'person' and docValue.type == 'asset'
		end,
		map = function( docValue, emit )

			if docValue.type == 'asset' and docValue.subtype == 'person' then
				
				local personId = derefOpt( docValue, '_id' )
				
				local name = derefOpt( docValue, 'name' )
				
				emit( personId, name )
				
			end

		end,

	}
	
	indexStoreSchema:addIndex [[CREATE INDEX person_personId_name ON person( personId, name )]]
	
end

--==============================================================================

return WFCatPerson
