--[[----------------------------------------------------------------------------

WFCatSearchAndSortConstants.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
local WFConfiguration = require "WFConfiguration"
------------------------------------------------------------

local WFCatSearchAndSortConstants = {}

----------------------------------------------------------
WFCatSearchAndSortConstants.General = {}
WFCatSearchAndSortConstants.General.NO_CAMERA_INFO = "NoCamera"

------------------------------------------------------------
 WFCatSearchAndSortConstants.OnlineSearchStatus =
	{
		NoOnlineRequest = "NoOnlineRequest",
		InProgress = "InProgress",
		Completed = "Completed",
		CompletedUnexpectedly = "CompletedUnexpectedly"
	}
	
------------------------------------------------------------	
WFCatSearchAndSortConstants.ONLINESEARCH_TEMP_TABLE_NAME = "Temp_Online_Search"
WFCatSearchAndSortConstants.SEARCH_VERSION_V1 = "v1"
WFCatSearchAndSortConstants.SEARCH_VERSION_V2= "v2"

------------------------------------------------------------
local PAGE_SIZE_V1 = 100
local PAGE_SIZE_V2 = 200
local DEFAULT_VERSION = WFCatSearchAndSortConstants.SEARCH_VERSION_V2
------------------------------------------------------------
local  ONLINE_SEARCH_URL_STG1  = "https://adobesearch-stage.adobe.io/search/"
--local  ONLINE_SEARCH_URL_STG2  = "https://adobesearch-stage.adobe.io/universal-search-lr/v2/search"
local  ONLINE_SEARCH_URL_STG2  = "https://adobesearch-fac-stage.adobe.io/universal-search-lr/v2/search"
--local  ONLINE_SEARCH_URL  = "https://adobesearch.adobe.io/search/"
local  ONLINE_SEARCH_URL  = "https://adobesearch.adobe.io/universal-search-lr/v2/search"

----------------search API version ---------------
-- below has to match with AgConfiguration.searchConfiguration.ussVersion
WFCatSearchAndSortConstants.SEARCH_VERSION = string.lower( WFConfiguration.searchConfiguration.ussVersion 
	or DEFAULT_VERSION )
local _useV1Search =  WFCatSearchAndSortConstants.SEARCH_VERSION == WFCatSearchAndSortConstants.SEARCH_VERSION_V1

------------------------------------------------------------

local _useStage = false
-- explicitly set to false, when trying on stage in the config file
if WFConfiguration.searchConfiguration.useProduction == false then
	_useStage = true
end
	
WFCatSearchAndSortConstants.PAGE_SIZE = ( 
			WFConfiguration.searchConfiguration.pageSize and tonumber( WFConfiguration.searchConfiguration.pageSize ) )
			or ( _useV1Search and PAGE_SIZE_V1 or PAGE_SIZE_V2 )

------------------------------------------------------------
WFCatSearchAndSortConstants.getSearchUrl = function()
	if  WFConfiguration.searchConfiguration.searchURL then
		return WFConfiguration.searchConfiguration.searchURL
	end
	if _useV1Search then
		return ( _useStage and ONLINE_SEARCH_URL_STG1 ) or ONLINE_SEARCH_URL	
	else
		return ( _useStage and ONLINE_SEARCH_URL_STG2 ) or ONLINE_SEARCH_URL	
	end
end
	
------------------------------------------------------------
WFCatSearchAndSortConstants.getSearchRequestArgs = function( )
	local requestHeaders = {
			{ header = 'Content-Type', value = 'application/vnd.adobe.search-request+json' },
			{ header = 'X-Product', value =  'Nimbus/1.0' },
			{ header = 'X-Product-Location', value = 'Asset-search' },
			{ header = 'Accept', value = 'application/json' },
		}
	-- 'Authorization' and 'X-Api-Key' are taken care by WFCatuser
	local postArgs = {
		requestHeaders = requestHeaders,
		requestMethod = 'POST',
		bypassRateLimiter = true,
	}	

	return postArgs
end

--------------------------------------------------------------------------------
return WFCatSearchAndSortConstants