--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFCloseSupport = require 'WFCloseSupport'

	-- WF.core.lua
local WFWorkerThread = require 'WFWorkerThread'

	-- WF.core.table
local WFTableArchiving = require 'WFTableArchiving'

	-- WF.core.rxlua
local WFRxRunUntilDone = require 'WFRxRunUntilDone'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

local runUntilDone = WFRxRunUntilDone

--==============================================================================

-- @mark WFCatSyncClient namespace

-- Manages the reference to Sync MessageProcessor where our Sync infrastructure
-- is running on. See WFCatSyncClientImp for the actual implementation

--------------------------------------------------------------------------------

local syncClientMethods = {

	_syncClientImp = false,
		-- (WFWorkerThread) Reference to the SyncClient WorkerThread instance
	
	WFCloseSupport.addCloseSupportMethods
	
}

--------------------------------------------------------------------------------

function syncClientMethods:__init( wfCatalog, optArgs )
	
	check( 'WFCatSyncClient', 'WFCatalog', '?table' )
	
	optArgs = optArgs or {}
	
	local optIndexSpecScript = optArgs.optIndexSpecScript or 
							wfCatalog:indexStore():indexSpecScript() 
								or 'WFCatIndexDefinitions.lua'
	local optSyncClientImpName = optArgs.optSyncClientImpName or 'WFCatSyncClientImp'
	
	local options = {
		syncClientImpName = optSyncClientImpName,
		serverBase = wfCatalog:ozUser():serverBaseUrl(),
		userId = wfCatalog:ozUserId(),
		userInfoCacheFolder = wfCatalog:ozUser():userInfoCacheFolder(),
		catalogId = wfCatalog:ozCatalogId(),
		catalogDoc = wfCatalog:ozCatalogSpec():catalogDoc(),
		indexSpecScript = optIndexSpecScript,
		rootCatalogPath = wfCatalog:fileSpec():rootPath(),
		docStorePath = wfCatalog:fileSpec():docStorePath(),
		
		metadataPullerArgs = optArgs.metadataPullerArgs,
	}
	
	local optionsString = WFTableArchiving.tableToString( options )
	
	self:setOwned {
		_syncClientImp = WFWorkerThread( 'WFCatSyncClientMessageProcessor.lua', optionsString )
	}
	
	WFCloseSupport.mustClose( self )
	
end

--------------------------------------------------------------------------------

function syncClientMethods:startSync( )

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'startSync' )
		:call( runUntilDone )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:stopSync()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'stopSync' )
		:call( runUntilDone )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:startSyncInResidualMode()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'startSyncInResidualMode' )
		:call( runUntilDone )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:stopSyncInResidualMode()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'stopSyncInResidualMode' )
		:call( runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:setUserIsOverQuota( isOverQuota )
	
	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'setUserIsOverQuota', isOverQuota )
		:call(runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:setUserIsExpired( isTrialOrSubscriptionExpired )
	
	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'setUserIsExpired', isTrialOrSubscriptionExpired )
		:call(runUntilDone )

end

--------------------------------------------------------------------------------

function syncClientMethods:getLatestDownloadedAssetIdsStream()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'getLatestDownloadedAssetIdsStream' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:getDeletedDocumentIdStream()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'getDeletedDocumentIdStream' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:isOzCatalogDeleted()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'isOzCatalogDeleted' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:isUploadingMetadata()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'isUploadingMetadata' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:isDownloadingMetadata()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'isDownloadingMetadata' )
		
end

--------------------------------------------------------------------------------

function syncClientMethods:getRequiredUpdateStream()

	check( 'WFCatSyncClient' )
	
	return self._syncClientImp:request( 'getRequiredUpdateStream' )
		
end

--------------------------------------------------------------------------------

return Rx.Class( 'WFCatSyncClient', syncClientMethods )
