--[[----------------------------------------------------------------------------

WFCrInfo.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.acr
local WFCrInfoImp = require 'WFCrInfoImp'

	-- WF.core.os
local WFPlatformInfo = require 'WFPlatformInfo'
local WFConfiguration = require 'WFConfiguration'

--------------------------------------------------------------------------------

local DISABLE_HEIC_SUPPORT = WFConfiguration.Shared.disableHeicSupport or false

assert( WFPlatformInfo.hasACR, "ACR integration not enabled" )


--============================================================================--

-- @mark WFCrInfo namespace

local WFCrInfo = {
	_heicSupported = nil
}

--------------------------------------------------------------------------------

--[[!

	\brief Get the Adobe Camera Raw version.
		
	\returns major, minor, dot, version
		
]]--

function WFCrInfo.getACRVersion()

	return WFCrInfoImp.getACRVersion()

end

--------------------------------------------------------------------------------

--[[!

	\brief Check if ACR supports decoding HEIC files
		
	\returns boolean
		
]]--

function WFCrInfo.supportsHeic()

	if WFCrInfo._heicSupported == nil then
		WFCrInfo._heicSupported = WFCrInfoImp.haveH265Decoder() and not DISABLE_HEIC_SUPPORT
	end
	
	return WFCrInfo._heicSupported

end

--------------------------------------------------------------------------------

function WFCrInfo.getUserPresetDir()

	return WFCrInfoImp.getUserPresetDir()

end


--------------------------------------------------------------------------------

function WFCrInfo.setUserPresetDir( path )

	return WFCrInfoImp.setUserPresetDir( path )

end

--------------------------------------------------------------------------------

function WFCrInfo.setUserCameraProfilesDir( path )

	return WFCrInfoImp.setUserCameraProfilesDir( path )

end

--------------------------------------------------------------------------------

function WFCrInfo.setUserPreferencesDir( path )

	return WFCrInfoImp.setUserPreferencesDir( path )

end

--------------------------------------------------------------------------------

return WFCrInfo
