--[[----------------------------------------------------------------------------

WFDocStoreImpPerformanceTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.doc_store
local WFDocStoreImp = require 'WFDocStoreImp'

local WFPlatformInfo = require 'WFPlatformInfo'


--============================================================================--

-- @mark WFDocStoreImpPerformanceTest

local WFDocStoreImpPerformanceTest = {}

--------------------------------------------------------------------------------

local docStore

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.setUp()

	-- Empty string for path means create an on-disk doc store.

	docStore = WFDocStoreImp( '' )
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.tearDown()

	if docStore then

		docStore:close()
		docStore = nil

	end
	
end

--------------------------------------------------------------------------------

-- @mark :setDocValue

--------------------------------------------------------------------------------

local function setDocValue( args )
		
			for i = 1, assert( args.n ) do
				
				docStore:setDocValue {
					_id = tostring( i ),
					_slot = 'main',
					foo = args.matchingContent
						and 'zzyfx'
						or i,
				}
			end	
end

--------------------------------------------------------------------------------

local function testCaseWithLotsOfDocs( args )

	assert( type( args ) == 'table' )
	
	return WFAsyncTestCase( assert( args.test ), args.timeout or 120 )
		
		:waitForPerf( function()
		
			setDocValue( args )
			return 'ok'
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testCreate100Docs_testPerformance( test )
	
	testCaseWithLotsOfDocs {
		test = test,
		n = 100,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testCreate1000Docs_testPerformance( test )
	
	testCaseWithLotsOfDocs {
		test = test,
		n = 1000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testCreate10000Docs_testPerformance( test )
	
	testCaseWithLotsOfDocs {
		test = test,
		n = 10000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testCreate100000Docs_testPerformance( test )
	
	-- Performance on this test scales almost perfectly linearly from 100 to 100K.
	-- This makes me very happy.
	
	testCaseWithLotsOfDocs {
		test = test,
		n = 100000,
	}
	
end

--------------------------------------------------------------------------------

-- @mark :bulkSetDocValue

--------------------------------------------------------------------------------

local function testCaseWithLotsOfBulkDocs( args )

	assert( type( args ) == 'table' )
	
	local matchingContent = args.matchingContent

	return WFAsyncTestCase( assert( args.test ), args.timeout or 120 )
		
		:waitFor( function()
		
			for i = 1, assert( args.n ) / 100 do
				
				local docValues = {}
				
				for j = 1, 100 do
					docValues[ j ] = {
						_id = 'testDoc' .. i .. '_' .. j,
						_slot = 'main',
						foo = matchingContent
							and 'zzyfx'
							or ( i + j - 1 ),
					}
				end
				
				docStore:bulkSetDocValue( docValues )
				
			end
		
			return 'ok'
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testBulkCreate1000Docs_testPerformance( test )
	
	testCaseWithLotsOfBulkDocs {
		test = test,
		n = 1000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testBulkCreate10000Docs_testPerformance( test )
	
	testCaseWithLotsOfBulkDocs {
		test = test,
		n = 10000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testBulkCreate100000Docs_testPerformance( test )
	
	-- Performance on this test scales almost perfectly linearly from 100 to 100K.
	-- This makes me very happy.

	testCaseWithLotsOfBulkDocs {
		test = test,
		n = 100000,
	}
	
end

--------------------------------------------------------------------------------

-- @mark :docValue

--------------------------------------------------------------------------------

local function getDocValueForLotsOfDocs( args )

	assert( type( args ) == 'table' )
	
	return WFAsyncTestCase( assert( args.test ), args.timeout or 120 )
		
		:waitFor( function()
		
			setDocValue( args )
			return 'ok'
		
		end )
		
		:waitForPerf( function()
		
			for i = 1, args.n do
				docStore:docValue( tostring( i ) )
					-- ignore return value
			end
			
			return 'ok'
		
		end )
		
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testRead100Docs_testPerformance( test )
	
	getDocValueForLotsOfDocs {
		test = test,
		n = 100,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testRead1000Docs_testPerformance( test )
	
	getDocValueForLotsOfDocs {
		test = test,
		n = 1000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testRead10000Docs_testPerformance( test )
	
	getDocValueForLotsOfDocs {
		test = test,
		n = 10000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testRead100000Docs_testPerformance( test )
	
	-- Performance on this test scales almost perfectly linearly from 100 to 100K.
	-- This makes me very happy.
	
	getDocValueForLotsOfDocs {
		test = test,
		n = 100000,
	}
	
end

--------------------------------------------------------------------------------

-- @mark :changedDocs

--------------------------------------------------------------------------------

local function getChangedDocsForLotsOfDocs( args )

	assert( type( args ) == 'table' )
	
	return WFAsyncTestCase( assert( args.test ), args.timeout or 120 )
		
		:waitFor( function()
		
			setDocValue( args )
			return 'ok'
		
		end )

		:waitForPerf( function()
		
			local fromChangeCounter = 0
			
			while true do
			
				local changes = docStore:changedDocs
					{ fromChangeCounter = fromChangeCounter, slot = 'main' }

				if #changes == 0 then break end
				
				fromChangeCounter = changes.continueFrom
		
			end
		
			return 'ok'
		
		end )
		
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testGetChanges100Docs_testPerformance( test )
	
	getChangedDocsForLotsOfDocs {
		test = test,
		n = 100,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testGetChanges1000Docs_testPerformance( test )
	
	getChangedDocsForLotsOfDocs {
		test = test,
		n = 1000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testGetChanges10000Docs_testPerformance( test )
	
	getChangedDocsForLotsOfDocs {
		test = test,
		n = 10000,
	}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpPerformanceTest.testGetChanges100000Docs_testPerformance( test )
	
	-- Performance on this test scales almost perfectly linearly from 100 to 100K.
	-- This makes me very happy.
	
	getChangedDocsForLotsOfDocs {
		test = test,
		n = 100000,
	}
	
end

--------------------------------------------------------------------------------

--FIXME [rseeler 2013-11-25]: These test all timeout.  This should be investigated at some point.

if WFPlatformInfo.isAndroid then
	return {}
else
	return WFDocStoreImpPerformanceTest
end
