--[[----------------------------------------------------------------------------

WFDocStoreImpTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.reactive
local KSPromises = require 'KSPromises'

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.core.utils
local KSIdGenerator = require 'KSIdGenerator'
local KSTestUtils = require 'KSTestUtils'
local WFArguments = require 'WFArguments'
local WFUnknown = require 'WFUnknown'

	-- WF.oz_client.doc_store
local WFDocStoreImp = require 'WFDocStoreImp'


--============================================================================--

local WFDocStoreImpTest = {}

local check = assert( WFArguments.check )

local docStore

--------------------------------------------------------------------------------

function WFDocStoreImpTest.setUp()

	docStore = WFDocStoreImp( ':memory:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.tearDown( coveragePath )

	if docStore then

		if coveragePath then
			docStore:DEBUG_dumpCoverage( coveragePath )
		end

		docStore:close()
		docStore = nil

	end

end

--------------------------------------------------------------------------------

-- @mark whole document values

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDoc( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_emptyDoc( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId' )

	WFAsyncTestCase( test )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_emptyDocAssertingNew( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId' )

	WFAsyncTestCase( test )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = ':new:',
				test = 42,
			}

			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_reuseExisting( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId' )

	WFAsyncTestCase( test )
	
		:waitFor( function()
			
			return docStore:docValueB( 'testDocId' )
			
		end )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_slotCrossContamination( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId', 'other' )

	WFAsyncTestCase( test )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}
		
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_replacingHash( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			return hash
		
		end )
		
		:waitFor( function()
			
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_hash = hash,
				test = 45,
			}
			
			expectedValue._hash = hash
			return hash
		
		end )
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_replacingChangeCounter( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			return hash
		
		end )
		
		:waitFor( function()
			
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = 1,
				test = 45,
			}
			
			expectedValue._hash = hash
			return hash
		
		end )
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDocNestedArray( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
		array = { 'a', 'b', 'c' },
		hash = { a = 1, b = 2, c = 3 },
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				array = { 'a', 'b', 'c' },
				hash = { a = 1, b = 2, c = 3 },
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowSyncStatusSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
		_syncStatus = { 'a', 'b', 'c' },
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				_syncStatus = { 'a', 'b', 'c' },
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowDeletedSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue_afterDelete = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_deleted = true,
		_changeCounter = 2,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
			
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_deleted = true,
			}

			expectedValue_afterDelete._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue_afterDelete )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowEmptyStringKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		[ '' ] = "now allowed",
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				[ '' ] = "now allowed",
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowNestedUnderscoreKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		noUnderscoresHere = {
			_butHereIsNowAllowed = true,
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				noUnderscoresHere = {
					_butHereIsNowAllowed = true,
				},
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_hashReflectsBodyContentOnly( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue_afterLocalOnlyChange = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_localOnly = { foo = 'bar' },
		test = 42,
		_changeCounter = 2,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			expectedValue._hash = hash
			expectedValue_afterLocalOnlyChange._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
			
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_localOnly = { foo = 'bar' },
				test = 42,
			}

			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue_afterLocalOnlyChange )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_hashReflectsDeletion( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
	}

	local firstHash

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
			}

			expectedValue._hash = hash
			firstHash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
			
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_deleted = true,
			}
			
			return tostring( hash == firstHash )

		end )
	
		:continueWhenValueMatches( 'data', 'false' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDocTempDB( test )

	-- Same test as above, but proving we can use ':temp:' pseudo-path as well.

	docStore:close()
	docStore = WFDocStoreImp( ':temp:' )

	return WFDocStoreImpTest.testGetSetDocValue_emptyDoc( test )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetDocValue_empty( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42
			}
		
		end )

		:waitFor( function()
		
			return docStore:docValue( 'bogusDocId' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'bogusDocId',
				_slot = 'main',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorReplacingWrongHash( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_hash = 'notTheRightHash',
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorNotNew( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = ':new:',
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorReplacingWrongChangeCounter( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 71,
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingTable()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue()
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingId()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIdNotString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIdEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingSlot()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { id = 'ok' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueSlotNotString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 'ok', _slot = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueSlotEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 'ok', _slot = '' }
	end )

end

--------------------------------------------------------------------------------

local function setDocValueShouldFail( value )

	value._id = 'testDocId'
	value._slot = 'main'

	KSTestUtils.assertFails( function()
		return docStore:setDocValue( value )
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIsInvalid( test )

	local valueWithMT = { test = 42 }
	setmetatable( valueWithMT, {} )

	setDocValueShouldFail { _transient = true, blah = 42 }
		-- Support for _transient flag removed 2013-09-03.

	setDocValueShouldFail( valueWithMT )

	setDocValueShouldFail { [ 42 ] = 'string keys only, please' }
	
	setDocValueShouldFail { _blah = 'must not start with underscore' }
	
	setDocValueShouldFail { func = function() end }
	
	setDocValueShouldFail
		{
			nested = {
				[ true ] = 'enforce non-string/number rule in nested tables',
			},
		}
	
	setDocValueShouldFail
		{
			nested = {
				nested2 = {
					[ true ] = 'enforce non-string/number rule in deeply nested tables',
				},
			},
		}
	
	setDocValueShouldFail
		{
			_deleted = true,
			nothingElseAllowed = true,
				-- If _deleted = true, then no other values are allowed in the table.
		}
	
	setDocValueShouldFail
		{
			nested = {
				[ 1 / 0 ] = "to infinity and beyond, but not in doc values"
			}
		}

	setDocValueShouldFail
		{
			nested = {
				[ -1 / 0 ] = "to negative infinity and beyond, but not in doc values"
			}
		}
	
	setDocValueShouldFail
		{
			a = 1 / 0, -- infinity not allowed as value (?)
		}
	
	setDocValueShouldFail
		{
			a = -1 / 0, -- negative infinity not allowed as value (?)
		}
	
	setDocValueShouldFail
		{
			a = 0 / 0, -- NaN not allowed as value (?)
		}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_eraseOldValue( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		blah = 95,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		-- Update doc value and test read-back again.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				blah = 95,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

		-- Make sure old value no longer exists in DB.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT 1 FROM bodyContent WHERE bodyContentHash == ?]],
									  firstHash ) or ':empty:'
		
		end )
		:continueWhenValueMatches( 'data', ':empty:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_differentSlots( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 3,
		blah = 95,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		-- Set second slot to same doc value.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
		
		end )
	
		-- Now change second slot value, leaving first slot at existing value.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				blah = 95,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		-- Now read both values back.
	
		:waitFor( function()
			return docStore:docValue( 'testDocId', 'main' )
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		:waitFor( function()
			return docStore:docValue( 'testDocId', 'other' )
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_reuseExistingValue( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		-- Set doc value to same value and capture rev ID.
		-- Should not bump change counter.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

-- @mark bulk set document values

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_emptyDoc( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
					}
				}
		
			local result = docStore:docValue( 'testDocId' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

local function docValueB_strippingHash( optSlotName )

	return docStore:docValueB( 'testDocId', optSlotName )
		:mapB( function( docValue )

			if docValue and WFUnknown.isKnown( docValue ) then
				docValue._hash = nil
			end

			return docValue

		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_emptyDocAssertingNew( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docValueB_strippingHash()

	WFAsyncTestCase( test )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						_changeCounter = ':new:',
						test = 42,
					}
				}
			
			return "ok"
		
		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_slotCrossContamination( test )

	local docValueB = docValueB_strippingHash( 'other' )

	WFAsyncTestCase( test )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}
		
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
					}
				}
			
			return "ok"
		
		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_replacingHash( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			return hash
		
		end )
		
		:waitFor( function()
			
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						_hash = hash,
						test = 45,
					}
				}
			
			return docValueB_strippingHash()
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_replacingChangeCounter( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						_changeCounter = 1,
						test = 45,
					}
				}
			
			return docValueB_strippingHash()
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_emptyDocNestedArray( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		test = 42,
		array = { 'a', 'b', 'c' },
		hash = { a = 1, b = 2, c = 3 },
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
						array = { 'a', 'b', 'c' },
						hash = { a = 1, b = 2, c = 3 },
					}
				}
		
			return docValueB_strippingHash()
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_allowSyncStatusSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		test = 42,
		_syncStatus = { 'a', 'b', 'c' },
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
						_syncStatus = { 'a', 'b', 'c' },
					}
				}

			return docValueB_strippingHash()
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_allowDeletedSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue_afterDelete = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_deleted = true,
		_changeCounter = 2,
	}

	local docValueB = docValueB_strippingHash()

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			return docValueB
		
		end )
		:continueWhenValueMatches( expectedValue )

		:waitFor( function()
			
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						_deleted = true,
					}
				}
		
			return docValueB
		
		end )
		:continueWhenValueMatches( expectedValue_afterDelete )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_allowEmptyStringKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		[ '' ] = "now allowed",
	}

	local docValueB = docValueB_strippingHash()

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				[ '' ] = "now allowed",
			}

			return docValueB
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_allowNestedUnderscoreKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		_changeCounter = 1,
		noUnderscoresHere = {
			_butHereIsNowAllowed = true,
		},
	}

	local docValueB = docValueB_strippingHash()

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				noUnderscoresHere = {
					_butHereIsNowAllowed = true,
				},
			}

			return docValueB
		
		end )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_hashReflectsBodyContentOnly( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue_afterLocalOnlyChange = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_localOnly = { foo = 'bar' },
		test = 42,
		_changeCounter = 2,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
					}
				}

			local result = docStore:docValue( 'testDocId' )
			
			expectedValue._hash = result._hash
			expectedValue_afterLocalOnlyChange._hash = result._hash
		
			return result
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
			
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						_localOnly = { foo = 'bar' },
						test = 42,
					}
				}

			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue_afterLocalOnlyChange )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_hashReflectsDeletion( test )

	docStore:bulkSetDocValue
		{
			{
				_id = 'testDocId',
				_slot = 'main',
			}
		}
	
	local firstHash = docStore:docValue( 'testDocId' )._hash
		
	docStore:bulkSetDocValue
		{
			{
				_id = 'testDocId',
				_slot = 'main',
				_deleted = true,
			}
		}
	
	local newDocValue = docStore:docValue( 'testDocId' )
	local hash = newDocValue._hash

	assert( hash ~= firstHash )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_accepts200Docs( test )

	local docValues = {}
	for i = 1, 200 do
		docValues[ i ] =
			{
				_id = 'test_' .. tostring( i ),
				_slot = 'main',
				foo = 'bar',
			}
	end

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			docStore:bulkSetDocValue( docValues )
			
			local result = docStore:docValue( 'test_1' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'test_1',
				_idLocal = 1,
				_slot = 'main',
				_changeCounter = 1,
				foo = 'bar',
			} )

		:waitFor( function()
			
			local result = docStore:docValue( 'test_200' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'test_200',
				_idLocal = 200,
				_slot = 'main',
				_changeCounter = 200,
				foo = 'bar',
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorReplacingWrongHash( test )

	docStore:bulkSetDocValue
		{
			{
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		}
		
	KSTestUtils.assertFails( function()
		docStore:bulkSetDocValue
			{
				{
					_id = 'testDocId',
					_slot = 'main',
					_hash = 'notTheRightHash',
					test = 45,
				}
			}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorNotNew( test )

	docStore:bulkSetDocValue
		{
			{
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		}
		
	KSTestUtils.assertFails( function()
		docStore:bulkSetDocValue
			{
				{
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = ':new:',
					test = 45,
				}
			}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorReplacingWrongChangeCounter( test )

	docStore:bulkSetDocValue
		{
			{
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		}
		
	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue
			{
				{
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 71,
					test = 45,
				}
			}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueMissingTable()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue()
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueMissingId()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { {} }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueIdNotString()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { { _id = 4 } }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueIdEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { { _id = '' } }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueMissingSlot()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { { id = 'ok' } }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueSlotNotString()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { { _id = 'ok', _slot = 4 } }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueSlotEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { { _id = 'ok', _slot = '' } }
	end )

end

--------------------------------------------------------------------------------

local function bulkSetDocValueShouldFail( value )

	value._id = 'testDocId'
	value._slot = 'main'

	KSTestUtils.assertFails( function()
		return docStore:bulkSetDocValue { value }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_errorDocValueIsInvalid( test )

	local valueWithMT = { test = 42 }
	setmetatable( valueWithMT, {} )

	bulkSetDocValueShouldFail { _transient = true, blah = 42 }
		-- Support for _transient flag removed 2013-09-03.

	bulkSetDocValueShouldFail( valueWithMT )

	bulkSetDocValueShouldFail { [ 42 ] = 'string keys only, please' }
	
	bulkSetDocValueShouldFail { _blah = 'must not start with underscore' }
	
	bulkSetDocValueShouldFail { func = function() end }
	
	bulkSetDocValueShouldFail
		{
			nested = {
				[ true ] = 'enforce non-string/number rule in nested tables',
			},
		}
	
	bulkSetDocValueShouldFail
		{
			nested = {
				nested2 = {
					[ true ] = 'enforce non-string/number rule in deeply nested tables',
				},
			},
		}
	
	bulkSetDocValueShouldFail
		{
			_deleted = true,
			nothingElseAllowed = true,
				-- If _deleted = true, then no other values are allowed in the table.
		}
	
	bulkSetDocValueShouldFail
		{
			nested = {
				[ 1 / 0 ] = "to infinity and beyond, but not in doc values"
			}
		}

	bulkSetDocValueShouldFail
		{
			nested = {
				[ -1 / 0 ] = "to negative infinity and beyond, but not in doc values"
			}
		}
	
	bulkSetDocValueShouldFail
		{
			a = 1 / 0, -- infinity not allowed as value (?)
		}
	
	bulkSetDocValueShouldFail
		{
			a = -1 / 0, -- negative infinity not allowed as value (?)
		}
	
	bulkSetDocValueShouldFail
		{
			a = 0 / 0, -- NaN not allowed as value (?)
		}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_eraseOldValue( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		blah = 95,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		-- Update doc value and test read-back again.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						blah = 95,
					}
				}
			
			return docStore:docValue( 'testDocId' )._hash
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

		-- Make sure old value no longer exists in DB.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT 1 FROM bodyContent WHERE bodyContentHash == ?]],
									  firstHash ) or ':empty:'
		
		end )
		:continueWhenValueMatches( 'data', ':empty:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_differentSlots( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 3,
		blah = 95,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		-- Set second slot to same doc value.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'other',
						test = 42,
					}
				}
			
			return 'ok'
		
		end )
	
		-- Now change second slot value, leaving first slot at existing value.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'other',
						blah = 95,
					}
				}
				
			return docStore:docValue( 'testDocId', 'other' )._hash
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		-- Now read both values back.
	
		:waitFor( function()
			return docStore:docValue( 'testDocId', 'main' )
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		:waitFor( function()
			return docStore:docValue( 'testDocId', 'other' )
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_reuseExistingValue( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		-- Set doc value to same value and capture rev ID.
		-- Should not bump change counter.
	
		:waitFor( function()
		
			docStore:bulkSetDocValue
				{
					{
						_id = 'testDocId',
						_slot = 'main',
						test = 42,
					}
				}
			
			return docStore:docValue( 'testDocId' )._hash
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testBulkSetDocValue_failsAtomically( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			KSTestUtils.assertFails( function()

				docStore:bulkSetDocValue
					{
						{
							_id = 'test_1',
							_slot = 'main',
							foo = 42,
						},
						{
							_id = 'test_2',
							_slot = 'main',
							_invalid = 'this should make it fail',
							foo = 45,
						},
					}

			end )
			
			return 'ok'
		
		end )
		
		-- Neither of these documents should persist.

		:waitFor( function()
			
			return docStore:docValue( 'test_1' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'test_1',
				_slot = 'main',
				_empty = true,
			} )

		:waitFor( function()
			
			return docStore:docValue( 'test_2' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'test_2',
				_slot = 'main',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

-- @mark partial document values

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_empty( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteFromEmpty( test )

	local result = docStore:applyOverlay {
		_id = 'testDocId',
		_slot = 'main',
		test = {
			_delete = true
		},
	}
	
	assert( result == nil, "applyOverlay to new document with only deletes in overlay should return nil" )

	local doc = docStore:docValue( 'testDocId' )
	
	assert( doc._empty, "Got empty document at the end" )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_afterSetDocValue( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueAfterOverlay = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = 45,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = 45,
			}
			
			expectedValueAfterOverlay._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueAfterOverlay )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_nestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_nestedTableReplacingScalar( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = 95,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_mergeNestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = {
					a = 1,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteNestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteScalar( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = "zip",
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteNothing( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_replaceExisting( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		nested = {
			a = 1,
			b = 2,
			[ 42 ] = 'c',
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				nested = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				nested = {
					_replace = true,
					a = 1,
					b = 2,
					[ 42 ] = 'c',
				},
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_replaceNothing( test )

	local expectedValue = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		nested = {
			a = 1,
			b = 2,
			[ 42 ] = 'c',
		},
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				nested = {
					_replace = true,
					a = 1,
					b = 2,
					[ 42 ] = 'c',
				},
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingTable()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay()
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingId()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorIdNotString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorIdEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingSlot()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { id = 'ok' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorSlotNotString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 'ok', _slot = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorSlotEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 'ok', _slot = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorChangingType( test )

	docStore:applyOverlay {
		_id = 'testDocId',
		_slot = 'main',
		test = 42,
		type = 'whatever',
	}
	
	KSTestUtils.assertFails( function()
	
		docStore:applyOverlay {
			_id = 'testDocId',
			_slot = 'main',
			test = 45,
			type = 'not_whatever',
		}
	
	end )
	
end

--------------------------------------------------------------------------------

-- @mark slot management

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_docValueB( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}
	
	local docValueB_main = docStore:docValueB( 'testDocId', 'main' )
	local docValueB_other = docStore:docValueB( 'testDocId', 'other' )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )
		
		:waitFor( docValueB_main )
		:continueWhenValueMatches( expectedValueMain )

		:waitFor( docValueB_other )
		:continueWhenValueMatches {
			_id = 'testDocId',
			_slot = 'other',
			_empty = true,
		}

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( docValueB_main )
		:continueWhenValueMatches( expectedValueMain )

		:waitFor( docValueB_other )
		:continueWhenValueMatches( expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_shortCircuitExistingValueMatches( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
			-- should not bump to 3 when we do :copySlotToSlot
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )

			expectedValueOther._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'other', 'main' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_errorNoSuchSource( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			KSTestUtils.assertFails( function()
				return docStore:copySlotToSlot( 'testDocId', 'bogus', 'other' )
			end )

			return true
			
		end )
		
		-- Make sure we can read back expected results.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )

		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_overwriteExistingValue( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_idLocal = 1,
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_idLocal = 2,
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 3,
		test = 42,
	}

	WFAsyncTestCase( test )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				blah = 57,
			}
			
		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot( test )

	local firstHash

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			firstHash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'blah',
				test = 42,
			}
			
			return firstHash

		end )
		
		:waitFor( function()
			
			local result = docStore:docValue( 'testDocId', 'blah' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_idLocal = 1,
				_slot = 'blah',
				_changeCounter = 1,
				test = 42,
			} )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

		-- Make sure old value no longer exists in DB.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT 1 FROM bodyContent WHERE bodyContentHash == ?]],
									  firstHash ) or ':empty:'
		
		end )
		:continueWhenValueMatches( 'data', ':empty:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_doesntExist( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_updatesDocValueB( test )

	local expectedValue =
		{
			_id = 'testDocId',
			_idLocal = 1,
			_slot = 'blah',
			_changeCounter = 1,
			test = 42,
		}
	
	local docValueB = docStore:docValueB( 'testDocId', 'blah' )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'blah',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash

		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches(
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_errorCantDeleteMain( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )
		
		:waitFor( function()
			
			local result = docStore:docValue( 'testDocId', 'main' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_idLocal = 1,
				_slot = 'main',
				_changeCounter = 1,
				test = 42,
			} )
		
		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				docStore:quietlyDeleteSlot( 'testDocId', 'main' )
			end )
			
			return true
		
		end )
		
end

--------------------------------------------------------------------------------

-- @mark change notification

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocChangeCounterB( test )

	local docChangeCounterB = docStore:docChangeCounterB()

	WFAsyncTestCase( test )
	
		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 0 )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )

		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 1 )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				rating = 3,
			}
			
		end )

		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 2 )
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocChangeCounter( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
				-- because WFAsyncTestCase interprets a number as a #seconds
				-- to wait :-(
		
		end )
		:continueWhenValueMatches( 'data', '0' )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )

		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
		
		end )
		:continueWhenValueMatches( 'data', '1' )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				rating = 3,
			}
			
		end )

		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
		
		end )
		:continueWhenValueMatches( 'data', '2' )
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs( test )

	local expectedChangedDocsResult =
		{

			{
				_id = 'testDocId',
				_idLocal = 1,
				_slot = 'main',
				_changeCounter = 1,
				_hash = nil, -- filled in later
				test = 42,
			},
			{
				_id = 'anotherPhoto',
				_idLocal = 2,
				_slot = 'main',
				_changeCounter = 2,
				_hash = nil, -- filled in later
				favorite = true,
				title = "My Nifty Photo",
			},
			
			continueFrom = 3,
		
		}

	local expectedChangedDocsResult2 =
		{

			{
				_id = 'testDocId',
				_idLocal = 1,
				_slot = 'main',
				_changeCounter = 3,
				_hash = nil, -- filled in later
				test = 53,
			},
			
			continueFrom = 4,
		
		}

	WFAsyncTestCase( test )
		
		-- Write two documents.
		
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			expectedChangedDocsResult[ 1 ]._hash = hash

			return hash

		end )
		
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'anotherPhoto',
				_slot = 'main',
				favorite = true,
				title = "My Nifty Photo",
			}

			expectedChangedDocsResult[ 2 ]._hash = hash

			return hash

		end )
	
		-- Read them back via :changedDocs.
		
		:waitFor( function()
			return docStore:changedDocs { fromChangeCounter = 1 }
		end )
		
		:continueWhenValueMatches( 'data', expectedChangedDocsResult )

		-- Change one of the docs again.
		
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = 1,
				test = 53,
			}
			expectedChangedDocsResult2[ 1 ]._hash = hash

			return hash

		end )
		
		-- Read this change (only) back via :changedDocs.
		
		:waitFor( function()
			return docStore:changedDocs { fromChangeCounter = 3 }
		end )
		
		:continueWhenValueMatches( 'data', expectedChangedDocsResult2 )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_empty( test )

	WFAsyncTestCase( test )
		
		:waitFor( function()
			return docStore:changedDocs { fromChangeCounter = 1 }
		end )
		
		:continueWhenValueMatches( 'data', { continueFrom = 1 } )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterMissing( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterString( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = '42' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterInfinity( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1/0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterNegativeInfinity( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = -1/0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterNaN( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 0/0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitNotString( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 'bogus' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitNotPositive( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitInfinity( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 1/0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitNaN( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 0/0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorSlotNotString( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, slot = 0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorSlotNameEmpty( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, slot = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesLimit( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 51 }
	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i ],
			_idLocal = i,
			_slot = 'main',
			_changeCounter = i,
			favorite = true,
		}
	end
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i <= 50 then
					expectedResult[ i ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, limit = 1000 }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesSlotName( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }
	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i * 2 ],
			_idLocal = i * 2,
			_slot = 'even',
			_changeCounter = i * 2,
			favorite = true,
		}
	end
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = i % 2 == 0 and 'even' or 'odd',
					favorite = true,
				}

				if i % 2 == 0 then
					expectedResult[ i / 2 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, slot = 'even' }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesSlotName_empty( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, slot = 'nonesuch' }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesDocInclusionList( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }

	expectedResult[ 1 ] = {
		_id = docIds[ 17 ],
		_idLocal = 17,
		_slot = 'main',
		_changeCounter = 17,
		favorite = true,
	}
	
	expectedResult[ 2 ] = {
		_id = docIds[ 50 ],
		_idLocal = 50,
		_slot = 'main',
		_changeCounter = 50,
		favorite = true,
	}
	
	expectedResult[ 3 ] = {
		_id = docIds[ 51 ],
		_idLocal = 51,
		_slot = 'main',
		_changeCounter = 51,
		favorite = true,
	}
	
	expectedResult[ 4 ] = {
		_id = docIds[ 87 ],
		_idLocal = 87,
		_slot = 'main',
		_changeCounter = 87,
		favorite = true,
	}
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i == 17 then
					expectedResult[ 1 ]._hash = hash
				elseif i == 50 then
					expectedResult[ 2 ]._hash = hash
				elseif i == 51 then
					expectedResult[ 3 ]._hash = hash
				elseif i == 87 then
					expectedResult[ 4 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesDocInclusionListAndSlotName( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }

	expectedResult[ 1 ] = {
		_id = docIds[ 17 ],
		_idLocal = 17,
		_slot = 'odd',
		_changeCounter = 17,
		favorite = true,
	}
	
	expectedResult[ 2 ] = {
		_id = docIds[ 51 ],
		_idLocal = 51,
		_slot = 'odd',
		_changeCounter = 51,
		favorite = true,
	}
	
	expectedResult[ 3 ] = {
		_id = docIds[ 87 ],
		_idLocal = 87,
		_slot = 'odd',
		_changeCounter = 87,
		favorite = true,
	}
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = i % 2 == 0 and 'even' or 'odd',
					favorite = true,
				}

				if i == 17 then
					expectedResult[ 1 ]._hash = hash
				elseif i == 51 then
					expectedResult[ 2 ]._hash = hash
				elseif i == 87 then
					expectedResult[ 3 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				slot = 'odd',
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorDocInclusionListNotTable( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, docs = 42 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorDocInclusionListEmptyTable( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, docs = {} }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_warnDocInclusionListIsLarge( test )

	local docIds = {}
	for i = 1, 120 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 51 }

	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i ],
			_idLocal = i,
			_slot = 'main',
			_changeCounter = i,
			favorite = true,
		}
	end

	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i <= 50 then
					expectedResult[ i ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				docs = docIds, -- yes, that's too many, but we'll warn and continue
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_idsOnlyObservesLimit( test )

	local docIds = {}
	for i = 1, 2000 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 1001 }
	for i = 1, 1000 do
		expectedResult[ i ] = docIds[ i ]
	end
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				idsOnly = true,
				limit = 1787,
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_idsOnlyObservesDocInclusionList( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = {
		continueFrom = 101,
		docIds[ 17 ],
		docIds[ 50 ],
		docIds[ 51 ],
		docIds[ 87 ],
	}
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot ='main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				idsOnly = true,
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

-- @mark complex logic

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_errorBadFormat( test )

	KSTestUtils.assertFails( function()

		return docStore:doComplexLogicInTransaction(
				'noPeriodInThisNamespaceAndFuncName' )

	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_errorNoSuchNamepace( test )

	KSTestUtils.assertFails( function()

		return docStore:doComplexLogicInTransaction(
				'WFDocStoreTestHelperFuncs_bogus.rollbackOnError' )

	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_errorNoSuchFunc( test )

	KSTestUtils.assertFails( function()

		return docStore:doComplexLogicInTransaction(
				'WFDocStoreTestHelperFuncs.noFunctionByThisName' )

	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_passesArgs( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.passesArgs', 42, nil, 'hello' ) )
		
		end )
		
		:continueWhenValueMatches( 'data', 42, nil, 'hello' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_docValue( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.docValue' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			return 'data', docStore:docValue( 'testDocId' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_getSetDocValue( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.getSetDocValue' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			local docValue = docStore:docValue( 'testDocId' )
			
			assert( docValue._id == 'testDocId' )
			assert( docValue._slot == 'main' )
			assert( docValue.test == 42 )
		
			return 'data', docStore:docValue( 'testDocId' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_rollbackOnError( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			KSTestUtils.assertFails( function()
		
				return KSPromises.resolvedPromiseP( 'data',
					docStore:doComplexLogicInTransaction(
						'WFDocStoreTestHelperFuncs.rollbackOnError' ) )

			end )
			
			return 'ok'
		
		end )
		
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:docValue( 'testDocId' ) )
			
		end )
		
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_applyOverlay( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.applyOverlay' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			local docValue = docStore:docValue( 'testDocId' )
			
			assert( docValue._id == 'testDocId' )
			assert( docValue._slot == 'main' )
			assert( docValue.test == 42 )
			assert( docValue.foo == 'bar' )
		
			return 'data', docStore:docValue( 'testDocId' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_copySlotToSlot( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.copySlotToSlot' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			local docValue = docStore:docValue( 'testDocId', 'other' )
			
			assert( docValue._idLocal == 2 )
			assert( docValue._id == 'testDocId' )
			assert( docValue._slot == 'other' )
			assert( docValue.test == 42 )
		
			return 'data', docStore:docValue( 'testDocId', 'other' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_copySlotToSlot_shortCircuit( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.copySlotToSlot' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			local docValue = docStore:docValue( 'testDocId', 'other' )
			
			assert( docValue._id == 'testDocId' )
			assert( docValue._slot == 'other' )
			assert( docValue.test == 42 )
		
			return 'data', docStore:docValue( 'testDocId', 'other' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_quietlyDeleteSlot( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
		
		end )
	
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.quietlyDeleteSlot' ) )
		
		end )
		
		:continueWhenValueMatchesFunction( function()
		
			local docValue = docStore:docValue( 'testDocId', 'other' )
			
			assert( docValue._id == 'testDocId' )
			assert( docValue._slot == 'other' )
			assert( docValue._empty == true )
		
			return 'data', docStore:docValue( 'testDocId', 'other' )
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDoComplexLogicInTransaction_changedDocs( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 51 }
	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i ],
			_idLocal = i,
			_slot = 'main',
			_changeCounter = i,
			favorite = true,
		}
	end
	
	WFAsyncTestCase( test )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i <= 50 then
					expectedResult[ i ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return KSPromises.resolvedPromiseP( 'data',
				docStore:doComplexLogicInTransaction(
					'WFDocStoreTestHelperFuncs.changedDocs' ) )
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

-- @mark miscellaneous

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testPath()

	local path = docStore:path()
	
	assert( type( path ) == 'string' and #path > 0,
		":path() should have returned a non-empty string" )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testToString()

	local docStoreAsString = tostring( docStore )
	
	assert( type( docStoreAsString ) == 'string' and #docStoreAsString > 0,
		"tostring() should have returned a non-empty string" )

end

--------------------------------------------------------------------------------

-- FIXME [jandrew 3/5/2015]: This test suite is unstable on Android.
local WFPlatformInfo = require 'WFPlatformInfo'
if WFPlatformInfo.isAndroid then
	return {}
else
	return WFDocStoreImpTest
end
