--[[----------------------------------------------------------------------------

WFIndexStore_OzDocStore_Test.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.utils
local KSIdGenerator = require 'KSIdGenerator'

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.doc_store
local OzDocStore = require 'OzDocStore'

	-- WF.oz_client.index_store
local WFIndexStore = require 'WFIndexStore'


--============================================================================--

local WFIndexStore_OzDocStore_Test = {}

local docStore, indexStore, failBecause

local randomDocId = '9b572d0e423c4654b078893ddd41c230'

--------------------------------------------------------------------------------

-- FIXME [scouten 2012-08-22]: This file would probably benefit from some of the
-- new tests in WFIndexStoreQueryServerTest. OTOH, that gives nearly 100% coverage
-- of the underlying code, so maybe not as critical.

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.setUp()

	docStore = OzDocStore( ':memory:' )

	failBecause = nil
	
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.tearDown( coveragePath )
	
	if coveragePath then
		if indexStore then
			indexStore:DEBUG_dumpCoverageP( coveragePath )
			:runUntilDone()
		end
	end
	
	if docStore then
		docStore:close()
		docStore = nil
	end

	if indexStore then
		indexStore:close()
		indexStore = nil
	end

	if failBecause then
		error( failBecause )
	end

end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testSimpleQueryB( test )

	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )

	local aspectRatiosB = indexStore:queryB(
		[[SELECT docId, aspectRatio FROM aspectRatio
					ORDER BY aspectRatio DESC]] )
					
	WFAsyncTestCase( test )

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						height = 150,
						width = 100,
					}
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches { { randomDocId, 1.5 } }

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = 'af21cdb6d2fa4d1c8099d08a6ba1d7c1',
					overlay = {
						height = 130,
						width = 100,
					}
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches {
			{ randomDocId, 1.5 },
			{ 'af21cdb6d2fa4d1c8099d08a6ba1d7c1', 1.3 },
		}
		
end

--------------------------------------------------------------------------------

local function mkIdArray( n )
	local t = {}
	for i = 1, n do
		t[ i ] = string.rep( string.format( "%08x", i ), 4 )
	end
	return t
end

--------------------------------------------------------------------------------

-- not really a unit test; used by arahn to assets perf of tempIds
function WFIndexStore_OzDocStore_Test.DISABLED_testPerformance( test )
	
	local KSPromises = require 'KSPromises'
	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )
	
	local batch = 1
	local ids = mkIdArray( 256 * batch )
	
	local function extendIds()
		batch = batch * 2
		ids = mkIdArray( 256 * batch )
	end
	
	local timings = {}
	local KSDate = require 'KSDate'

	local function simplyQueryToForceTableUpToDateP()
		return indexStore:queryP {
			sql = [[SELECT aspectRatio.docId FROM aspectRatio LIMIT 1]],
		}
	end

	local function aspectRatiosWithTempIdsP()
		local curTiming = { start = KSDate.timeNowForAnimation(), test = "temp_ids", N = #ids}
		return indexStore:queryP {
			sql = [[SELECT count( aspectRatio.docId ) FROM aspectRatio
				INNER JOIN TEMP_ids ON aspectRatio.docId = TEMP_ids.docId]],
--					WHERE docId IN ( SELECT docId FROM TEMP_ids )]],
			tempIds = ids,
		}
		:mapDataP( function()
			curTiming.endTime = KSDate.timeNowForAnimation()
			curTiming.duration = curTiming.endTime - curTiming.start
			timings[ #timings + 1 ] = curTiming
		end )
	end
	
	local function aspectRatiosWithIdConcatenatedSqlP()
		local curTiming = { start = KSDate.timeNowForAnimation(), test = "idConcatenation", N = #ids }
		return indexStore:queryP {
			sql = [[SELECT count( aspectRatio.docId ) FROM aspectRatio
					WHERE docId IN ( ']] .. table.concat( ids, "', '" ) .. [[' )]],
		}
		:mapDataP( function()
			curTiming.endTime = KSDate.timeNowForAnimation()
			curTiming.duration = curTiming.endTime - curTiming.start
			timings[ #timings + 1 ] = curTiming
		end )
	end
	
	local function add( )
		extendIds()
		return KSPromises.allP( function( id )
			local size = { 100, 100 }
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = id,
					overlay = {
						height = size[ 2 ],
						width = size[ 1 ],
					}
				}
		end, ids )
	end
	
	local result = WFAsyncTestCase( test, 999 )
	
	for i = 1, 7 do
		
		local expected = function()
			return 'data', { { #ids } }
		end
		result = result
			:waitFor( function()
				
				return add( ids )
	
			end )
			
			:waitFor( simplyQueryToForceTableUpToDateP )
			:waitFor( aspectRatiosWithTempIdsP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( aspectRatiosWithIdConcatenatedSqlP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( aspectRatiosWithTempIdsP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( aspectRatiosWithIdConcatenatedSqlP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( aspectRatiosWithTempIdsP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( aspectRatiosWithIdConcatenatedSqlP )
			:continueWhenValueMatchesFunction( expected )
			:waitFor( function()
				timings[ #timings + 1 ] = "REPEAT"
				local summary
				local cur
				local function printCur()
					if summary then
						print( string.format( "%d\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f", summary.N,
							summary.idConcatenation[ 1 ], summary.idConcatenation[ 2 ],
							summary.idConcatenation[ 3 ], unpack( summary.temp_ids ) ) )
					else
						print( "N", "ID_Concatenation_1", "ID_Concatenation_2", "ID_Concatenation_3",
							"TEMP_ids_1", "TEMP_ids_2", "TEMP_ids_3" )
					end
					summary = { idConcatenation = {}, temp_ids = {} }
				end
				for _, entry in ipairs( timings ) do
					if entry.N then
						if entry.N ~= cur then
							printCur()
						end
						cur = entry.N
						summary.N = entry.N
						local t = summary[ entry.test ]
						t[ #t + 1 ] = entry.duration
					end
				end
				printCur()
				return KSPromises.resolvedPromiseP( 'data', true )
			end )
	
	end
		
end

--------------------------------------------------------------------------------

local function mkIds( n )
	return unpack( mkIdArray( n ) )
end

--------------------------------------------------------------------------------

local function testTempIdsQueryP( test, testBehavior )
	
	local queryFn = testBehavior and 'queryB' or 'queryP'
	
	local KSPromises = require 'KSPromises'
	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )
	
	local id1, id2, id3, id4, id5, id6, id7, id8 = mkIds( 8 )
	local inclIds = { id1, id2, id3, id4 }
	local firstDocs = { id1, id5, id6, id7, id8 }
	local secondDocs = { id2, id3, id4 }
	local sizes = { [ id1 ] = { 100, 150 }, [ id2 ] = { 100, 140 },
		[ id3 ] = { 100, 120 }, [ id4 ] = { 100, 110 }, [ id5 ] = { 100, 90 },
		[ id6 ] = { 100, 80 }, [ id7 ] = { 100, 70 }, [ id8 ] = { 100, 60 }	}
	
	local _aspectRatiosBorP
	local function aspectRatiosBorP()
		if _aspectRatiosBorP and testBehavior then
			return _aspectRatiosBorP
		end
		_aspectRatiosBorP = indexStore[ queryFn ]( indexStore, {
			sql = [[SELECT aspectRatio.docId, aspectRatio FROM aspectRatio
					INNER JOIN TEMP_ids ON aspectRatio.docId == TEMP_ids.docId
						ORDER BY aspectRatio DESC]],
			tempIds = inclIds,
		} )
		return _aspectRatiosBorP
	end
	
	local function matchyMatchy( x )
		if testBehavior then
			return x
		end
		return 'data', x
	end
	
	local function add( ids )
		return KSPromises.allP( function( id )
			local size = sizes[ id ]
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = id,
					overlay = {
						height = size[ 2 ],
						width = size[ 1 ],
					}
				}
		end, ids )
	end

	WFAsyncTestCase( test )

		:waitFor( function()
		
			return add( firstDocs )

		end )
		
		:waitFor( aspectRatiosBorP )
		:continueWhenValueMatches( matchyMatchy( { { id1, 1.5 } } ) )

		:waitFor( function()
		
			return add( secondDocs )

		end )
		
		:waitFor( aspectRatiosBorP )
		:continueWhenValueMatches( matchyMatchy( {
			{ id1, 1.5 },
			{ id2, 1.4 },
			{ id3, 1.2 },
			{ id4, 1.1 },
		} ) )
		
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testTempIdsQueryP( test )
	return testTempIdsQueryP( test, false )
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testTempIdsQueryB( test )
	return testTempIdsQueryP( test, true )
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testReindexSingleDoc( test )

	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )

	local aspectRatiosB = indexStore:queryB(
		[[SELECT docId, aspectRatio FROM aspectRatio
					ORDER BY aspectRatio DESC]] )

	WFAsyncTestCase( test )

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						height = 150,
						width = 100,
					}
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches { { randomDocId, 1.5 } }

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						height = 177,
						width = 100,
					}
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches {
			{ randomDocId, 1.77 },
		}
		
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testIndexRecordsForDeletedDocsGetDeleted( test )

	-- NOTE: There's a possible gap where docs could be deleted (or more
	-- likely, their creation undone). If the index store for any given table
	-- isn't being actively updated, it's possible that deletion might linger
	-- and thus the index store data would be out of date.
	
	-- For the moment, that isn't an issue but once we implement garbage
	-- collection of primary doc store content this could crop up.

	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )

	local aspectRatiosB = indexStore:queryB(
		[[SELECT docId, aspectRatio FROM aspectRatio
					ORDER BY aspectRatio DESC]] )

	WFAsyncTestCase( test )

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						height = 150,
						width = 100,
					},
				
				}
			
		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches { { randomDocId, 1.5 } }

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						_deleted = true,
					},
				
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches {}

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						_deleted = { _delete = true },
						height = 150,
						width = 100,
					},
				
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches { { randomDocId, 1.5 } }

end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testMultipleResultsPerDoc( test )

	indexStore = WFIndexStore( docStore, 'TestKeywords.lua' )

	local keywordsB = indexStore:queryB(
		[[SELECT docId, keyword FROM keywords
					ORDER BY docId, keyword]] )

	WFAsyncTestCase( test )
	
		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						keywords = {
							dog = true,
							cat = true,
						},
					}
				}

		end )
		
		:waitFor( keywordsB )
		:continueWhenValueMatches {
			{ randomDocId, "cat" },
			{ randomDocId, "dog" },
		}

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						keywords = {
							_replace = true,
							dog = true,
							bat = true,
						},
					}
				}

		end )
		
		:waitFor( keywordsB )
		:continueWhenValueMatches {
			{ randomDocId, "bat" },
			{ randomDocId, "dog" },
		}
		
		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						keywords = {
							_delete = true,
						},
					}
				}

		end )
		
		:waitFor( keywordsB )
		:continueWhenValueMatches {}
		
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testDoesntSendRedundantUpdates( test )

	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )

	local aspectRatiosB = indexStore:queryB(
		[[SELECT docId, aspectRatio FROM aspectRatio
					ORDER BY aspectRatio DESC]] )

	local updateCount = 0
	local radioSilenceExpected = false

	local updateDetectorB = aspectRatiosB:mapB( function( data )
		debug.print "remapping aspectRatio data"
		updateCount = updateCount + 1
		return updateCount
	end )

	local onDataTrigger

	WFAsyncTestCase( test, 5 )

		:waitFor( function()
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						height = 150,
						width = 100,
					}
				}

		end )
		
		:waitFor( aspectRatiosB )
		:continueWhenValueMatches { { randomDocId, 1.5 } }

		:waitFor( .1 )
			-- Hack around a race condition: The updateDetectorB might
			-- be triggered before or after this.

		:waitFor( function()

			onDataTrigger = aspectRatiosB:ownedOnDataE( test, function()
				if radioSilenceExpected then
					debug.print "got unwanted onDataE callback!"
					updateCount = updateCount + 1
				end
			end )

			updateCount = 0
			radioSilenceExpected = true
			
			-- debug.print( "-------------------------------------------- should be radio silence after this point --------------------------------------------" )
		
			return docStore:applyOverlayToDocumentP
				{
					docType = 'asset',
					docId = randomDocId,
					overlay = {
						foo = 'bar',
							-- not part of aspecRatio calculation,
							-- should not cause index update
					}
				}

		end )
		
		:waitFor( .75 )
		:whenResolvedDo( function()
			
			-- [scouten 2011-04-08]: I don't really expect either
			-- of these conditions to ever happen, but I'm using this
			-- to keep the two triggers from being GC'd.

			if not onDataTrigger or not updateDetectorB then
				error "huh"
			end
			
			if updateCount > 0 then
				error "Redundant update notification sent!"
			end
			
		end )
		
end

--------------------------------------------------------------------------------

function WFIndexStore_OzDocStore_Test.testIndexingProgress( test )
	indexStore = WFIndexStore( docStore, 'TestAspectRatio.lua' )

	local function pollUntilP( expected, ... )
		local args = { ... }
		local argN = select( '#', ... )
		return indexStore:shouldWaitForIndexingP( ... )
		:mapDataP_new( function( value )
			if value == expected then
				return value
			end
			return pollUntilP( expected, unpack( args, 1, argN ) )
		end )
	end

	WFAsyncTestCase( test, 10 )

		-- get caught up
		:waitFor( function()
			return indexStore:indexingProgressB()
		end )
		:continueWhenValueMatches( 1 )

		-- check that we're all caught up
		:waitFor( function()
			return pollUntilP( false )
		end )
		:continueWhenValueMatches( 'data', false )
		
		-- add a bunch of docs
		:waitFor( function()
			for i = 1, 188 do
				docStore:applyOverlayToDocumentP
					{
						docType = 'asset',
						docId = KSIdGenerator.uuid(),
						overlay = {
							height = 150,
							width = 100,
						}
					}
			end
			return true
		end )

		-- wait until we realize we're behind
		:waitFor( function()
			return pollUntilP( true )
		end )
		:continueWhenValueMatches( 'data', true )
		
		-- confirm we know we're not this far behind
		:waitFor( function()
			return pollUntilP( false, 200 )
		end )
		:continueWhenValueMatches( 'data', false )

		-- but we are at least this far behind
		:waitFor( function()
			return pollUntilP( true, 150 )
		end )
		:continueWhenValueMatches( 'data', true )
		
		-- get caught up and track progress
		:waitFor( function()
			return indexStore:indexingProgressB()
		end )
		:continueWhenValueMatches( 1 )
		
		-- check that we're still totally up to date
		:waitFor( function()
			return pollUntilP( false )
		end )
		:continueWhenValueMatches( 'data', false )
		
		-- in this state,
		:waitFor( function()
			return indexStore:indexingProgressB()
		end )
		:continueWhenValueMatches( 1 )

end

--------------------------------------------------------------------------------

return WFIndexStore_OzDocStore_Test
