--[[----------------------------------------------------------------------------

WFMediaCoreClipCache.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law. Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.os
local WFPlatformInfo = require 'WFPlatformInfo'

	-- WF.core.rxlua
local WFRxNativeObservable = require 'WFRxNativeObservable'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFCloseSupport = require 'WFCloseSupport'

	-- WF.mediacore
local WFMediaCoreContextImp = require 'WFMediaCoreContextImp'


--------------------------------------------------------------------------------

assert( WFPlatformInfo.hasMediaCore, "MediaCore integration not enabled" )

local check = assert( WFArguments.check )

--============================================================================--

-- @mark WFMediaCoreClipCache class

--[[!
	\brief WFMediaCoreClipCache 
	
]]--

local WFMediaCoreClipCache = {

	_imp = false,
		-- (WFMediaCoreClipCacheImp)

	WFCloseSupport.addCloseSupportMethods,

}

--------------------------------------------------------------------------------
--[[!
	\brief 
]]--

function WFMediaCoreClipCache:__init( filePath )

	check( 'WFMediaCoreContext', '+string' )

end

--------------------------------------------------------------------------------

local function assertIfClosed( self )

	if WFCloseSupport.hasBeenOrWillBeClosed( self ) then
		error( "WFMediaCoreClipCache can not be used after it has been closed" )
	end

end

--------------------------------------------------------------------------------
--[[!
	\brief 

]]--

function WFMediaCoreClipCache:getClip( filePath )

	check( 'WFMediaCoreClipCache' )

	assertIfClosed( self )

	return WFRxNativeObservable( WFMediaCoreContextImp.readMetadata, self._imp )

end

--------------------------------------------------------------------------------

function WFMediaCoreClipCache:close()

	check( 'WFMediaCoreClipCache' )

	if self._imp then
		self._imp:close()
	end

	self._imp = nil

end

--------------------------------------------------------------------------------

return Rx.Class 'WFMediaCoreClipCache'( WFMediaCoreClipCache )
