--[[----------------------------------------------------------------------------

WFOzBinaryUploadUtilsTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.reactive
local KSPromises = require 'KSPromises'

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.sync_engine.binary_upload
local WFOzBinaryUploadUtils = require 'WFOzBinaryUploadUtils'


--============================================================================--

local WFOzBinaryUploadUtilsTest = {}

--------------------------------------------------------------------------------

local function testShouldUploadRendition( testName, renditionType,
										  expectedResult, expectedNotes )

	return function( docValue )
	
		WFOzBinaryUploadUtilsTest[ 'testShouldUploadRendition_' .. testName ] =
			function( test )
			
				WFAsyncTestCase( test )
					:waitFor( function()
					
						local result, notes
							= WFOzBinaryUploadUtils.shouldUploadRendition( docValue,
																		   renditionType )

						return KSPromises.resolvedPromiseP( 'data', result, notes )
					
					end )
					:continueWhenValueMatches( 'data', expectedResult, expectedNotes )
			
			end
	
	end

end

--------------------------------------------------------------------------------

testShouldUploadRendition( 'notAsset', 'thumbnail2x', false, 'notAsset' )
{
	_changeCounter = 3,
	_id = "8cca1eed95d04e48a247516feef79eaf",
	created = "2013-06-12T14:07:21.153069Z",
	created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
	id = "urn:revel:catalog:64007543a06d48d1b4d104be45866f2e:album:8cca1eed95d04e48a247516feef79eaf",
	links = {
		["/rels/album_assets"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf/assets",
		},
		["self"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf",
		},
	},
	name = "lightroom-mobile-album",
	type = "album",
	updated = "2013-06-12T14:07:21.153069Z",
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'metadataNotUploaded', 'thumbnail2x', false, 'metadataNotUploaded' )
{
	_changeCounter = 16,
	_id = "824e539b49e949269e97fca2aa217497",
	_localOnly = {
		localUrl = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 3694104,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/3793F7F3-8D1A-4585-8251-69AB4568F991/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2012/2012-07/2012-07-17/IMG_0003.JPG",
		preview = {
			epoch = 1,
			md5 = "e6c200070795b2ef1f7485992ce72c68",
			revId = "pending",
			size = 416769,
		},
		thumbnail = {
			epoch = 1,
			md5 = "f5544b0c88c5c18616c2433507233132",
			revId = "pending",
			size = 6171,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fad1884d91b762eb8d20e5360394bc2d",
			revId = "pending",
			size = 19372,
		},
	},
	captureDate = "2012-07-17T15:07:19",
	changedAtTime = "2013-06-13T09:40:30.442",
	changedOnDevice = "iPad Simulator",
	height = 2880,
	importSource = {
		fileCreationDate = "2012-07-17T15:07:19",
		fileName = "IMG_0003.JPG",
		filePath = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		fileSize = 3694104,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T09:40:30.404",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "0b1d3b54ca3dc7a2aab5ec754589644bc25cbe0e910dbae3f6a714ab517eafc4",
	},
	originalCaptureDate = "2012-07-17T15:07:19",
	type = "asset",
	version = 1,
	width = 2880,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemMissingLocally', 'thumbnail2x', false, 'itemMissingLocally' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
		renditions = {
			['thumbnail2x'] = {
				missing = true,
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemMissingLocally', 'thumbnail2x', false, 'itemFailedToUpload' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
		renditions = {
			['thumbnail2x'] = {
				error = "I dunno, I just couldn't upload it",
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemNotUploaded', 'thumbnail2x', true, 'itemNotUploaded' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemIsInvalid', 'thumbnail2x', true, 'itemIsInvalid' )
{
	_changeCounter = 108,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			doNotBackUpFlagSet = true,
			epoch = 2,
			isInCacheFolder = true,
			ozUrl = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/master",
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Library/Caches/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 2,
			md5 = "8a0d69b82534096635a2a05f27f490a9",
			referenceHeight = 960,
			referenceWidth = 640,
			renditionId = "88c6d23290674f30ab08771d4186ae70",
			revId = "aae62a3d51794f61ab6d950a2fa4a603",
			size = 70780,
		},
		thumbnail = {
			epoch = 2,
			md5 = "5c285a4b26bb48b7380fb122a5afc547",
			referenceHeight = 960,
			referenceWidth = 640,
			renditionId = "36f7761b2f6d4629810718debf531cb7",
			revId = "aae62a3d51794f61ab6d950a2fa4a603",
			size = 6005,
		},
		thumbnail2x = {
			epoch = 2,
			md5 = "f0de85c0740ea208db0b75abb50d5261",
			referenceHeight = 960,
			referenceWidth = 640,
			renditionId = "854438b8989b4505889f2c9a99d65054",
			revId = "aae62a3d51794f61ab6d950a2fa4a603",
			size = 14409,
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603",
			},
			["/rels/master"] = {
				content_type = "image/jpeg",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/master",
			},
			["/rels/rendition_type/2048"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/renditions/88c6d23290674f30ab08771d4186ae70",
				invalid = true,
			},
			["/rels/rendition_type/thumbnail2x"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/renditions/854438b8989b4505889f2c9a99d65054",
				invalid = true,
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:15:48.489584Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:15:47.990",
	changedOnDevice = "iPad Simulator",
	develop = {
		croppedHeight = 960,
		croppedWidth = 640,
		userOrientation = 1,
		xmpBase = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:LensProfileSetup=\"LensDefaults\"\
   crs:HasSettings=\"True\"\
   crs:AlreadyApplied=\"False\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
		xmpLook = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:IncrementalTemperature=\"+23\"\
   crs:IncrementalTint=\"-11\"\
   crs:Vibrance=\"+32\"\
   crs:SplitToningShadowHue=\"240\"\
   crs:SplitToningShadowSaturation=\"99\"\
   crs:SplitToningHighlightHue=\"63\"\
   crs:SplitToningHighlightSaturation=\"75\"\
   crs:SplitToningBalance=\"0\"\
   crs:ParametricShadows=\"0\"\
   crs:ParametricDarks=\"-7\"\
   crs:ParametricLights=\"+28\"\
   crs:ParametricHighlights=\"-26\"\
   crs:ParametricShadowSplit=\"25\"\
   crs:ParametricMidtoneSplit=\"50\"\
   crs:ParametricHighlightSplit=\"75\"\
   crs:Exposure2012=\"-0.55\"\
   crs:Contrast2012=\"+52\"\
   crs:Shadows2012=\"+60\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:HasSettings=\"True\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
	},
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemIsCurrent', 'thumbnail2x', false, 'itemIsCurrent' )
{
	_changeCounter = 5,
	_id = "de297d9f8b154ead8573879e4d42706b",
	_serverProvidedMetadata = {
		created = "2013-06-12T14:07:21.805773Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f75e86959270b3e73ab0c8e09a0db90f/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f6de508626f74f1790211e047e77f172/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f6de508626f74f1790211e047e77f172",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f75e86959270b3e73ab0c8e09a0db90f",
			},
			["/rels/master"] = {
				content_type = "application/octet-stream",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f6de508626f74f1790211e047e77f172/master",
			},
			["/rels/rendition_type/2048"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f6de508626f74f1790211e047e77f172/renditions/6fd7202d8d45437c9ecbb0d371dff5d3",
			},
			["/rels/rendition_type/thumbnail2x"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b/revisions/f6de508626f74f1790211e047e77f172/renditions/727e1591d671441cbed12d74c7f18f1a",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/de297d9f8b154ead8573879e4d42706b",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-12T14:07:24.920755Z",
	},
	captureDate = "2013-06-11T13:09:53",
	develop = {
		croppedHeight = 1905,
		croppedWidth = 2540,
		userOrientation = 6,
		xmpBase = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45        \">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:tiff=\"http://ns.adobe.com/tiff/1.0/\"\
    xmlns:exif=\"http://ns.adobe.com/exif/1.0/\"\
    xmlns:xmp=\"http://ns.adobe.com/xap/1.0/\"\
    xmlns:photoshop=\"http://ns.adobe.com/photoshop/1.0/\"\
    xmlns:xmpMM=\"http://ns.adobe.com/xap/1.0/mm/\"\
    xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\
   tiff:Make=\"Apple\"\
   tiff:Model=\"iPhone 4S\"\
   tiff:Orientation=\"6\"\
   tiff:ImageWidth=\"3264\"\
   tiff:ImageLength=\"2448\"\
   exif:ExifVersion=\"0221\"\
   exif:ExposureTime=\"1/800\"\
   exif:ShutterSpeedValue=\"9643856/1000000\"\
   exif:FNumber=\"24/10\"\
   exif:ApertureValue=\"2526069/1000000\"\
   exif:ExposureProgram=\"2\"\
   exif:BrightnessValue=\"6348/709\"\
   exif:MeteringMode=\"5\"\
   exif:FocalLength=\"107/25\"\
   exif:SensingMethod=\"2\"\
   exif:FocalLengthIn35mmFilm=\"35\"\
   exif:ExposureMode=\"0\"\
   exif:WhiteBalance=\"0\"\
   exif:SceneCaptureType=\"0\"\
   exif:GPSLatitude=\"53,32.84N\"\
   exif:GPSLongitude=\"9,56.19E\"\
   exif:GPSAltitudeRef=\"0\"\
   exif:GPSAltitude=\"38/1\"\
   exif:GPSTimeStamp=\"11:09:52.18Z\"\
   exif:GPSImgDirectionRef=\"T\"\
   exif:GPSImgDirection=\"35431/238\"\
   exif:DateTimeOriginal=\"2013-06-11T13:09:53\"\
   exif:PixelXDimension=\"3264\"\
   exif:PixelYDimension=\"2448\"\
   xmp:CreatorTool=\"6.1.3\"\
   xmp:ModifyDate=\"2013-06-11T13:09:53\"\
   xmp:CreateDate=\"2013-06-11T13:09:53\"\
   photoshop:DateCreated=\"2013-06-11T13:09:53\"\
   xmpMM:DocumentID=\"A8058BAE9061C6E03FC4597E7DB7CA70\"\
   dc:format=\"image/jpeg\">\
   <exif:ISOSpeedRatings>\
    <rdf:Seq>\
     <rdf:li>50</rdf:li>\
    </rdf:Seq>\
   </exif:ISOSpeedRatings>\
   <exif:Flash\
    exif:Fired=\"False\"\
    exif:Return=\"0\"\
    exif:Mode=\"3\"\
    exif:Function=\"False\"\
    exif:RedEyeMode=\"False\"/>\
   <exif:SubjectArea>\
    <rdf:Seq>\
     <rdf:li>1631</rdf:li>\
     <rdf:li>1223</rdf:li>\
     <rdf:li>881</rdf:li>\
     <rdf:li>881</rdf:li>\
    </rdf:Seq>\
   </exif:SubjectArea>\
  </rdf:Description>\
 </rdf:RDF>\
</x:xmpmeta>\
",
	},
	height = 2448,
	importSource = {
		fileCreationDate = "2013-06-11T13:09:53",
		fileName = "es-0001-001-2.jpg",
		filePath = "/Users/foo/Dropbox/Pictures/Edit Queue/Edit Stage 1 - Basic Metadata/2013-06-11/es-0001-001-2.jpg",
		fileSize = 992646,
		folder = "/Users/foo/Dropbox/Pictures/Edit Queue/Edit Stage 1 - Basic Metadata/2013-06-11",
		folderPath = "/Users/foo/Dropbox/Pictures/Edit Queue/Edit Stage 1 - Basic Metadata/2013-06-11",
		importTimestamp = "2013-06-12T14:07:17",
		importedBy = "importedBy_LR",
		importedOnDevice = "importedOnDevice_LR_computer",
		sha256 = "4ed805a8013f305c037529b15f1de47c5c35e0f3d633ce9251c7d590fac790e7",
	},
	location = {
		altitude = 38,
		latitude = 53.547333,
		longitude = 9.9365,
	},
	type = "asset",
	version = 1,
	width = 3264,
}

--------------------------------------------------------------------------------

testShouldUploadRendition( 'itemSyncStatusError', 'thumbnail2x', false, 'itemSyncStatusError' )
{
	_changeCounter = 108,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_syncStatus = {
		lightroom = {
			accepted = "2014-07-07T01:37:44.278Z",
			timestamp = 57480,
		},
		oz = {
			error = {
				message = "Upload rejected",
				resultCode = "1005",
				resultData = {
					code = 1005,
					description = "Input Validation Error",
					errors = {
						parent_revision_id = {
							"must be the head revision",
						},
					},
				},
				retry = "2014-07-31T21:32:54.751Z",
				time = "2014-07-31T21:32:24.376Z",
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:15:48.489584Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:15:47.990",
	changedOnDevice = "iPad Simulator",
	develop = {
		croppedHeight = 960,
		croppedWidth = 640,
		userOrientation = 1,
		xmpBase = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:LensProfileSetup=\"LensDefaults\"\
   crs:HasSettings=\"True\"\
   crs:AlreadyApplied=\"False\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
		xmpLook = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:IncrementalTemperature=\"+23\"\
   crs:IncrementalTint=\"-11\"\
   crs:Vibrance=\"+32\"\
   crs:SplitToningShadowHue=\"240\"\
   crs:SplitToningShadowSaturation=\"99\"\
   crs:SplitToningHighlightHue=\"63\"\
   crs:SplitToningHighlightSaturation=\"75\"\
   crs:SplitToningBalance=\"0\"\
   crs:ParametricShadows=\"0\"\
   crs:ParametricDarks=\"-7\"\
   crs:ParametricLights=\"+28\"\
   crs:ParametricHighlights=\"-26\"\
   crs:ParametricShadowSplit=\"25\"\
   crs:ParametricMidtoneSplit=\"50\"\
   crs:ParametricHighlightSplit=\"75\"\
   crs:Exposure2012=\"-0.55\"\
   crs:Contrast2012=\"+52\"\
   crs:Shadows2012=\"+60\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:HasSettings=\"True\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
	},
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

local function testShouldUploadProxy( testName, renditionType,
									  expectedResult, expectedNotes )

	return function( docValue )
	
		WFOzBinaryUploadUtilsTest[ 'testShouldUploadProxy_' .. testName ] =
			function( test )
			
				WFAsyncTestCase( test )
					:waitFor( function()
					
						local result, notes
							= WFOzBinaryUploadUtils.shouldUploadProxy( docValue,
																	   renditionType )

						return KSPromises.resolvedPromiseP( 'data', result, notes )
					
					end )
					:continueWhenValueMatches( 'data', expectedResult, expectedNotes )
			
			end
	
	end

end

--------------------------------------------------------------------------------

testShouldUploadProxy( 'notAsset', '2560', false, 'notAsset' )
{
	_changeCounter = 3,
	_id = "8cca1eed95d04e48a247516feef79eaf",
	created = "2013-06-12T14:07:21.153069Z",
	created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
	id = "urn:revel:catalog:64007543a06d48d1b4d104be45866f2e:album:8cca1eed95d04e48a247516feef79eaf",
	links = {
		["/rels/album_assets"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf/assets",
		},
		["self"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf",
		},
	},
	name = "lightroom-mobile-album",
	type = "album",
	updated = "2013-06-12T14:07:21.153069Z",
}

--------------------------------------------------------------------------------

testShouldUploadProxy( 'metadataNotUploaded', '2560', false, 'metadataNotUploaded' )
{
	_changeCounter = 16,
	_id = "824e539b49e949269e97fca2aa217497",
	_localOnly = {
		localUrl = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 3694104,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/3793F7F3-8D1A-4585-8251-69AB4568F991/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2012/2012-07/2012-07-17/IMG_0003.JPG",
		preview = {
			epoch = 1,
			md5 = "e6c200070795b2ef1f7485992ce72c68",
			revId = "pending",
			size = 416769,
		},
		thumbnail = {
			epoch = 1,
			md5 = "f5544b0c88c5c18616c2433507233132",
			revId = "pending",
			size = 6171,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fad1884d91b762eb8d20e5360394bc2d",
			revId = "pending",
			size = 19372,
		},
	},
	captureDate = "2012-07-17T15:07:19",
	changedAtTime = "2013-06-13T09:40:30.442",
	changedOnDevice = "iPad Simulator",
	height = 2880,
	importSource = {
		fileCreationDate = "2012-07-17T15:07:19",
		fileName = "IMG_0003.JPG",
		filePath = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		fileSize = 3694104,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T09:40:30.404",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "0b1d3b54ca3dc7a2aab5ec754589644bc25cbe0e910dbae3f6a714ab517eafc4",
	},
	originalCaptureDate = "2012-07-17T15:07:19",
	type = "asset",
	version = 1,
	width = 2880,
}

--------------------------------------------------------------------------------

testShouldUploadProxy( 'itemMissingLocally', '2560', false, 'itemMissingLocally' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
		proxy = {
			['2560'] = {
				missing = true,
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadProxy( 'itemNotUploaded', '2560', true, 'itemNotUploaded' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadProxy( 'itemIsCurrent', '2560', false, 'itemIsCurrent' )
{
	_changeCounter = 91,
	_id = "14de91f974fd404e989aadf1a23a65d5",
	_serverProvidedMetadata = {
		created = "2012-03-07T21:52:56.276321Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/e97f4143664e4e0483b0854113d3b156/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:ff710c5ecca7439fbb00d3c909a713cb",
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/e97f4143664e4e0483b0854113d3b156",
			},
			["/rels/master"] = {
				content_type = "image/jpeg",
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/master",
			},
			["/rels/proxy_type/2560"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/proxies/4a6f4b0ed70d4054a54918848d4a9adb",
			},
			["/rels/rendition_type/2048"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/renditions/f5a6d15f22854608bb9187d594972382",
			},
			["/rels/rendition_type/thumbnail2x"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/renditions/9f8e3507d17a43638cf70988d442b13b",
			},
			["self"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2012-03-23T17:48:25.020973Z",
	},
	captureDate = "2012-03-07T12:12:28",
	changedAtTime = "2012-03-07T13:52:57.436",
	changedOnDevice = "erwright-mbp",
	eventName = "Revel Seattle Photo Walk",
	eventTimestamp = "2012-03-07T22:16:06.729Z",
	height = 3744,
	importSource = {
		fileName = "IMG_6689.JPG",
		fileSize = 5770939,
		folder = "/Users/erwright/Pictures/Photowalk/",
		importTimestamp = "2012-03-07T13:52:57.329",
		importedBy = "3936db2bcc8848d09148b4c6ca826d59",
		importedOnDevice = "erwright-mbp",
		sha256 = "d2631c77f52f494ea79c37385bafa9a3e709e3d0d75c453bf3ad870acf52ef64",
	},
	keywords = {
		Photowalk = true,
	},
	originalCaptureDate = "2012-03-07T12:12:28",
	type = "asset",
	version = 1,
	width = 5616,
}

--------------------------------------------------------------------------------

testShouldUploadProxy( 'itemSyncStatusError', '2560', false, 'itemSyncStatusError' )
{
	_changeCounter = 108,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_syncStatus = {
		lightroom = {
			accepted = "2014-07-07T01:37:44.278Z",
			timestamp = 57480,
		},
		oz = {
			error = {
				message = "Upload rejected",
				resultCode = "1005",
				resultData = {
					code = 1005,
					description = "Input Validation Error",
					errors = {
						parent_revision_id = {
							"must be the head revision",
						},
					},
				},
				retry = "2014-07-31T21:32:54.751Z",
				time = "2014-07-31T21:32:24.376Z",
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:15:48.489584Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:15:47.990",
	changedOnDevice = "iPad Simulator",
	develop = {
		croppedHeight = 960,
		croppedWidth = 640,
		userOrientation = 1,
		xmpBase = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:LensProfileSetup=\"LensDefaults\"\
   crs:HasSettings=\"True\"\
   crs:AlreadyApplied=\"False\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
		xmpLook = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:IncrementalTemperature=\"+23\"\
   crs:IncrementalTint=\"-11\"\
   crs:Vibrance=\"+32\"\
   crs:SplitToningShadowHue=\"240\"\
   crs:SplitToningShadowSaturation=\"99\"\
   crs:SplitToningHighlightHue=\"63\"\
   crs:SplitToningHighlightSaturation=\"75\"\
   crs:SplitToningBalance=\"0\"\
   crs:ParametricShadows=\"0\"\
   crs:ParametricDarks=\"-7\"\
   crs:ParametricLights=\"+28\"\
   crs:ParametricHighlights=\"-26\"\
   crs:ParametricShadowSplit=\"25\"\
   crs:ParametricMidtoneSplit=\"50\"\
   crs:ParametricHighlightSplit=\"75\"\
   crs:Exposure2012=\"-0.55\"\
   crs:Contrast2012=\"+52\"\
   crs:Shadows2012=\"+60\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:HasSettings=\"True\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
	},
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

local function testShouldUploadMaster( testName, expectedResult, expectedNotes )

	return function( docValue )
	
		WFOzBinaryUploadUtilsTest[ 'testShouldUploadMaster_' .. testName ] =
			function( test )
			
				WFAsyncTestCase( test )
					:waitFor( function()
					
						local result, notes
							= WFOzBinaryUploadUtils.shouldUploadMaster( docValue )

						return KSPromises.resolvedPromiseP( 'data', result, notes )
					
					end )
					:continueWhenValueMatches( 'data', expectedResult, expectedNotes )
			
			end
	
	end

end

--------------------------------------------------------------------------------

testShouldUploadMaster( 'notAsset', false, 'notAsset' )
{
	_changeCounter = 3,
	_id = "8cca1eed95d04e48a247516feef79eaf",
	created = "2013-06-12T14:07:21.153069Z",
	created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
	id = "urn:revel:catalog:64007543a06d48d1b4d104be45866f2e:album:8cca1eed95d04e48a247516feef79eaf",
	links = {
		["/rels/album_assets"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf/assets",
		},
		["self"] = {
			href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/albums/8cca1eed95d04e48a247516feef79eaf",
		},
	},
	name = "lightroom-mobile-album",
	type = "album",
	updated = "2013-06-12T14:07:21.153069Z",
}

--------------------------------------------------------------------------------

testShouldUploadMaster( 'metadataNotUploaded', false, 'metadataNotUploaded' )
{
	_changeCounter = 16,
	_id = "824e539b49e949269e97fca2aa217497",
	_localOnly = {
		localUrl = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 3694104,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/3793F7F3-8D1A-4585-8251-69AB4568F991/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2012/2012-07/2012-07-17/IMG_0003.JPG",
		preview = {
			epoch = 1,
			md5 = "e6c200070795b2ef1f7485992ce72c68",
			revId = "pending",
			size = 416769,
		},
		thumbnail = {
			epoch = 1,
			md5 = "f5544b0c88c5c18616c2433507233132",
			revId = "pending",
			size = 6171,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fad1884d91b762eb8d20e5360394bc2d",
			revId = "pending",
			size = 19372,
		},
	},
	captureDate = "2012-07-17T15:07:19",
	changedAtTime = "2013-06-13T09:40:30.442",
	changedOnDevice = "iPad Simulator",
	height = 2880,
	importSource = {
		fileCreationDate = "2012-07-17T15:07:19",
		fileName = "IMG_0003.JPG",
		filePath = "assets-library://asset/asset.JPG?id=9BF2DA60-E7E7-4513-9401-9C3BD8D4F0C5&ext=JPG",
		fileSize = 3694104,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T09:40:30.404",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "0b1d3b54ca3dc7a2aab5ec754589644bc25cbe0e910dbae3f6a714ab517eafc4",
	},
	originalCaptureDate = "2012-07-17T15:07:19",
	type = "asset",
	version = 1,
	width = 2880,
}

--------------------------------------------------------------------------------

testShouldUploadMaster( 'itemMissingLocally', false, 'itemMissingLocally' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
		master = {
			missing = true,
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadMaster( 'itemNotUploaded', true, 'itemNotUploaded' )
{
	_changeCounter = 34,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_localOnly = {
		devSettingsLastSetAtRev = "331513478d5b4f9ab5d1fa11fdc8c265",
		localUrl = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		original = {
			epoch = 1,
			isInCacheFolder = false,
			size = 43095,
		},
		originalFilePath = "/Users/foo/Library/Application Support/iPhone Simulator/6.1/Applications/CA8600CF-FB2C-4FBD-B41B-6F0123C471D3/Documents/64007543a06d48d1b4d104be45866f2e/Originals/2013/2013-06/2013-06-13/IMG_0001.JPG",
		preview = {
			epoch = 1,
			md5 = "894cff4a930b1e772bbaf9e810c94c3d",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 39977,
		},
		thumbnail = {
			epoch = 1,
			md5 = "19937b6868b01848b4db97de79a5cb50",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 3627,
		},
		thumbnail2x = {
			epoch = 1,
			md5 = "fe6cd376b07bbb9e7e4ebb32390f70ac",
			revId = "331513478d5b4f9ab5d1fa11fdc8c265",
			size = 8516,
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/children/{revision_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:07:09.468690Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:07:06.740",
	changedOnDevice = "iPad Simulator",
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

testShouldUploadMaster( 'itemIsCurrent', false, 'itemIsCurrent' )
{
	_changeCounter = 91,
	_id = "14de91f974fd404e989aadf1a23a65d5",
	_serverProvidedMetadata = {
		created = "2012-03-07T21:52:56.276321Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/e97f4143664e4e0483b0854113d3b156/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:ff710c5ecca7439fbb00d3c909a713cb",
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/e97f4143664e4e0483b0854113d3b156",
			},
			["/rels/master"] = {
				content_type = "image/jpeg",
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/master",
			},
			["/rels/proxy_type/2560"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/proxies/4a6f4b0ed70d4054a54918848d4a9adb",
			},
			["/rels/rendition_type/2048"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/renditions/f5a6d15f22854608bb9187d594972382",
			},
			["/rels/rendition_type/thumbnail2x"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5/revisions/70e33f72d1ab4b94885399c4717443fc/renditions/9f8e3507d17a43638cf70988d442b13b",
			},
			["self"] = {
				href = "/v1.0/catalogs/fa6be48bfab74784a509b78bbbdb7232/assets/14de91f974fd404e989aadf1a23a65d5",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2012-03-23T17:48:25.020973Z",
	},
	captureDate = "2012-03-07T12:12:28",
	changedAtTime = "2012-03-07T13:52:57.436",
	changedOnDevice = "erwright-mbp",
	eventName = "Revel Seattle Photo Walk",
	eventTimestamp = "2012-03-07T22:16:06.729Z",
	height = 3744,
	importSource = {
		fileName = "IMG_6689.JPG",
		fileSize = 5770939,
		folder = "/Users/erwright/Pictures/Photowalk/",
		importTimestamp = "2012-03-07T13:52:57.329",
		importedBy = "3936db2bcc8848d09148b4c6ca826d59",
		importedOnDevice = "erwright-mbp",
		sha256 = "d2631c77f52f494ea79c37385bafa9a3e709e3d0d75c453bf3ad870acf52ef64",
	},
	keywords = {
		Photowalk = true,
	},
	originalCaptureDate = "2012-03-07T12:12:28",
	type = "asset",
	version = 1,
	width = 5616,
}

--------------------------------------------------------------------------------

testShouldUploadMaster( 'itemSyncStatusError', false, 'itemSyncStatusError' )
{
	_changeCounter = 108,
	_id = "8caa3dafe3c44d639edbd43baddc384e",
	_syncStatus = {
		lightroom = {
			accepted = "2014-07-07T01:37:44.278Z",
			timestamp = 57480,
		},
		oz = {
			error = {
				message = "Upload rejected",
				resultCode = "1005",
				resultData = {
					code = 1005,
					description = "Input Validation Error",
					errors = {
						parent_revision_id = {
							"must be the head revision",
						},
					},
				},
				retry = "2014-07-31T21:32:54.751Z",
				time = "2014-07-31T21:32:24.376Z",
			},
		},
	},
	_serverProvidedMetadata = {
		created = "2013-06-13T14:07:09.468690Z",
		link_templates = {
			["/rels/child_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603/children/{revision_id}",
			},
			["/rels/proxy_generate"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265/proxies/{proxy_id}",
			},
		},
		links = {
			["/rels/first_revision"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/331513478d5b4f9ab5d1fa11fdc8c265",
			},
			["/rels/latest_revision"] = {
				created_by = "urn:carousel:account:3c3edb640d7243068074d029e4d62c72",
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e/revisions/aae62a3d51794f61ab6d950a2fa4a603",
			},
			["self"] = {
				href = "/v1.0/catalogs/64007543a06d48d1b4d104be45866f2e/assets/8caa3dafe3c44d639edbd43baddc384e",
			},
		},
		subtype = "jpeg",
		type = "asset",
		updated = "2013-06-13T14:15:48.489584Z",
	},
	captureDate = "2013-06-13T09:37:50",
	changedAtTime = "2013-06-13T16:15:47.990",
	changedOnDevice = "iPad Simulator",
	develop = {
		croppedHeight = 960,
		croppedWidth = 640,
		userOrientation = 1,
		xmpBase = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:LensProfileSetup=\"LensDefaults\"\
   crs:HasSettings=\"True\"\
   crs:AlreadyApplied=\"False\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
		xmpLook = "<x:xmpmeta xmlns:x=\"adobe:ns:meta/\" x:xmptk=\"Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)\">\
 <rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\
  <rdf:Description rdf:about=\"\"\
    xmlns:crs=\"http://ns.adobe.com/camera-raw-settings/1.0/\"\
   crs:Version=\"7.4\"\
   crs:ProcessVersion=\"6.7\"\
   crs:IncrementalTemperature=\"+23\"\
   crs:IncrementalTint=\"-11\"\
   crs:Vibrance=\"+32\"\
   crs:SplitToningShadowHue=\"240\"\
   crs:SplitToningShadowSaturation=\"99\"\
   crs:SplitToningHighlightHue=\"63\"\
   crs:SplitToningHighlightSaturation=\"75\"\
   crs:SplitToningBalance=\"0\"\
   crs:ParametricShadows=\"0\"\
   crs:ParametricDarks=\"-7\"\
   crs:ParametricLights=\"+28\"\
   crs:ParametricHighlights=\"-26\"\
   crs:ParametricShadowSplit=\"25\"\
   crs:ParametricMidtoneSplit=\"50\"\
   crs:ParametricHighlightSplit=\"75\"\
   crs:Exposure2012=\"-0.55\"\
   crs:Contrast2012=\"+52\"\
   crs:Shadows2012=\"+60\"\
   crs:ConvertToGrayscale=\"False\"\
   crs:HasSettings=\"True\"/>\
 </rdf:RDF>\
</x:xmpmeta>\
",
	},
	height = 960,
	importSource = {
		fileCreationDate = "2013-06-13T09:37:50",
		fileName = "IMG_0001.JPG",
		filePath = "assets-library://asset/asset.JPG?id=DC4B230D-7B1C-474C-A44A-AA55F4BBBE02&ext=JPG",
		fileSize = 43095,
		folder = "assets-library://asset",
		folderPath = "assets-library://asset",
		importTimestamp = "2013-06-13T16:07:06.717",
		importedBy = "3c3edb640d7243068074d029e4d62c72",
		importedOnDevice = "iPad Simulator",
		sha256 = "010cf15ed22112e47ff16adbaa469c128095b4d47b1d0dd9d4fc1c3c50c3d492",
	},
	originalCaptureDate = "2013-06-13T09:37:50",
	type = "asset",
	version = 1,
	width = 640,
}

--------------------------------------------------------------------------------

local WFPlatformInfo = require 'WFPlatformInfo'

if WFPlatformInfo.isWindows then
	return {}
else
	return WFOzBinaryUploadUtilsTest
end
