--[[----------------------------------------------------------------------------

WFOzCatalog.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local _ = require 'WFArguments_uuid'
local WFCloseSupport = require 'WFCloseSupport'
local WFObjectProxy = require 'WFObjectProxy'

	-- WF.oz_client.doc_store
local WFDocStore = require 'WFDocStore'

	-- WF.oz_client.index_store
local WFIndexStore = require 'WFIndexStore'


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--

-- @mark WFOzCatalog class

local catalogMethods = {

	_ozCatalogSpec = false,
		-- (WFOzCatalogSpec) specification for the catalog user and catalog ID
	
	_docStore = false,
		-- (WFDocStore)
	
	_indexStore = false,
		-- (WFIndexStore)

	WFCloseSupport.addCloseSupportMethods,

}

--------------------------------------------------------------------------------
--[[!
	\brief Create a WFOzCatalog.
	
	\param ozCatalogSpec (WFOzCatalogSpec) specification for the catalog user and catalog ID
	\param optArgs (string) optional args for doc store/index store creation
		- docStorePath 	  -- path to doc store
		- indexSpecScript -- name of script to use for index store
]]--

function catalogMethods:__init( ozCatalogSpec, optArgs )

	check( 'WFOzCatalog', 'WFOzCatalogSpec', '?table' )
	
	local optDocStorePath, optIndexSpecScript
	if optArgs then
		optDocStorePath = optArgs.docStorePath
		optIndexSpecScript = optArgs.indexSpecScript
	end
	
	local catalogId = ozCatalogSpec:catalogId()
	
	local docStore = WFDocStore( optDocStorePath or catalogId )
	local indexStore = WFIndexStore( docStore, optIndexSpecScript or 'WFOzCatalogIndexDefinitions.lua' )
	
	self._ozCatalogSpec = ozCatalogSpec

	self:setOwned {
		_docStore = docStore,
		_indexStore = indexStore,
	}

	WFCloseSupport.mustClose( self )

end

--------------------------------------------------------------------------------

function catalogMethods:__tostring()

	return string.format( "WFOzCatalog( %s )", tostring( self._ozCatalogSpec ) )

end

--------------------------------------------------------------------------------

function catalogMethods:ozCatalogSpec()

	return self._ozCatalogSpec
	
end

--------------------------------------------------------------------------------

function catalogMethods:ozCatalogId()

	return self._ozCatalogSpec:catalogId()
	
end

--------------------------------------------------------------------------------

function catalogMethods:ozUser()

	return self._ozCatalogSpec:wfOzUser()
	
end

--------------------------------------------------------------------------------

function catalogMethods:ozUserId()

	return self._ozCatalogSpec:wfOzUser():userId()

end

--------------------------------------------------------------------------------

function catalogMethods:docStore()

	return self._docStore

end

--------------------------------------------------------------------------------

function catalogMethods:indexStore()

	return self._indexStore

end

--------------------------------------------------------------------------------

local WFOzCatalog_class = Rx.Class 'WFOzCatalog'( catalogMethods )

return WFObjectProxy( 'WFOzCatalog', WFOzCatalog_class, nil, true )
