--[[----------------------------------------------------------------------------

WFOzCatalogTestSessionTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.reactive
local KSPromises = require 'KSPromises'

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.sync_engine.oz_utils
local WFOzCatalogSpec = require 'WFOzCatalogSpec'
local WFOzCatalogTestSession = require 'WFOzCatalogTestSession'


--============================================================================--

local WFOzCatalogTestSessionTest = {}

--------------------------------------------------------------------------------

function WFOzCatalogTestSessionTest.testCatalogTestSession_testNightly( test )

	local catalogTestSession = WFOzCatalogTestSession( 'WFOzCatalogTestSessionTest' )
	catalogTestSession:setTestName( test:testName() )

	local catalogSpec
	local expectedHrefAndId = { idLength = 32 }

	WFAsyncTestCase( test, 240 )

		:waitFor( function()

			return catalogTestSession:catalogSpecP()
				:observeP( function( _, _catalogSpec )
					catalogSpec = _catalogSpec
				end )

		end )

		:waitFor( function()

			local id = catalogSpec:catalogId()
			local href = catalogSpec:catalogHref()

			expectedHrefAndId.id = id
			expectedHrefAndId.href = "/v1.0/catalogs/" .. id

			return KSPromises.resolvedPromiseP( 'data',
				{
					idLength = #id,
					id = id,
					href = href,
				} )

		end )
		:continueWhenValueMatches( 'data', expectedHrefAndId )

		:waitFor( function()

			return catalogTestSession:catalogSpecWithThreePhotosP()
				:mapDataP( function( _, catalogSpec )

					return 'data', WFOzCatalogSpec:isClassOf( catalogSpec )

				end )

		end )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()

			return catalogTestSession:releaseTestUserP()

		end )

end

--------------------------------------------------------------------------------

local WFPlatformInfo = require "WFPlatformInfo"

if WFPlatformInfo.isWindows then
	return {}
else
	return WFOzCatalogTestSessionTest
end
