--[[----------------------------------------------------------------------------

WFOzCommentsTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.catalog
local WFOzCatalog = require 'WFOzCatalog'

	-- WF.oz_client.sync_engine
local WFOzCatalogTestSession = require 'WFOzCatalogTestSession'
local WFOzComments = require 'WFOzComments'


--------------------------------------------------------------------------------

local TEST_WAIT_TIME = 30


--==============================================================================

local function makeFunnyUniqueAlbumName()
	
	local adjectives = { "Crazy", "Disasterous", "Uncensored", "Frightening", "Lazy",
		"Freezing", "Hot", "Catastrophic", "Mysterious", }
	local events = { "Reunion", "Wedding", "Adoption", "Space Launch", "Fishing Trip",
		"Photo Safari", "Walkabout", "Ramble", "Hike", "Camping Adventure" }
	local places = { "in Transylvania", "in Abu Dahbi", "in Pukwana", "in Peru", "in Outer Mongolia",
		"in Siberia", "on the North Shore" }
	
	return string.format( "My %s %s %s",
		adjectives[ math.random( #adjectives ) ],
		events[ math.random( #events ) ],
		places[ math.random( #places ) ] )
		
end

math.randomseed( os.time() )

--============================================================================--

-- @mark WFOzCommentsTest

local WFOzCommentsTest = {}

local wfOzCatalogTestSession, wfOzCatalog, kCurrentUserId

--------------------------------------------------------------------------------

local kAllison = string.rep( 'a', 32 )
local kBarney = string.rep( 'b', 32 )
local kChloe = string.rep( 'c', 32 )

local kPhoto1 = string.rep( '1', 32 )

local kCommentId = string.rep( 'c0', 16 )
local kCommentBody = string.rep( "hello ", 12 )
local kAlbumId = string.rep( 'a1bb', 8 )

local kSpaceAlbumId = string.rep( '54acea1b', 4 )
local kSpaceId = string.rep( '5pac', 8 )

--------------------------------------------------------------------------------

local kUsers = {
	[ kAllison ] = {
		first_name = 'Alli',
		last_name = 'Ansi',
		full_name = 'Allison Ansi',
	},
	[ kBarney ] = {
		first_name = 'Barney',
		last_name = 'Barnes',
		full_name = 'Barney B Barnes',
	},
	[ kChloe ] = {
		first_name = 'Chloe',
		last_name = 'Chism',
		full_name = 'Chloe Chism-Chisholm',
	},
}

local kHackedCurrentUser =
	{
		first_name = 'Oz',
		last_name = 'Developer',
		full_name = 'Oz Developer',
	}
--------------------------------------------------------------------------------

local function makeOzUser( id )
	local src = kUsers[ id ]
	local oz = {
		_id = id,
		first_name = src.first_name,
		last_name = src.last_name,
		full_name = src.full_name,
	}
	return oz
end

--------------------------------------------------------------------------------

local function makeWFUser( id )
	local oz = kUsers[ id ]
	local wf = {
		id = id,
		firstName = oz.first_name,
		lastName = oz.last_name,
		fullName = oz.full_name,
	}
	return wf
end

--==============================================================================

function WFOzCommentsTest.setUp()
		
	wfOzCatalogTestSession = WFOzCatalogTestSession( 'WFOzCommentsTest' )

end

--------------------------------------------------------------------------------

function WFOzCommentsTest.tearDown()
	
	if wfOzCatalog then
		wfOzCatalog:close()
		wfOzCatalog = nil
	end
	
	if wfOzCatalogTestSession then
		wfOzCatalogTestSession:close()
		wfOzCatalogTestSession = nil
	end
end

--------------------------------------------------------------------------------

local function addTestCommentDocs( wfOzCatalog, numAlbumComments, numCatalogComments )
	
	local docStore = wfOzCatalog:docStore()
	
	docStore:setDocValueP(
		{
			type = 'asset',
			_id = kPhoto1,
			_slot = 'main',
			_serverProvidedMetadata = {
				created = '2013-01-03T12:23:45Z',
			},
			importSource = {
				importedBy = kAllison,
			},
			favorites = {
			},
		}
	)
	
	docStore:setDocValueP(
		{
			type = 'space_album',
			_id = kSpaceAlbumId,
			_slot = 'main',
			album = {
				_id = kAlbumId,
			},
			space = {
				_id = kSpaceId
			},
		}
	)
	
	local n = 0
	for i = 1, ( numAlbumComments or 1 ) do
		local authors = { kAllison, kBarney, kChloe, kCurrentUserId }
		local author = authors[ ( ( i - 1 ) % 4 ) + 1 ]
		docStore:setDocValueP(
			{
				type = 'comment',
				_id = kCommentId:sub( 1, 30 ) .. string.format( "%02d", n ),
				_slot = 'main',
				created = string.format('2%03d-02-02T12:23:45Z',n),
				asset = {
					_id = kPhoto1,
				},
				body = kCommentBody,
				author = makeOzUser( author ),
				space = {
					_id = kSpaceId,
				},
			}
		)
		n = n + 1
	end

	
	for i = 1, ( numCatalogComments or 1 ) do
		local authors = { kChloe, kBarney, kAllison, kCurrentUserId }
		local author = authors[ ( ( i - 1 ) % 4 ) + 1 ]
		docStore:setDocValueP(
			{
				type = 'comment',
				_id = kCommentId:sub( 1, 30 ) .. string.format( "%02d", n ),
				_slot = 'main',
				created = string.format('2%03d-02-02T12:23:45Z',n),
				asset = {
					_id = kPhoto1,
				},
				body = kCommentBody,
				author = makeOzUser( author ),
				space = {
					_id = 0,
				},
			}
		)
		n = n + 1
	end
	
	return true
end

--------------------------------------------------------------------------------

local function hackWFOzUserP( wfOzCatalog )
	-- The WFOzUser in wfOzCatalog won't be fully formed (with name, etc.) for these tests
	-- so we'll directly modify the doc store so that if WFOzComments uses it, it will
	-- find a docValue that has more than just the id in it
	local userDocStore = wfOzCatalog:ozUser():userInfoCache()
	
		
	local user = makeOzUser( kCurrentUserId )
	user._slot = 'main'
	
	return userDocStore:applyOverlayP( user )
		:mapDataP( function()
			return 'data', wfOzCatalog
		end )
	
end

--------------------------------------------------------------------------------

local function getWFOzCatalogP( )
	
	return wfOzCatalogTestSession:catalogSpecP()
		:mapDataP( function( _, wfOzCatalogSpec )
		
			kCurrentUserId = wfOzCatalogSpec:ozUserId()
			kUsers[ kCurrentUserId ] = kHackedCurrentUser
					
			wfOzCatalog = WFOzCatalog( wfOzCatalogSpec, { docStorePath = ':memory:' } )
			
			return hackWFOzUserP( wfOzCatalog )
		
		end )
	
end

--==============================================================================

function WFOzCommentsTest.test_getCommentsOnAlbumAssetB_testNightly( test )
	
	return WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( getWFOzCatalogP )
		:waitFor( function( )
			return addTestCommentDocs( wfOzCatalog, 1, 0 )
		end )
		:waitFor( function()
		
			return WFOzComments.getCommentsOnAlbumAssetB( wfOzCatalog, kAlbumId, kPhoto1 )
	
		end )
		:continueWhenValueMatches {
			{
				id = kCommentId:sub(1, 30) .. '00',
				body = kCommentBody,
				created = '2000-02-02T12:23:45Z',
				author = makeWFUser( kAllison )
			}
		}

		:waitFor( function()
			return wfOzCatalogTestSession:releaseTestUserP()
		end )

end

--------------------------------------------------------------------------------

function WFOzCommentsTest.test_addDelete_testNightly( test )
	
	local newComments = {}

	local function expectedCommentList()
		
		local t = {}
		for idx, comment in ipairs( newComments ) do
			t[ idx * 3 - 2 ] = comment.id
			t[ idx * 3 - 1 ] = comment.body
			t[ idx * 3 - 0 ] = makeWFUser( kCurrentUserId )
		end
		if t[1] then
			return unpack( t )
		else
			-- if KSReactive had ALLOW_EMPTY_TUPLES_FROM_MAP we wouldn't need
			-- to ensure we return nil here
			return nil
		end
	end
	
	local function addAnotherCommentP()
		local comment = {
			 body = string.format( "That sounds as much fun as %s", makeFunnyUniqueAlbumName() )
		}
		return WFOzComments.addCommentP( wfOzCatalog, kAlbumId, kPhoto1, comment.body )
			:mapDataP( function( _, newCommentId )
				comment.id = newCommentId
				table.insert( newComments, comment )
			end )
	end
	
	local function deleteFirstCommentP()
		local deleted = table.remove( newComments, 1 )
		return WFOzComments.deleteCommentP( wfOzCatalog, deleted.id )
	end
	
	local function getCommentsB( )
		return WFOzComments.getCommentsOnAlbumAssetB( wfOzCatalog, kAlbumId, kPhoto1 )
			:mapB( function( commentList )
				local t = {}
				for idx, comment in ipairs( commentList or {} ) do
					t[ idx * 3 - 2 ] = comment.id
					t[ idx * 3 - 1 ] = comment.body
					t[ idx * 3 - 0 ] = comment.author
				end
				return unpack( t )
			end )
	end
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( getWFOzCatalogP )
		:waitFor( function()
			return addTestCommentDocs( wfOzCatalog, 0, 0 )
		end )
		:waitFor( addAnotherCommentP )
		:waitFor( getCommentsB )
		:continueWhenValueMatchesFunction( expectedCommentList )
		:waitFor( addAnotherCommentP )
		:waitFor( getCommentsB )
		:continueWhenValueMatchesFunction( expectedCommentList )
		:waitFor( deleteFirstCommentP )
		:waitFor( getCommentsB )
		:continueWhenValueMatchesFunction( expectedCommentList )
		:waitFor( deleteFirstCommentP )
		:waitFor( getCommentsB )
		:continueWhenValueMatchesFunction( expectedCommentList )
		
		:waitFor( function()
			return wfOzCatalogTestSession:releaseTestUserP()
		end )
		
end

--------------------------------------------------------------------------------
-- @mark Get Comments
--------------------------------------------------------------------------------

function WFOzCommentsTest.test_getNumCommentsOnAssetB_testNightly( test )
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( getWFOzCatalogP )
		:waitFor( function()
			return addTestCommentDocs( wfOzCatalog, 8, 12 )
		end )
		:waitFor( function()
		
			return WFOzComments.getNumCommentsOnAssetB( wfOzCatalog, kPhoto1, { kAlbumId } )
	
		end )
		:continueWhenValueMatches( 8, 2 )
		
		:waitFor( function()
		
			return WFOzComments.getNumCommentsOnAssetB( wfOzCatalog, kPhoto1, { 'all' } )
	
		end )
		:continueWhenValueMatches( 20, 5 )
		
		:waitFor( function()
			return wfOzCatalogTestSession:releaseTestUserP()
		end )
		
end

--------------------------------------------------------------------------------

function WFOzCommentsTest.test_getCommentsOnAssetB_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( getWFOzCatalogP )
		:waitFor( function()
			return addTestCommentDocs( wfOzCatalog, 1, 2 )
		end )
		:waitFor( function()
		
			return WFOzComments.getCommentsOnAssetB( wfOzCatalog, kPhoto1, { 'all' } )
	
		end )
		:continueWhenValueMatches {
			{
				comments = {
					{
						author = makeWFUser( kChloe ),
						body = kCommentBody,
						created = "2001-02-02T12:23:45Z",
						id = kCommentId:sub(1,30) .. "01",
					},
					{
						author = makeWFUser( kBarney ),
						body = kCommentBody,
						created = "2002-02-02T12:23:45Z",
						id = kCommentId:sub(1,30) .. "02",
					},
				},
				source = "catalog",
			},
			{
				comments = {
					{
						author = makeWFUser( kAllison ),
						body = kCommentBody,
						created = "2000-02-02T12:23:45Z",
						id = kCommentId:sub(1,30) .. "00",
					},
				},
				source = "archived",
			},
		}
		
		:waitFor( function()
			return wfOzCatalogTestSession:releaseTestUserP()
		end )

end

--==============================================================================

local WFPlatformInfo = require "WFPlatformInfo"

if WFPlatformInfo.isWindows then
	return {}
else
	return WFOzCommentsTest
end
