--[[----------------------------------------------------------------------------

WFOzCurrentUser.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

DEPRECATED: This file is still in use by some clients. We are deprecating the
concept of "current user" in the framework and insisting that user ID be passed
down through code instead.

------------------------------------------------------------------------------]]

	-- WF.core.logging
local WFLogger = require 'WFLogger'

	-- WF.core.os
local KSPrefs = require 'KSPrefs'

	-- WF.core.reactive
local KSPromises = require 'KSPromises'
local KSReactive = require 'KSReactive'
local KSReactivePromiseBridge = require 'KSReactivePromiseBridge'

	-- WF.core.utils
local WFArguments = require 'WFArguments'
local WFCloseSupport = require 'WFCloseSupport'
local WFUnknown = require 'WFUnknown'

	-- WF.oz_client.account
local OzAccountService = require 'OzAccountService'

	-- WF.oz_client.catalog.old
local OzCatalogUrls = require 'OzCatalogUrls'

	-- WF.oz_client.sync_engine.user
local WFOzUser = require 'WFOzUser'
local WFOzUserLogin = require 'WFOzUserLogin'
local WFOzUserSynchronizer = require 'WFOzUserSynchronizer'


--------------------------------------------------------------------------------

local disallowCurrentUser = require 'WFCapabilities'.getCapability 'WFOzCurrentUser.disabled' == 'disabled'

local logger = WFLogger 'WFOzCurrentUser'
--	:setLevel 'debug'

local check = assert( WFArguments.check )

--------------------------------------------------------------------------------

local function mapBThatClosesItsResult( sourceB, rawMapFn )

	-- Maybe useful as a general pattern?
	
	local oldResult
	
	return sourceB:mapB( function( ... )
	
		local newResult, extra = rawMapFn( ... )

		if oldResult and oldResult ~= newResult then
			oldResult:close()
			oldResult = nil
		end

		oldResult = newResult
		
		if newResult then
			sourceB:addCloseTarget( newResult )
		end
		
		return newResult, extra
		
	end )

end

--------------------------------------------------------------------------------

local singletonSourcesByName = setmetatable( {}, { __mode = 'kv' } )

local function lazyInstantiatedRelayB( name, createFn )

	check( '+string', 'function' )
	
	local result = WFCloseSupport.fetchCached( singletonSourcesByName, name )
	
	if not result then
		
		result = createFn()
		singletonSourcesByName = result
	
	end
	
	return result:relayB()
	
end

--------------------------------------------------------------------------------

local currentUserIdB = KSPrefs.prefB 'currentUserId'

currentUserIdB:ownedOnDataB( currentUserIdB, function( id )

	logger:infof( "currentUserIdB changed to %s", id )

end )


--============================================================================--

-- @mark WFOzCurrentUser namespace

local WFOzCurrentUser = {}

--------------------------------------------------------------------------------

-- @mark basic user info

--------------------------------------------------------------------------------

function WFOzCurrentUser.currentUserId()

	-- TODO [scouten 2013-12-17]: Note that we hope to change this away from
	-- preferences at some future time, so this API is preferred over directly
	-- accessing the prefs.

	return currentUserIdB:valueNow()

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.currentUserIdB()

	-- TODO [scouten 2013-12-17]: Note that we hope to change this away from
	-- preferences at some future time, so this API is preferred over directly
	-- accessing the prefs.

	return currentUserIdB:relayB()

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.setCurrentUserId( newUserId )

	check( '?uuid' )
	
	KSPrefs.currentUserId = newUserId

end

--------------------------------------------------------------------------------

local wfOzUserB = not disallowCurrentUser and mapBThatClosesItsResult( currentUserIdB,
	function( ozUserId )
		if ozUserId then
			return WFOzUser( OzCatalogUrls.baseServerUrl, ozUserId )
		end
	end )

--------------------------------------------------------------------------------

function WFOzCurrentUser.wfOzUserB()

	return wfOzUserB:relayB():closingObjectRelayB()

end

--------------------------------------------------------------------------------

local userInfoCacheB = not disallowCurrentUser and wfOzUserB:mapB(
	function( wfOzUser )

		logger:infof( "new user info cache for ozUser = %s",
					  wfOzUser and wfOzUser:userId() or "" )

		if wfOzUser then
			return wfOzUser:userInfoCache(), wfOzUser:userId()
		else
			return nil
		end

	end )

--------------------------------------------------------------------------------

function WFOzCurrentUser.userInfoB()

	return lazyInstantiatedRelayB( 'userInfoB',

		function()
		
			return userInfoCacheB:mapSwitchB( function( userInfoCache, userId )
				
				if userInfoCache and userId then
					return userInfoCache:docValueB( userId )
						:mapB( WFUnknown.mapUnknownToNil )
				else
					return KSReactive.constantB( nil )
				end
		
			end )
		
		end )

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.userInfoP()

	local userInfoCache, userId = userInfoCacheB:valueNow()
	
	if userInfoCache and userId then
		return userInfoCache:docValueP( userId )
	else
		return KSPromises.resolvedPromiseP( 'data', {} )
	end

end

--------------------------------------------------------------------------------

local currentUserFieldValueBForPath = WFCloseSupport.memoizeFn(
	function( memberFieldPath )

		local result = userInfoCacheB:mapSwitchB( function( userInfoCache, userId )

			if userInfoCache and userId then
				return userInfoCache:fieldValueB( userId, memberFieldPath )
					:mapB( WFUnknown.mapUnknownToNil )
			else
				return KSReactive.constantB( nil )
			end
		
		end )
		
		-- Attach a log to the node. Closing the node or reaching done will clean
		-- up.

		result:ownedOnDataB( result, function( value )
			logger:infof( "%s = %s", memberFieldPath, tostring( value ) )
		end )
		
		return result
	
	end )
	
--------------------------------------------------------------------------------

function WFOzCurrentUser.currentUserFieldValueBForPath( path )

	check( '+string' )
	
	return currentUserFieldValueBForPath( path ):relayB()

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.emailB()
	
	return WFOzCurrentUser.currentUserFieldValueBForPath 'email'
	
end

--------------------------------------------------------------------------------

function WFOzCurrentUser.fullNameB()

	return WFOzCurrentUser.currentUserFieldValueBForPath 'full_name'

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.firstNameB()

	return WFOzCurrentUser.currentUserFieldValueBForPath 'first_name'

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.lastNameB()

	return WFOzCurrentUser.currentUserFieldValueBForPath 'last_name'

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.syncCollectionsByDefaultB()
	
	return WFOzCurrentUser.currentUserFieldValueBForPath( 'config/lrd_sync_collections_by_default' )
		:mapB( function( value )
			
			if WFUnknown.isUnknown( value ) then
				return true
			else
				return not not value
			end
		end )
	
end

--------------------------------------------------------------------------------

function WFOzCurrentUser.hideSyncPromptB() 
	
	return WFOzCurrentUser.currentUserFieldValueBForPath( 'config/hide_lrd_sync_switch' )
		:mapB( function( value )
			
			if WFUnknown.isUnknown( value ) then
				return false
			else
				return not not value 
			end
		end )
	
end

--------------------------------------------------------------------------------

-- @mark sign in

--------------------------------------------------------------------------------
--[[!
	\brief Updates IMS credentials for an Oz user, and makes that user the current user.

	Call when a user has successfully authenticated against the IMS server.
	Triggers a call to the Oz server to obtain further user information
	(such as user name, subscription status, and so on).

	\param baseServerUrl (string) The base URL for the Oz server to use.
	\param accessToken (string) The new access token.
	\param refreshToken (string) The new refresh token.
	\param optDeviceToken (string or nil) The device token, if applicable
	
	\return A Promise that signals completion. If the request is successful,
		the change is reflected in \c OzAccountService.currentUserB.
		If unsuccessful, \c OzAccountService.currentUserB is unchanged and the
		promise resolves as an error.
]]--

function WFOzCurrentUser.loginWithNewTokensP( baseServerUrl, accessToken, refreshToken, optDeviceToken )

	check( '+string', '+string', '+string' )

	logger:infof( "loginWithNewTokensP( accessToken = %s, refreshToken = %s, "
				  .. "optDeviceToken = %s )",
				  accessToken, refreshToken, optDeviceToken )
						
	return WFOzUserLogin.loginFromTokensP( baseServerUrl, accessToken, refreshToken, optDeviceToken )
		:waitForP( function( status, wfOzUser, userInfo )

			local userId
			
			if status == 'data' then
			
				userId = wfOzUser:userId()
				
				KSPrefs.currentUserId = userId
				
			else
				KSPrefs.currentUserId = nil
				-- bail early and let the error bubble up
				return
			end
			
			return KSReactivePromiseBridge.closingBehaviorToPromise( OzAccountService.currentUserB(),
				function( user )
					return user and user:id() == userId
				end )
		end )
	
end

--------------------------------------------------------------------------------
--[[!
	\brief Log out any current user.
]]--

function WFOzCurrentUser.logout()

	-- If we have a current user, log out now.

	local wfOzUser = wfOzUserB:valueNow()
	if wfOzUser then
		wfOzUser:logoutP()
	end
	
	-- Regardless, make sure we clear out the current user pref.

	KSPrefs.currentUserId = nil

end

--------------------------------------------------------------------------------

-- @mark user info synchronization status

--------------------------------------------------------------------------------

local shouldSyncUserInfoB = KSReactive.variableB( false )

--------------------------------------------------------------------------------

function WFOzCurrentUser.shouldSyncUserInfoB()

	return shouldSyncUserInfoB:relayB()

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.setShouldSyncUserInfo( shouldSync )

	check( 'boolean' )
	
	shouldSyncUserInfoB:set( shouldSync )

end

--------------------------------------------------------------------------------

local currentUserSyncAgent

--------------------------------------------------------------------------------

local userInfoSyncAgentB = KSReactive.callB(

	function( shouldSync, wfOzUser )
	
		logger:infof( "new user info sync agent for ozUser = %s shouldSync = %s",
					  wfOzUser and wfOzUser:userId() or "", shouldSync )

		if currentUserSyncAgent then
			currentUserSyncAgent:close()
			currentUserSyncAgent = nil
		end

		if shouldSync and wfOzUser then
		
			currentUserSyncAgent = WFOzUserSynchronizer( wfOzUser:serverBaseUrl(), wfOzUser:userId() )
			
			return currentUserSyncAgent
		
		end
	
	end,
	shouldSyncUserInfoB, wfOzUserB )

--------------------------------------------------------------------------------

local function extractBFromSyncPeerB( syncPeerMethodName )

	return userInfoSyncAgentB:mapSwitchB( function( currentUserSyncAgent )
	
		if currentUserSyncAgent then
			return currentUserSyncAgent[ syncPeerMethodName ]( currentUserSyncAgent )
		else
			return KSReactive.constantB( nil )
		end
	
	end )

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.isSyncingUserInfoB()

	return userInfoSyncAgentB:mapB( function( value )
		return not not value
	end )

end

--------------------------------------------------------------------------------

-- @mark host system status

--------------------------------------------------------------------------------

function WFOzCurrentUser.hasValidConnectionB()

	return lazyInstantiatedRelayB( 'hasValidConnectionB',
	
		function()
		
			return wfOzUserB:mapSwitchB( function( wfOzUser )
	
				if wfOzUser then
					return wfOzUser:hasValidConnectionToOzB()
						:closingMapB( WFUnknown.mapUnknownToNil )
				else
					return KSReactive.constantB( false )
				end
			
			end )
		
		end )
	
end

--------------------------------------------------------------------------------

function WFOzCurrentUser.hostIsReachableB()

	return extractBFromSyncPeerB 'hostIsReachableB'

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.currentHostErrorB()

	return extractBFromSyncPeerB 'currentHostErrorB'

end

--------------------------------------------------------------------------------

function WFOzCurrentUser.forcedUpgradeLinkB()

	return extractBFromSyncPeerB 'forcedUpgradeLinkB'

end

--------------------------------------------------------------------------------

-- @mark account/trial status

--------------------------------------------------------------------------------

function WFOzCurrentUser.accountStatusB()

	return WFOzCurrentUser.currentUserFieldValueBForPath 'entitlement/status'

end

--------------------------------------------------------------------------------

if disallowCurrentUser then

	local whitelist = {
		afterLoginGetOzUserIdP = true,
	}
	
	return setmetatable( {}, {
		__index = function( self, k )
			if whitelist[ k ] then
				return WFOzCurrentUser[ k ]
			end
			local result
			if string.find( k, "P^" ) then
				result = function()
					logger:errorf( "%s disallowed when WFOzCurrentUser.disabled\nvia %s",
								   k, ':traceback:' )
					return KSPromises.resolvedPromiseP( 'error', 'WFOzCurrentUser.disabled' )
				end
			elseif string.find( k, "B^" ) then
				result = function()
					logger:errorf( "%s disallowed when WFOzCurrentUser.disabled\nvia %s",
								   k, ':traceback:' )
					return KSReactive.constantErrorB( 'WFOzCurrentUser.disabled' )
				end
			else
				result = function()
					logger:errorf( "%s disallowed when WFOzCurrentUser.disabled\nvia %s",
								   k, ':traceback:' )
					error 'WFOzCurrentUser.disabled'
				end
			end
			rawset( self, k, result )
			return result
		end
	} )
	
end

--------------------------------------------------------------------------------

return WFOzCurrentUser
