--[[----------------------------------------------------------------------------

WFOzFavoritesTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

local WFOzCatalogTestSession = require 'WFOzCatalogTestSession'

	-- WF.oz_client.catalog
local WFOzCatalog = require 'WFOzCatalog'

	-- WF.oz_client.sync_engine.oz_utils
local WFOzFavorites = require 'WFOzFavorites'

--============================================================================--

local TEST_WAIT_TIME = 30

-- @mark WFOzFavoritesTest

local WFOzFavoritesTest = {}

local wfOzCatalogTestSession, wfOzCatalog, kCurrentUserId

--------------------------------------------------------------------------------

local kAllison = string.rep( 'a', 32 )
local kBarney = string.rep( 'b', 32 )
local kChloe = string.rep( 'c', 32 )

local kPhotoId = string.rep( '1', 32 )
local kAlbumId = string.rep( 'a1bb', 8 )
local kSpaceAlbumId = string.rep( '54acea1b', 4 )
local kSpaceId = string.rep( '5pac', 8 )
local kSpaceFavoriteId = string.rep( 'f0', 16 )

--------------------------------------------------------------------------------

local kUsers = {
	[ kAllison ] = {
		first_name = 'Alli',
		last_name = 'Ansi',
		full_name = 'Allison Ansi',
	},
	[ kBarney ] = {
		first_name = 'Barney',
		last_name = 'Barnes',
		full_name = 'Barney B Barnes',
	},
	[ kChloe ] = {
		first_name = 'Chloe',
		last_name = 'Chism',
		full_name = 'Chloe Chism-Chisholm',
	},
}

local kHackedCurrentUser =
	{
		first_name = 'Oz',
		last_name = 'Developer',
		full_name = 'Oz Developer',
	}
--------------------------------------------------------------------------------

local function makeOzUser( id )
	local src = kUsers[ id ]
	local oz = {
		_id = id,
		first_name = src.first_name,
		last_name = src.last_name,
		full_name = src.full_name,
	}
	return oz
end

--------------------------------------------------------------------------------

local function addTestFavoriteDocs( wfOzCatalog, assetFavoriteUsers, addFavsToSpace )
	
	local docStore = wfOzCatalog:docStore()
	
	docStore:setDocValueP(
		{
			type = 'asset',
			_id = kPhotoId,
			_slot = 'main',
			_serverProvidedMetadata = {
				created = '2013-01-03T12:23:45Z',
			},
			importSource = {
				importedBy = kAllison,
			},
			favorites = {
			},
		}
	)
	
	docStore:setDocValueP(
		{
			type = 'space_album',
			_id = kSpaceAlbumId,
			_slot = 'main',
			album = {
				_id = kAlbumId,
			},
			space = {
				_id = kSpaceId
			},
		}
	)
		
	for i, userId in ipairs( assetFavoriteUsers ) do
	
		if addFavsToSpace then

			docStore:setDocValueP
				{
					type = 'spacefavorite',
					_id = kSpaceFavoriteId:sub( 1, 30 ) .. string.format( "%02d", i ),
					_slot = 'main',
					created = string.format('2%03d-02-02T12:23:45Z', i ),
					author = makeOzUser( userId ),
					asset = {
						_id = kPhotoId,
					},
					space = {
						_id = kSpaceId,
					},
				}

		else
			docStore:applyOverlayP
				{
					_id = kPhotoId,
					_slot = 'main',
					favorites = {
						[ userId ] = true,
					}
				}
		end
	end
	
	return true
end

--==============================================================================

function WFOzFavoritesTest.setUp()
		
	wfOzCatalogTestSession = WFOzCatalogTestSession( 'WFOzFavoritesTest' )

end

--------------------------------------------------------------------------------

function WFOzFavoritesTest.tearDown()
	
	if wfOzCatalog then
		wfOzCatalog:close()
		wfOzCatalog = nil
	end
	
	if wfOzCatalogTestSession then
		wfOzCatalogTestSession:close()
		wfOzCatalogTestSession = nil
	end
end

--------------------------------------------------------------------------------

local function hackWFOzUserP( wfOzCatalog )
	-- The WFOzUser in wfOzCatalog won't be fully formed (with name, etc.) for these tests
	-- so we'll directly modify the doc store so that if WFOzComments uses it, it will
	-- find a docValue that has more than just the id in it
	local userDocStore = wfOzCatalog:ozUser():userInfoCache()
	
		
	local user = makeOzUser( kCurrentUserId )
	user._slot = 'main'
	
	return userDocStore:applyOverlayP( user )
		:mapDataP( function()
			return 'data', wfOzCatalog
		end )
	
end

--------------------------------------------------------------------------------

local function getWFOzCatalogP( )
	
	return wfOzCatalogTestSession:catalogSpecP()
		:mapDataP( function( _, wfOzCatalogSpec )
		
			kCurrentUserId = wfOzCatalogSpec:ozUserId()
			kUsers[ kCurrentUserId ] = kHackedCurrentUser
					
			wfOzCatalog = WFOzCatalog( wfOzCatalogSpec, { docStorePath = ':memory:' } )
			
			return hackWFOzUserP( wfOzCatalog )
		
		end )
	
end

--------------------------------------------------------------------------------

function WFOzFavoritesTest.test_countFavoritesOnAlbumAssetB_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( getWFOzCatalogP )
		:waitFor( function()
			return addTestFavoriteDocs( wfOzCatalog, { kAllison, kChloe, kCurrentUserId }, true )
		end )
		:waitFor( function()
		
			return WFOzFavorites.countFavoritesOnAlbumAssetB( wfOzCatalog, kAlbumId, kPhotoId )
		
		end )
		:continueWhenValueMatches( 3 )
		
		:waitFor( function()
			return wfOzCatalogTestSession:releaseTestUserP()
		end )
end

--==============================================================================

local WFPlatformInfo = require "WFPlatformInfo"

if WFPlatformInfo.isWindows then
	return {}
else
	return WFOzFavoritesTest
end
