--[[----------------------------------------------------------------------------

WFOzSyncManager.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

DEPRECATED [scouten 2015-04-10]: Please avoid further use of this file.
It has been deprecated in favor of WFOzCatalogSyncModel.

------------------------------------------------------------------------------]]

	-- WF.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'

	-- WF.core.logging
local WFLogger = require 'WFLogger'

	-- WF.core.os
local KSPathUtils = require 'KSPathUtils'

	-- WF.core.reactive
local KSReactive = require 'KSReactive'

	-- WF.oz_client.account
local OzAccountService = require 'OzAccountService'

	-- WF.oz_client.catalog
local OzCatalogUrls = require 'OzCatalogUrls'

	-- WF.oz_client.doc_store
local WFDocStore = require 'WFDocStore'
local WFOzDocStorePath = require 'WFOzDocStorePath'

	-- WF.oz_client.index_store
local WFIndexStore = require 'WFIndexStore'

	-- WF.oz_client.sync_engine
local WFOzAlbumAssetUploader = require 'WFOzAlbumAssetUploader'
local WFOzBinaryUploader = require 'WFOzBinaryUploader'
local WFOzCatalogSpec = require 'WFOzCatalogSpec'
local WFOzSyncPeer = require 'WFOzSyncPeer'
local WFOzSyncStatus = require 'WFOzSyncStatus'
local WFOzUser = require 'WFOzUser'
local WFSyncAgent = require 'WFSyncAgent'
local WFOzBinaryProvider = require 'WFOzBinaryProvider'

local disallowCurrentUser = require 'WFCapabilities'.getCapability 'WFOzCurrentUser.disabled' == 'disabled'

--------------------------------------------------------------------------------

local logger = WFLogger 'WFOzSyncManager'
--	:setLevel 'debug'


--============================================================================--
--[[
	\brief The \c WFOzSyncManager namespace coordinates the Oz sync activities
		for one or more catalogs.
]]--

-- @mark WFOzSyncManager namespace

local WFOzSyncManager = {}

--------------------------------------------------------------------------------

local activeSyncAgentBehaviors = {}
	-- key: (string) catalogId
	-- value: (B< nil or WFSyncAgent >) sync agent, but only if
		-- sync is currently enabled for this catalog

	-- NOTE: This is intentionally not a weak map. Once referenced, the
	-- behaviors stay present indefinitely so we can do mapB and mapSwitchB
	-- on them. We do clear out the agent itself.

--------------------------------------------------------------------------------

local function syncAgentBehaviorForCatalogId( catalogId )

	local resultB = activeSyncAgentBehaviors[ assert( catalogId ) ]
	
	if not resultB then

		logger:debugf( "Creating new syncAgentB for catalog ID %s", catalogId )
	
		resultB = KSReactive.variableB( nil )
		activeSyncAgentBehaviors[ catalogId ] = resultB
	
	end
	
	return resultB

end

--------------------------------------------------------------------------------

local function ozSyncAgentForCatalogAndUser( catalogId, ozUser, isUserAccount )

	local serverBase = OzCatalogUrls.baseServerUrl
		-- FIXME [scouten 2013-07-20]: Server base URL should be passed in,
		-- not derived here. Doing it this way for now for compatibility with
		-- old APIs.

	local wfOzUser = WFOzUser( serverBase, ozUser:id() )

	local catalogSpec = WFOzCatalogSpec( wfOzUser, catalogId )

	local ozSyncPeer = WFOzSyncPeer( catalogSpec, { suppressAlbumAssetUploads = true } )
	
	local docStore = WFDocStore( catalogId )
	
	local syncAgent = WFSyncAgent( docStore, ozSyncPeer, 'oz' )
	
	local indexStore = WFIndexStore( docStore, 'WFOzCatalogIndexDefinitions.lua' )
	
	rawset( syncAgent, '_WFIndexStore', indexStore )
	syncAgent:attachHelperObject( indexStore )
		-- FIXME [scouten 2013-07-22]: Hacky way to get back to the index store.

	syncAgent:startAutomaticSync()

	local docStorePath = WFOzDocStorePath.dirPathForCatalogId( catalogId )
	local cachePath = KSPathUtils.child( docStorePath, "previewCache" )
	local dataCache = require 'KSDataCache'( cachePath )
	
	local binaryUploader = WFOzBinaryUploader {
		catalogSpec = catalogSpec,
		docStore = docStore,
		indexStore = indexStore,
		binaryProvider = WFOzBinaryProvider( dataCache, ozUser:id(), catalogId ),
		uploadSequence = 'photosFirst',
	}
	
	syncAgent:attachHelperObject( binaryUploader )
	
	local albumAssetUploader = WFOzAlbumAssetUploader {
		catalogSpec = catalogSpec,
		docStore = docStore,
		indexStore = indexStore,
	}
	 
	syncAgent:attachHelperObject( albumAssetUploader )
	
	return syncAgent

end


--============================================================================--

-- @mark sync agent management

--------------------------------------------------------------------------------

local currentUserB
if disallowCurrentUser then
	currentUserB = KSReactive.constantErrorB( "WFOzCurrentUser.disabled" )
else
 	currentUserB = OzAccountService.currentUserB()
end

--------------------------------------------------------------------------------

local function startSyncForCatalogId( catalogId )
			
	assert( type( catalogId ) == 'string',
		"startSync: catalogId was not a string" )
	
	local syncAgentB = syncAgentBehaviorForCatalogId( catalogId )
	
	if not syncAgentB:valueNow() then

		logger:debugf( "Creating Oz sync agent for catalog %s",
					   catalogId )

		syncAgentB:set( ozSyncAgentForCatalogAndUser( catalogId, currentUserB:valueNow() ) )

	end
	
end

--------------------------------------------------------------------------------

function WFOzSyncManager.installMessageHandlers()

	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessage( 'startSync', startSyncForCatalogId )
	
	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessage( 'stopSync',
		function( catalogId )
		
			assert( type( catalogId ) == 'string',
				"stopSync: catalogId was not a string" )
			
			local syncAgentB = syncAgentBehaviorForCatalogId( catalogId )
			
			local syncAgent = syncAgentB:valueNow()
			
			if syncAgent then

				logger:debug( "Closing Oz sync agent for catalog %s", catalogId )

				syncAgentB:set( nil )
					-- Do this first, so behaviors that are mapped from
					-- syncAgentB don't also get closed.

				syncAgent:close()

			end
			
		end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'syncEnabledB', function( catalogId )

		return syncAgentBehaviorForCatalogId( catalogId ):toBooleanB()
		
	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'isUploadingToOzB', function( catalogId )
	
		return syncAgentBehaviorForCatalogId( catalogId )
			:mapSwitchB( function( syncAgent )
			
				if not syncAgent then
					return KSReactive.constantB( false )
				end
				
				local indexStore = rawget( syncAgent, '_WFIndexStore' )
				
				return WFOzSyncStatus.countAssetsToBeUploadedB( indexStore )
					:mapB( function( count )
						
						if type( count ) == 'number' then
							return count > 0
						else
							return false
						end
					
					end )
			
			end )
		
	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'isDownloadingFromOzB', function( catalogId )

		return KSReactive.constantB( false )
	
--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'isDownloadingFromOzB', false )
		
	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'hasDownloadedSuccessfullyB', function( catalogId )

		return KSReactive.constantB( true )

--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'hasDownloadedSuccessfullyB', false )

	end )
		
	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'uploadErrorB', function( catalogId )

		return KSReactive.constantB( nil )

--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'uploadErrorB', nil )

	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'downloadErrorB', function( catalogId )

		return KSReactive.constantB( nil )

--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'downloadErrorB', nil )

	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'existsOnOzB', function( catalogId )
	
		return KSReactive.constantB( true )
	
--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'existsOnOzB', true )
	
	end )
	
	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'DEBUG_statusReport', function( catalogId, photoId )
	
		local syncAgentB = syncAgentBehaviorForCatalogId( catalogId )
		
		local syncAgent = syncAgentB:valueNow()
		
		return syncAgent and syncAgent:DEBUG_statusReport( photoId )
			or "(not currently syncing)"
	
	end )
	
	----------------------------------------------------------------------------
		
	WFRequestDispatcher.onMessage( 'itmsLinkB', function( catalogId )
		
		return KSReactive.constantB( nil )
		
--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'itmsLinkB', nil )
		
	end )
	
	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessage( 'accessErrorB', function( catalogId )

		return KSReactive.constantB( false )

--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'accessErrorB', false )

	end )
	
	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessage( 'serviceAvailableB', function( catalogId )

		return KSReactive.constantB( true )

--		return syncAgentBehaviorForCatalogId( catalogId )
--			:mcallSwitchB( 'serviceAvailableB', true )

	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'syncNowP',
		function( catalogId )
	
			assert( type( catalogId ) == 'string',
				"stopSync: catalogId was not a string" )
			
			local syncAgentB = syncAgentBehaviorForCatalogId( catalogId )
			
			local syncAgent = syncAgentB:valueNow()
			
			if syncAgent then

				return syncAgent:syncUntilDoneP()

			else
			
				-- Manufacture an error promise to avoid halting here in the debugger.
				-- Due to the way we use KSPrefs to manage the current user,
				-- it is all to common to hit this error when logging out.
				local KSPromises = require "KSPromises"
				KSPromises.resolvedPromiseP( "error", "sync not enabled for this catalog" )

			end
			
	end )

end

--------------------------------------------------------------------------------

return WFOzSyncManager
