--[[----------------------------------------------------------------------------

WFOzUserModelTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------]]

	-- WF.core.lua.threading
local WFWorkerThread = require 'WFWorkerThread'

	-- WF.core.reactive
local KSPromises = require 'KSPromises'

	-- WF.core.rxlua
local WFRxPromiseBridge = require 'WFRxPromiseBridge'

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'
local WFTestModelClient = require 'WFTestModelClient'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'

	-- WF.oz_client.catalog
local OzCatalogUrls = require 'OzCatalogUrls'

	-- WF.oz_client.user
local WFOzUserTestSession = require 'WFOzUserTestSession'


--============================================================================--

-- @mark WFOzUserModelTest

local WFOzUserModelTest = {}

--------------------------------------------------------------------------------

local TEST_WAIT_TIME = 120

local userEmail
local userPassword
local userCatalogName
local coveragePath
local userTestSession

--------------------------------------------------------------------------------

function WFOzUserModelTest.setUp( inCoveragePath )

	coveragePath = inCoveragePath

end

--------------------------------------------------------------------------------

local function cleanupAfterOzTestP()

	if userTestSession then

		local oldUserTestSession = userTestSession
		userTestSession = nil

		return oldUserTestSession:releaseUser()
			:call( WFRxPromiseBridge )

	else

		return KSPromises.resolvedPromiseP( 'data', true )

	end

end

--------------------------------------------------------------------------------

local function setupOzTestP( model )

	userTestSession = WFOzUserTestSession( 'WFOzUserModelTest' )
	
	return userTestSession:userInfo()
	
		:tap( function( user, password, otherData )

			userEmail = user
			userPassword = password
			userCatalogName = otherData.oz_catalog_name
			debug.print( "Using " .. user .. " account to run tests"  )

			model:postEvent( 'loginFromUsernameAndPassword',
							  OzCatalogUrls.baseServerUrl,
							  user, password )

		end )

		:call( WFRxPromiseBridge )

		:runUntilDoneP()

end

--------------------------------------------------------------------------------

function WFOzUserModelTest.testCurrentUserModel_testNightly( test )

	local factory = WFTestModelClient.makeModelFactory( WFWorkerThread( 'WFModelsTest_Server.lua' ) )

	local model = factory:openModel( 'WFOzUserModel.makeUserModel' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( function()
			return setupOzTestP( model )
		end )

		:waitFor( function()
			if coveragePath then
				model:postEvent( 'DEBUG_dumpCoverage', coveragePath )
			end
			return 'ok'
		end )

		:waitFor( model:valueB()
			:closingMapB( function( value )

				if type( value ) ~= 'table' or WFUnknown.isUnknown( value ) then
					return false, value
				end

				if type( value.ozUserId ) ~= 'string' then
					return 'ozUserId', value
				end

				if type( value.isLoggedIn ) ~= 'boolean' then
					return 'isLoggedIn', value
				end

				if value.email ~= userEmail then
					return 'email:' .. userEmail, value
				end

				if not value.fullName then
					return 'missing fullName'
				end

				if not value.firstName then
					return 'missing firstName'
				end

				if not value.lastName then
					return 'missing lastName'
				end

				if value.userInfoCachePath ~= nil and type( value.userInfoCachePath ) ~= 'string' then
					return 'userInfoCachePath', value
				end

				if type( value.shouldSyncUserInfo ) ~= 'boolean' then
					return 'shouldSyncUserInfo', value
				end

				if type( value.isSyncingUserInfo ) ~= 'boolean' then
					return 'isSyncingUserInfo', value
				end

				if type( value.hasValidConnectionToOz ) ~= 'boolean' then
					return 'hasValidConnectionToOz', value
				end

				if value.currentHostError ~= nil and type( value.currentHostError ) ~= 'string' then
					return 'currentHostError', value
				end

				if value.forcedUpgradeLink ~= nil and type( value.forcedUpgradeLink ) ~= 'string' then
					return 'forcedUpgradeLink', value
				end

				if value.accountStatus ~= nil and type( value.accountStatus ) ~= 'string' then
					return 'accountStatus', value
				end

				if value.trialEndDate ~= nil and type( value.trialEndDate ) ~= '?string' then
					return 'trialEndDate', value
				end

				if value.daysRemainingInTrial ~= nil
					and type( value.daysRemainingInTrial ) ~= 'number'
					and type( value.daysRemainingInTrial ) ~= 'boolean'
				then
					return 'daysRemainingInTrial', value
				end

				if value.assetLimit ~= nil and type( value.assetLimit ) ~= 'number' then
					return 'assetLimit', value
				end

				if value.assetLimitForDisplay ~= nil and type( value.assetLimitForDisplay ) ~= 'number' then
					return 'assetLimitForDisplay', value
				end

				if type( value.syncCollectionsByDefault ) ~= 'boolean' then
					return 'syncCollectionsByDefault', value
				end

				return 'OK', value

			end ) )
		:continueWhenValueMatches( 'OK', ':ignore:' )

		:waitFor( function()
			if coveragePath then
				model:postEvent( 'DEBUG_dumpCoverage', coveragePath )
			end
			return 'ok'
		end )

		:waitFor( cleanupAfterOzTestP )

end

--------------------------------------------------------------------------------

function WFOzUserModelTest.testStartStopUserSyncInfo_testNightly( test )

	local factory = WFTestModelClient.makeModelFactory( WFWorkerThread( 'WFModelsTest_Server.lua' ) )

	local model = factory:openModel( 'WFOzUserModel.makeUserModel' )
		-- TODO [scouten 2015-03-24]: Is this Oz next???

	local shouldSyncUserInfoB = model:valueB():mapB(
		function( value )
			if type( value ) == 'table' then
				return value.shouldSyncUserInfo
			end
		end )

	local isSyncingUserInfoB = model:valueB():mapB(
		function( value )
			if type( value ) == 'table' then
				return value.isSyncingUserInfo
			end
		end )

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( function()
			return setupOzTestP( model )
		end )

		:waitFor( function()
			model:postEvent( 'startSync' )
			return 'ok'
		end )

		:waitFor( shouldSyncUserInfoB )
		:continueWhenValueMatches( true )

		:waitFor( isSyncingUserInfoB )
		:continueWhenValueMatches( true )

		:waitFor( function()
			model:postEvent( 'stopSync' )
			return 'ok'
		end )

		:waitFor( shouldSyncUserInfoB )
		:continueWhenValueMatches( false )

		:waitFor( isSyncingUserInfoB )
		:continueWhenValueMatches( false )

		:waitFor( function()
			if coveragePath then
				model:postEvent( 'DEBUG_dumpCoverage', coveragePath )
			end
			return 'ok'
		end )

		:waitFor( cleanupAfterOzTestP )

end

--------------------------------------------------------------------------------

return WFOzUserModelTest
